-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local util      = chunks.util
local events    = chunks.events
local load      = chunks.load
local globals   = chunks.globals

--Comment this out to enabled debug prints
local print = function() end

-- Data -------------------------------
local wave_to_start_tracking = 3

local m_game = nil

local m_wave_data = {
    allied_units_spawned = nil,
    wave_id = 0,
    ash_went_below_x_percent_health = false,
    num_enemies_who_made_it_to_the_courtyard = 0,
    num_spells_used = 0,
    was_necro_damaged = false,
    ash_used_weapons = false,
}

-- Wave Completion
local complete_x_waves = { num = 25, achievement_id = "ach_gonna_be_a_long_night" }
local complete_all_waves = "ach_he_who_is_prophesized"
local complete_x_waves_without_using_spells = { num = 5, achievement_id = "ach_brute_force" }
local complete_x_waves_without_allowing_necro_damage = { num = 5, achievement_id = "ach_protect_the_book" }
local ash_complete_wave_without_using_weapons = { list = { "chainsaw", "iron_fist", "boomstick" }, achievement_id = "ach_stay_with_the_book" }

-- Wave Completion With Certain Units
local complete_wave_without_spawning_units = { min_wave = wave_to_start_tracking, achievement_id = "ach_one_man_show" }
local complete_wave_with_certain_units_list = {
    or_these = {
        { list = { "horseman", "henry" }, achievement_id = "ach_cavalry" },
    },
    and_these = {
        { list = { "armored_guard", "archer", "spearman" }, achievement_id = "ach_tank_n_spank" },
        { list = { "peasant", "swordboy" }, achievement_id = "ach_sword_boy" },
        { list = { "archer" }, achievement_id = "ach_bend_my_bow" },
        { list = { "swordsman" }, achievement_id = "ach_you_have_my_shteel" },
    },
}

-- Killing Blows
local knock_sheila_into_pit_id = "ach_faithful_reenactment"
local ash_kills_sheila_id = "ach_honey_you_got_ugly"
local ash_kills_evil_ash_id = "ach_self_destruction"

-- Torch/Swordboy Buffs
local upgrade_x_units_with_swordboy = { num = 25, achievement_id = "ach_poor_mans_army" }
local upgrade_x_units_with_torchboy = { num = 25, achievement_id = "ach_burn_baby" }

-- Kill X Units With Y Weapon
local kill_x_units_with_boomstick = { num = 100, achievement_id = "ach_this_is_my_bookstick" }

local kill_x_units_with_weapon_list = {
    deathcoaster = { num = 250, achievement_id = "ach_plenty_for_everybody" },
    chainsaw = { num = 100, achievement_id = "ach_bone_saw" },
    iron_fist = { num = 100, achievement_id = "ach_skull_crusher" },
    boomstick = kill_x_units_with_boomstick,
    super_boomstick = kill_x_units_with_boomstick,
}

-- Total Kills
local kill_x_crossbow_skeletons_data = { num = 20, achievement_id = "ach_ground_defense" }
local kill_x_min_ashes_data = { num = 10, achievement_id = "ach_london_bridge_falling_down" }
local kill_x_pit_monsters_data = { num = 20, achievement_id = "ach_monster_basher" }
local kill_x_skeletons_data = { num = 50, achievement_id = "ach_bone_crusher" }

local total_kill_list = {
    sword_skeleton = { kill_x_skeletons_data },
    shield_skeleton = { kill_x_skeletons_data },
    spear_skeleton = { kill_x_skeletons_data },
    crossbow_skeleton = { kill_x_skeletons_data, kill_x_crossbow_skeletons_data },
    drummer_skeleton = { kill_x_skeletons_data },
    flute_skeleton = { kill_x_skeletons_data },
    horseback_skeleton = { kill_x_skeletons_data },
    mini_ash = { kill_x_min_ashes_data },
    pit_monster = { kill_x_pit_monsters_data },
}

-- Upgrades
local shop_smart_achievement_id = "ach_shop_smart"
local type_upgrade_list = {
    -- spawnable units
    peasant         = { full_upgrade_achievement_id = "ach_it_goes_to_eleven"},
    swordsman       = { full_upgrade_achievement_id = "ach_build_better_foot_soldier"},
    spearman        = { full_upgrade_achievement_id = "ach_stabby_stabby_ftw"},
    archer          = { full_upgrade_achievement_id = "ach_archers_at_the_ready"},
    arthur          = { full_upgrade_achievement_id = "ach_king_of_this_land"},
    wiseman         = { full_upgrade_achievement_id = "ach_spinach_chin"},
    armored_guard   = { full_upgrade_achievement_id = "ach_like_a_brick_house"},
    horseman        = { full_upgrade_achievement_id = "ach_equestrian_elite"},
    henry           = { full_upgrade_achievement_id = "ach_lord_of_the_northlands"},

    --castle units
    necronomicon        = { full_upgrade_achievement_id = "ach_last_defense"},
    inner_wall_archer   = { full_upgrade_achievement_id = "ach_walls_of_doom"},
    catapults           = { full_upgrade_achievement_id = "ach_rock_and_roll"},
    smithy              = { full_upgrade_achievement_id = "ach_oh_the_irony"},
    the_pit             = { full_upgrade_achievement_id = "ach_isnt_the_pit_wonderful"},

    -- spells
    vortex                  = { full_upgrade_achievement_id = "ach_wrong_book"},
    arrow_volley            = { full_upgrade_achievement_id = "ach_rain_on_my_parade"},
    magic_words             = { full_upgrade_achievement_id = "ach_every_last_syllable"},
    catapults_spell         = { full_upgrade_achievement_id = "ach_rain_of_death"},
    deathcoaster_spell      = { full_upgrade_achievement_id = "ach_hello_21st_centry"},
}

-- Gold Achievements
local save_up_x_gold_list = {
    { num = 50000, achievement_id = "ach_avaricious" },
    { num = 10000, achievement_id = "ach_hoarder" },
    { num = 5000, achievement_id = "ach_materialistic" },
}

-- Miscellaneous Achievements
local get_necktie_from_magic_words_x_times = { num = 25, achievement_id = "ach_definitely_n_word" }
local ash_healed_from_x_percent_health = { num = 0.1, achievement_id = "ach_its_a_miracle" }
local upgrade_smith_to_highest_level_id = "ach_iron_constitution"
local defend_the_castle = { x_location = 1080.0, achievement_id = "ach_defend_the_castle" }
local summon_every_possible_unit_id = "ach_whos_with_me"


-- Script init -------------------------------
local num_type_upgrades = 0

local function scriptInit()
    num_type_upgrades = 0
    for k,v in pairs(type_upgrade_list) do
        num_type_upgrades = num_type_upgrades + 1
    end
end

scriptInit()

-- Utility -------------------------------
local function resetWaveData()
    m_game = nil
    m_wave_data = {}

    m_wave_data.allied_units_spawned = {}
    m_wave_data.wave_id = 0
    m_wave_data.ash_went_below_x_percent_health = false
    m_wave_data.num_enemies_who_made_it_to_the_courtyard = 0
    m_wave_data.num_spells_used = 0
    m_wave_data.was_necro_damaged = false
    m_wave_data.ash_used_weapons = false
end

local function shouldBeTrackingAchievements()
    if (m_game ~= nil) and (m_wave_data.wave_id < wave_to_start_tracking) then
        return false
    end

    return true
end

-- NOTE: achievement completion is on a 1 to 100 scale, NOT 0 to 1
local function pushAchievementData(achievement_id, progress)
    progress = math.min(100.0, math.max(0.0, progress))
    local gc = GameCenterService.get()
    if gc then
        gc:reportCompletion(progress, achievement_id, function() end)
    end
end

local function trackAchievement(achievement_id, num_to_ompletion)
    local count = util.getNumberValue(achievement_id) + 1
    util.setNumberValue(achievement_id, count)

    local progress = math.min(100.0, (count / num_to_ompletion) * 100.0)     -- Game Center uses 0 to 100, NOT 0 to 1
    pushAchievementData(achievement_id, progress)
    print("\t\tAchievement Progress", achievement_id, count, num_to_ompletion, progress)
end

local function trackAchievementGroup(args)
    trackAchievement(args.achievement_id, args.num)
end

local function printSpawned()
    for k,v in pairs(m_wave_data.allied_units_spawned) do
        print("\t\t", k, v)
    end
end

local function incrementAlliedUnitSpawnCount(unit_name)
    local num = m_wave_data.allied_units_spawned[unit_name]
    if num == nil then
        num = 0
    end

    m_wave_data.allied_units_spawned[unit_name] = num + 1
end

local function getNumAlliedUnitsSpawned()
    local num_units = 0
    local num_unit_types = 0
    for k,v in pairs(m_wave_data.allied_units_spawned) do
        num_units = num_units + v
        num_unit_types = num_unit_types + 1
    end

    return num_units, num_unit_types
end

local function wereOnlyTheseAlliedUnitsSpawned(unit_list)
    for unit1,num in pairs(m_wave_data.allied_units_spawned) do
        local found = false
        for _,unit2 in pairs(unit_list) do
            if unit1 == unit2 then
                found = true
                break
            end
        end

        if not found then
            return false
        end
    end

    return true
end

local function updateWaveEndedData(wave_id)
    if m_wave_data.num_spells_used > 0 then
        util.setNumberValue(complete_x_waves_without_using_spells.achievement_id, wave_id) -- Last wave a spell was used
    end

    if m_wave_data.was_necro_damaged == true then
        util.setNumberValue(complete_x_waves_without_allowing_necro_damage.achievement_id, wave_id) -- Last wave the necro was damaged
    end
end

local function trackSpawnedUnits()
    local units_spawned = util.loadStringList(summon_every_possible_unit_id)

    local update_achievement = function(spawned_list)
        local progress = math.min(100.0, (#spawned_list / #globals.ally_units) * 100.0)
        pushAchievementData(summon_every_possible_unit_id, progress)

        if progress >= 100.0 then
            return true
        end

        return false
    end

    if update_achievement(units_spawned) == true then
        return -- Achievement Unlocked!
    end

    local global_spawned_units = {}

    -- add saved units to list
    for _,unit in pairs(units_spawned) do
        global_spawned_units[unit] = true
    end

    -- add units spawned this wave to list
    for unit,_ in pairs(m_wave_data.allied_units_spawned) do
        global_spawned_units[unit] = true
    end

    -- generate save data
    local save_data = {}
    for unit,_ in pairs(global_spawned_units) do
        table.insert(save_data, unit)
    end

    util.saveStringList(summon_every_possible_unit_id, save_data)
    update_achievement(save_data)
end

-- Wave Functions -------------------------------
function waveStarted(game, wave_id)
    resetWaveData()

    m_game = game
    m_wave_data.wave_id = wave_id
end

function waveEnded(wave_id, won)
    if shouldBeTrackingAchievements() == true then
        updateWaveEndedData(wave_id)

        if won == true then
            do -- Check wave progress
                local progress_to_x_waves = math.min(100.0, (wave_id / complete_x_waves.num) * 100.0)
                local progress_to_end = math.min(100.0, (wave_id / globals.final_wave) * 100.0)
                pushAchievementData(complete_x_waves.achievement_id, progress_to_x_waves)
                pushAchievementData(complete_all_waves, progress_to_end)
            end

            do -- Check for beating waves with certain types of units
                local num_units, num_unit_types = getNumAlliedUnitsSpawned()
                if num_units <= 0 then
                    if wave_id >= complete_wave_without_spawning_units.min_wave then        -- No units spawned
                        pushAchievementData(complete_wave_without_spawning_units.achievement_id, 100.0)
                    end
                else
                    -- Beating a wave using any one of these units
                    for _,achievement_data in pairs(complete_wave_with_certain_units_list.or_these) do
                        if wereOnlyTheseAlliedUnitsSpawned(achievement_data.list) == true then
                            pushAchievementData(achievement_data.achievement_id, 100.0)
                        end
                    end

                    -- Beating a wave using only and all of these units
                    for _,achievement_data in pairs(complete_wave_with_certain_units_list.and_these) do
                        if num_unit_types == #achievement_data.list then
                            if wereOnlyTheseAlliedUnitsSpawned(achievement_data.list) == true then
                                pushAchievementData(achievement_data.achievement_id, 100.0)
                            end
                        end
                    end
                end
            end

            do -- Defend the castle
                if m_wave_data.num_enemies_who_made_it_to_the_courtyard <= 0 then
                    pushAchievementData(defend_the_castle.achievement_id, 100.0)
                end
            end

            -- Complete X waves without Y happening
            do
                local check_wave_diff = function(achievement_data)
                    local last_wave_y_happened = util.getNumberValue(achievement_data.achievement_id, wave_to_start_tracking - 1)
                    local diff = math.max(0.0, wave_id - last_wave_y_happened)
                    if diff >= achievement_data.num then
                        pushAchievementData(achievement_data.achievement_id, 100.0)
                    end
                end

                check_wave_diff(complete_x_waves_without_using_spells)              -- Complete x waves without using a spell
                check_wave_diff(complete_x_waves_without_allowing_necro_damage)     -- Complete x waves without allowing damage to the necronomicon a spell
            end

            do -- See if Ash used any weapons this wave
                if m_wave_data.ash_used_weapons == false then
                    pushAchievementData(ash_complete_wave_without_using_weapons.achievement_id, 100.0)
                end
            end
        end

        trackSpawnedUnits()
    end

    resetWaveData()
end

function waveQuit(wave_id)
    print("\t\tACHIEVEMENTS: waveQuit", wave_id)

    if shouldBeTrackingAchievements() == true then
        updateWaveEndedData(wave_id)
        trackSpawnedUnits()
    end

    resetWaveData()
end


-- Spell Functions -------------------------------
function spellCast(name)
    if shouldBeTrackingAchievements() == false then
        return
    end
    print("\t\tACHIEVEMENTS: spellCast", name)

    m_wave_data.num_spells_used = m_wave_data.num_spells_used + 1
end

function magicWordsResult(result)
    if shouldBeTrackingAchievements() == false then
        return
    end
    print("\t\tACHIEVEMENTS: magicWordsResult", result)

    if result == "necktie" then
        trackAchievementGroup(get_necktie_from_magic_words_x_times)
    end
end


-- Weapon Functions -------------------------------
function weaponFired(weapon_index)
    if shouldBeTrackingAchievements() == false then
        return
    end

    local weapon = weapon_index:get()
    if weapon == nil then
        return
    end

    local weapon_holder = weapon:getWeaponHolder():get()
    if weapon_holder == nil then
        return
    end

    local weapon_name = weapon:getObjectIndex():get().label
    local weapon_holder_name = weapon_holder.label

    print("\t\tACHIEVEMENTS: weaponFired", weapon_name)

    if weapon_holder_name == "ash" then
        ashFiredWeapon(weapon_name)
    end
end


-- Unit Functions -------------------------------
function unitSpawned(name, team)
    if shouldBeTrackingAchievements() == false then
        return
    end
    print("\t\tACHIEVEMENTS: unitSpawned", name)

    -- Track allied unit spawning
    if team == GameObject.TEAM_ALLY then
        incrementAlliedUnitSpawnCount(name)
    end
end

function unitDied(name, team, x_location, weapon_name)
    print("achievements: unit died")
    
    if shouldBeTrackingAchievements() == false then
        print "NOT TRACKING ACHIEVEMENTS"
        return
    end
    print("\t\tACHIEVEMENTS: unitDied", name, x_location, weapon_name)

    do -- Total kill checks
        local achievement_data = total_kill_list[name]
        if achievement_data ~= nil then
            for k,v in pairs(achievement_data) do
                trackAchievementGroup(v)
            end
        end
    end

    do -- Kill X units with Y weapon
        if #weapon_name > 0 then
            local achievement_data = kill_x_units_with_weapon_list[weapon_name]
            if achievement_data ~= nil then
                trackAchievementGroup(achievement_data)
            end
        end
    end

    do -- Defend the castle tracking
        if team == GameObject.TEAM_ENEMY then
            if x_location <= defend_the_castle.x_location then
                m_wave_data.num_enemies_who_made_it_to_the_courtyard = m_wave_data.num_enemies_who_made_it_to_the_courtyard + 1
            end
        end
    end
end

function unitDamaged(unit, damage)
    if shouldBeTrackingAchievements() == false then
        return
    end
    print("\t\tACHIEVEMENTS: unitDamaged", damage)

    local name = unit.label
    if name == "ash" then
        ashDamaged(unit, damage)
    elseif name == "necronomicon" then
        if damage > 0 then
            m_wave_data.was_necro_damaged = true
        end
    end
end

function unitKnockedIntoPit(name)
    if shouldBeTrackingAchievements() == false then
        return
    end
    print("\t\tACHIEVEMENTS: unitKnockedIntoPit", name)

    if name == "evil_sheila" then
        pushAchievementData(knock_sheila_into_pit_id, 100.0)
    end
end

-- Type Functions -------------------------------
function typeUpgraded(category, name)
    print("\t\tACHIEVEMENTS: typeUpgraded", category, name)

    local achievement_data = type_upgrade_list[name]
    if achievement_data == nil then
        return
    end

    local full_upgrade_achievement_id = achievement_data.full_upgrade_achievement_id
    if #full_upgrade_achievement_id == 0 then
        return
    end

    if load.hasNextLevel(category, name) == true then
        return
    end

    pushAchievementData(full_upgrade_achievement_id, 100.0)

    if num_type_upgrades > 0 then -- shop smart achievement checks
        trackAchievement(shop_smart_achievement_id, num_type_upgrades)
    end
end


-- Ash Functions -------------------------------
function ashDamaged(ash, damage)
    if shouldBeTrackingAchievements() == false then
        return
    end
    print("\t\tACHIEVEMENTS: ashDamaged", damage)

    local get_ash_health_percent = function()
        local health = PartHealthIndex(ash:getObjectIndex()):get()
        if health == nil then
            return 0.0
        end

        local percent = health:getHealthRemaining() / health.health

        return percent
    end

    if damage > 0 then
        -- Damaged
        if m_wave_data.ash_went_below_x_percent_health == false then
            local percent = get_ash_health_percent()
            if percent <= ash_healed_from_x_percent_health.num then
                m_wave_data.ash_went_below_x_percent_health = true
            end
        end
    else
        -- Healing
        if m_wave_data.ash_went_below_x_percent_health == true then
            local percent = get_ash_health_percent()
            if percent >= 1.0 then
                pushAchievementData(ash_healed_from_x_percent_health.achievement_id, 100.0)
            end
        end
    end
end

function ashKilledUnit(name)
    if shouldBeTrackingAchievements() == false then
        return
    end
    print("\t\tACHIEVEMENTS: ashKilledUnit", name)

    if name == "evil_sheila" then
        pushAchievementData(ash_kills_sheila_id, 100.0)
    elseif name == "evil_ash" then
        pushAchievementData(ash_kills_evil_ash_id, 100.0)
    end
end

function ashFiredWeapon(name)
    if shouldBeTrackingAchievements() == false then
        return
    end
    print("\t\tACHIEVEMENTS: ashFiredWeapon", name)

    do -- Track any invalid weapons weapons Ash may be using
        for _,weapon in pairs(ash_complete_wave_without_using_weapons.list) do
            if weapon == name then
                m_wave_data.ash_used_weapons = true
                break
            end
        end
    end

end


-- Economy Functions -------------------------------
function goldChanged()
    local current_gold = UserData.instance():getGold()
    print("\t\tACHIEVEMENTS: goldChanged", current_gold)

    for _,achievement_data in pairs(save_up_x_gold_list) do
        local progress = math.min(100.0, (current_gold / achievement_data.num) * 100.0)
        pushAchievementData(achievement_data.achievement_id, progress)
        print("\t\tAchievement Progress", achievement_data.achievement_id, current_gold, achievement_data.num, progress)
    end
end

function ironCapUpgraded(current_level, max_level)
    if shouldBeTrackingAchievements() == false then
        return
    end
    print("\t\tACHIEVEMENTS: ironCapUpgraded", current_level, max_level)

    if current_level >= max_level then
        pushAchievementData(upgrade_smith_to_highest_level_id, 100.0)
    end
end



-- Misc Functions -------------------------------
function swordBoyUpgradedUnit(name)
    if shouldBeTrackingAchievements() == false then
        return
    end
    print("\t\tACHIEVEMENTS: swordBoyUpgradedUnit", name)

    trackAchievementGroup(upgrade_x_units_with_swordboy)
end

function torchBoyUpgradedUnit(name)
    if shouldBeTrackingAchievements() == false then
        return
    end
    print("\t\tACHIEVEMENTS: torchBoyUpgradedUnit", name)

    trackAchievementGroup(upgrade_x_units_with_torchboy)
end