module(...,package.seeall)

local m_data_queue = {}

local function buildAndSend(name, parameters)
	local e = AnalyticsEvent(name)
	
	if parameters ~= nil then
		for key,value in pairs(parameters) do
		  	e:add(key, tostring(value)) 
		end
	end
	e:send()
end

local function queueData(name, parameters)
	table.insert(m_data_queue, { name = name, parameters = parameters })
end

function flush()	
	for i,data in ipairs(m_data_queue) do
		buildAndSend(data.name, data.parameters)
	end
	
	m_data_queue = {}
end

function sendLeaderboardUIOpened()
	buildAndSend("LEADERBOARD_UI_OPENED")
end

function sendAchievementsUIOpened()
	buildAndSend("LEADERBOARD_UI_OPENED")
end

function sendGameDataReset()
	buildAndSend("GAME_DATA_RESET", 
		{
			WAVE = UserData.instance():getCurrentWave(),
			GOLD = UserData.instance():getGold()
		}
	)
end

function sendWaveEnd(wave_id, data)
	queueData(string.format("WAVE_%02d", wave_id), data)
end

function sendIncentMenuDecision(did_watch, wave_id)
	buildAndSend("REWARDED_VID_COINS",
		{
			WAVE = UserData.instance():getCurrentWave(),
			WATCHED = did_watch
		}
	)
end

function sendUnitSpawn(unit_type)
   -- commenting out because this causes multi-second delays on the loading screen between levels
	--[[queueData("UNIT_SPAWN", 
		{
			UNIT = unit_type .. "," .. UserData.instance():getTypeLevel(unit_type),
			WAVE = UserData.instance():getCurrentWave(),			
		}
	)
   --]]
end

function sendSpellCast(spell_type)
   -- commenting out because this causes multi-second delays on the loading screen between levels
	--[[queueData("SPELL_CALLED",
		{
			SPELL = spell_type .. "," .. UserData.instance():getTypeLevel(spell_type),
			WAVE = UserData.instance():getCurrentWave(),			
		}
	)
   --]]
end