module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool

function cast(caster, type_name, location) 
	local game = caster:get():getMainGame()
	local arrow_separation_x = 7.5
	local location_offset = v2f(-300, 400)
	local level = UserData.instance():getTypeLevel(type_name)
	local ammo_name = chunks.load.getStat("spells", type_name, "ammo", level)
	local range = chunks.load.getStat("spells", type_name, "range", level)
	local start_location = location --- v3f(range, 0.0, 0.0)
	
	local num_arrows = math.floor((range * 2) / arrow_separation_x)
	local arrow_vertical_separation = 100
	local damage_y = arrow_vertical_separation * num_arrows * 0.25
	
	-- Fire real, damage dealing, hidden arrows
	local victims = game:getObjectsInRange(GameObject.TEAM_ENEMY, location.x, location.x + (range * 2)).values
	for i,enemy_index in ipairs(victims) do
		local arrow = game:createGameObject("ammo", ammo_name, level)		
		local ammo_part = PartAmmoIndex(arrow):get()
		if ammo_part == nil then
			arrow:get():destroy()
			return false
		end
		
		local movement_part = PartMovementIndex(arrow):get()
		if movement_part == nil then
			arrow:get():destroy()
			return false
		end
		
		ammo_part.trajectory = PartAmmo.STRAIGHT
		movement_part.rotation_speed = 120
		
		local enemy = enemy_index:get()	
		arrow:get().location = enemy.location + location_offset + pool.get_v3f(0.0, damage_y, 0.0)
		ammo_part:setTarget(enemy_index)
		ammo_part:setDirectionNow()
		ammo_part:fire()
		arrow:get().hidden_recursive = true
	end
	
	local arrow_y_locs = {}
	for i=1, num_arrows do
		arrow_y_locs[i] = i * arrow_vertical_separation
	end
	
	-- Fire visually appealing arrows
	for i=0, num_arrows-1 do
		local arrow = game:createGameObject("ammo", ammo_name, level)		
		local ammo_part = PartAmmoIndex(arrow):get()
		if ammo_part == nil then
			arrow:get():destroy()
			return false
		end
		
		local movement_part = PartMovementIndex(arrow):get()
		if movement_part == nil then
			arrow:get():destroy()
			return false
		end
		
		ammo_part.trajectory = PartAmmo.STRAIGHT
		movement_part.rotation_speed = 120
		
		local render_part = PartRenderIndex(arrow):get()
		if render_part ~= nil and render_part.render_actor ~= nil then
			local sprite = SpriteActor.fast_cast(render_part.render_actor)
			if sprite ~= nil then
				sprite.location = sprite.location - pool.get_v3f(sprite.size.x * 0.5, 0.0, 0.0) -- put the center point for the arrows at the tip so they look better when they hit 
			end
		end
		
		local random_index = math.random(1, #arrow_y_locs)
		
		local target_location = start_location + v3f(i * arrow_separation_x, 0.0, 0.0)
		target_location.y = Map.get():getYGivenX(GameObject.TEAM_NEUTRAL, target_location.x) + math.random(-20, 20) + 20.0
		target_location.z = 399.0
		arrow:get().location = target_location + location_offset + pool.get_v3f(0.0, arrow_y_locs[random_index], 0.0)
		ammo_part:setTarget(target_location)
		ammo_part:setDirectionNow()
		ammo_part:fire()	
		
		table.remove(arrow_y_locs, random_index)
	end
	
	return true
end