module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local util      = chunks.util
local load      = chunks.load
local globals   = chunks.globals
local achievements = chunks.achievements
local assets    = chunks.assets

local killedTargetSounds = {
	assets.sounds.bye_bye,
	assets.sounds.come_get_some,
	assets.sounds.come_to_papa,
	assets.sounds.groovy,
	assets.sounds.howd_you_like_the_taste_of_that,
	assets.sounds.shoelace,
	assets.sounds.that_will_teach_you,
	assets.sounds.thats_gotta_hurt,
	assets.sounds.whats_that_you_got_on_your_face,
	assets.sounds.who_wants_some,
	assets.sounds.whos_next,
	}

local time_between_ash_sounds = 3.0
local time_between_grunt_sounds = 0.75

LuaAshUnit = chunks.class.inherit(chunks.baseunit.LuaBaseUnit)
{
	m_necktie = nil,
	m_smithy_locaiton = nil,
	m_original_texture = nil,
	m_damage_texture = nil,
	m_kill_sound_chance = 1.0,
	m_next_kill_sound_time = 0.0,
	m_next_grunt_sound_time = 0.0,
}

bindToObject = chunks.class.gameObjectBinder(LuaAshUnit)

function LuaAshUnit:onBindToObject(object)
	LuaAshUnit.super.onBindToObject(self, object)
	local actual_object = self.owner

	do -- Set up necktie
		local render_part = PartRenderIndex(object):get()
		local granny_actor = render_part:getGrannyActor()
		local necktie = granny_actor:spawn(SpriteActor())
		assets.setTexture(necktie, assets.textures.necktie)
		granny_actor:attach(necktie, "bone_tie")
		necktie.location = pool.get_v3f(0.0, -necktie.size.y * 0.5, 2.0)
		necktie.hidden_recursive = true
		self.m_necktie = necktie
	end

	self.m_original_texture = load.getStat("units", actual_object.label, "texture", actual_object:getLevel())
	if self.m_original_texture then
		self.m_damage_texture = "?://" .. self.m_original_texture .. "_damage"
		self.m_original_texture = "?://" .. self.m_original_texture
	end

	local kill_sound_chance_early = 1.0
	local kill_sound_chance_late = 0.25
	local num_levels = 50
	local current_wave = UserData.instance():getCurrentWave()
	local slope = (kill_sound_chance_late - kill_sound_chance_early) / (num_levels - 1)
	local y_intercept = kill_sound_chance_early - slope
	self.m_kill_sound_chance = slope * current_wave + y_intercept
	--print(num_levels, current_wave, slope, y_intercept, self.m_kill_sound_chance)
end

-- Callbacks from C++
function LuaAshUnit:processPartLogic(part, dt)

	self:checkForCoins()

	local object = part:get():getObjectIndex()
	local orders = PartOrdersIndex(object)
	if orders:get():hasOrders() then
		part:get():requestProcess(0.1)
		return
	end

	local attack_object = object:get():getMainGame():getClosestObjectToAttack(GameObject.TEAM_ENEMY, object:get().location.x)
	if attack_object:get() == nil then
		part:get():requestProcess(0.1)
		return
	end

	orders:get():addOrder(AshAttackOrder(attack_object), PartOrders.QUEUE_END)

	part:get():requestProcess(0.1)
end

function LuaAshUnit:onTakeDamage(damage_packet, died)
	LuaAshUnit.super.onTakeDamage(self, damage_packet, died)

	self.owner:getMainGame().lua.m_game_window.lua:updateHealthBars()

	if damage_packet.damage_info.damage <= 0 then
		return
	end
	
	self:playGruntSound()

	local object = self.owner:getObjectIndex():get()
	local render = PartRenderIndex(object:getObjectIndex()):get()


	if render == nil or render.render_actor == nil then
		return
	end

	local render_actor = render.render_actor
	local health = PartHealthIndex(object:getObjectIndex()):get()

	do -- damage effects
		util.createBloodEffect(object)

		if self.m_damage_texture then
			--Flash Ash red
			render_actor:clearActions()
			render_actor:addAction(LuaAction(0.0, function (inner)
				inner.texture = self.m_damage_texture
			end))
			render_actor:addAction(LuaAction(0.1, function (inner)
				inner.texture = self.m_original_texture
			end))
			render_actor:addAction(LuaAction(0.1, function (inner)
				inner.texture = self.m_damage_texture
			end))
			render_actor:addAction(LuaAction(0.1, function (inner)
				inner.texture = self.m_original_texture
			end))
		end

		if not died then
			local percent = health:getHealthRemaining() / health.health
			if percent < 0.50 then
				object:getMainGame().lua.m_game_window.lua:showBloodOverlay()
			end
		end
	end
end

function LuaAshUnit:checkForCoins()
	local owner = self.owner
	local x = owner.location.x
	local game = owner:getMainGame()
	local range = 37.5
	local game_lua = game.lua

	if self.m_smithy_location == nil then
		local button_location = v3f(0.0, 0.0, 0.0)
		if game_lua.m_game_window.lua.m_smithy_button ~= nil then
			button_location = game_lua.m_game_window.lua.m_smithy_button:getWorldLocation()
		end
		self.m_smithy_location = button_location + pool.get_v3f(13.5, -13.5, 10)
	end

	local coins = game:getObjectsInRange(GameObject.TEAM_COIN, x-range, x+range).values
	for i,coin in ipairs(coins) do
		local coin_object = coin:get()
		if coin_object then
			coin_object.team = GameObject.TEAM_NEUTRAL

			if coin_object.label == "iron_bar" then
				local window_location = coin_object:getWorldLocation() - game_lua.m_scene_camera:getWorldLocation()
				coin_object:setOwner(game_lua.m_game_window)
				coin_object.location = window_location

				local split = SplitAction()
				--split:addAction(AlphaAction(0.25, InterpAction.Lerp, 1.0))
				--split:addAction(MoveOffsetAction(1.0, InterpAction.Lerp, pool.get_v3f(-100,0,0)))
				--split:addAction(MoveOffsetAction(1.0, InterpAction.Ramp, pool.get_v3f(0,-225,0)))
				coin_object.location = pool.get_v3f(coin_object.location.x, coin_object.location.y, self.m_smithy_location.z + 1.0)
				split:addAction(MoveAction(0.7, InterpAction.Smooth, self.m_smithy_location))
				coin_object:clearActions()
				coin_object:addAction(split)
				coin_object:addAction(LuaAction(0, function()
					game_lua:incrementIron(2)
				end ))
				coin_object:addAction(DestroyAction(0))

				util.playSoundAtCamera("?://" .. assets.sounds.pickup_iron.path)

			elseif coin_object.label == "gold_coin" then
				local length = 1.0

				coin_object:clearActions()

				coin_object:addAction(MoveOffsetAction(length*0.5, InterpAction.Fall, pool.get_v3f(0, 100, 0)))

				local split = SplitAction()
				split:addAction(AlphaAction(length*0.5, InterpAction.Lerp, 0))

				local sequence = SequenceAction()
				sequence:addAction(ScaleAction(length*0.25, InterpAction.Smooth, pool.get_v3f(2.0, 2.0, 1)))
				sequence:addAction(ScaleAction(length*0.25, InterpAction.Smooth, pool.get_v3f(1.0, 1.0, 1)))
				split:addAction(sequence)

				coin_object:addAction(split)

				local value = coin_object:getLevel()
				coin_object:addAction(LuaAction(0, function()
					game_lua:addGold(value)
				end ))

				coin_object:addAction(DestroyAction(0))

				util.playSoundAtCamera("?://" .. assets.sounds.pickup_gold.path)
			end
		end
	end
end

function LuaAshUnit:onKilledTarget(target)
	local game = self.owner:getMainGame()
	local num_enemies = game:getNumObjectsOnTeam(GameObject.TEAM_ENEMY)
	local game_time = game:getGameTime()
	--print("num enemies", num_enemies)

	local sound = killedTargetSounds[math.random(1, #killedTargetSounds)]
	if sound ~= nil and num_enemies ~= 0 and math.random() <= self.m_kill_sound_chance and game_time >= self.m_next_kill_sound_time then
		util.playSoundAtCamera("?://" .. sound.path, {priority=10.0})
		--print("Ash Just Killed A Guy", sound.path)
		self.m_next_kill_sound_time = game_time + time_between_ash_sounds
	end

	local target_object = target:get()
	if target_object then
		local bonus = load.getStat( "units", target_object.label, "ash_kill_bonus", target_object:getLevel() ) or 0
		local bonus = math.floor( bonus * 0.9 + 0.5 )
		self.owner:getMainGame().lua:addGold(bonus)
		--print("ASH KILL AWARDED ", bonus)

		achievements.ashKilledUnit(target_object.label)
	end
end

function LuaAshUnit:showNecktie(show_time)
	if self.m_necktie == nil then
		return
	end

	self.m_necktie:clearActions()
	self.m_necktie.hidden_recursive = false

	if show_time > 0.0 then
		self.m_necktie:addAction(LuaAction(show_time, function (inner)
			inner.hidden_recursive = true
		end))
	end
end

function LuaAshUnit:addHealthBar()
	-- Overiding the BaseUnit addHealthBar function because ash doesn't need one
end

function LuaAshUnit:onDeath(object)
	LuaAshUnit.super.onDeath(self, object)

	local render = PartRenderIndex(object:getObjectIndex()):get()
	if render ~= nil and render.render_actor ~= nil then
		render.render_actor.texture = self.m_original_texture
	end
end

function LuaAshUnit:playGruntSound()
	local game = self.owner:getMainGame()
	local game_time = game:getGameTime()
	if game_time >= self.m_next_grunt_sound_time then
		self.m_next_grunt_sound_time = game_time + time_between_grunt_sounds
		util.playRandomSoundFromTable(assets.sounds.ash_pain_sounds, {priority=1.0})
	end

end