module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local util      = chunks.util
local load      = chunks.load

LuaBaseUnit = chunks.class.new
{
	m_health_bar = nil,
	m_health_bar_container = nil,
	m_enflamed_effect = nil,
}

function LuaBaseUnit:onBindToObject(object)
	local actual_object = object:get()
	actual_object:addPart(PartManager.PART_LOGIC)

	self:addHealthBar()
end

function LuaBaseUnit:addHealthBar()
	if self.m_health_bar_container ~= nil then
		return
	end

	local object = self.owner:getObjectIndex()
	local part_health = PartHealthIndex(object):get()
	if part_health == nil then
		return
	end

	-- Create Health Bar
	local part_render = PartRenderIndex(object):get()
	if part_render == nil then
		return nil
	end

	local render_actor = part_render.render_actor
	if self.owner.team == GameObject.TEAM_ENEMY then
		self.m_health_bar, self.m_health_bar_container = util.createEnemyHealthBar(render_actor)
	elseif self.owner.team == GameObject.TEAM_ALLY then
		self.m_health_bar, self.m_health_bar_container = util.createAllyHealthBar(render_actor)
	else
		return
	end

	local height = chunks.load.getHeight("units", self.owner.label)
	self.m_health_bar_container.location = self.m_health_bar_container.location + pool.get_v3f(0.0, height + 10.0, 0.0)
	self.m_health_bar_container.hidden_recursive = true
end

function LuaBaseUnit:removeHealthBar()
	if self.m_health_bar_container == nil then
		return
	end

	self.m_health_bar_container:destroy()

	self.m_health_bar_container = nil
	self.m_health_bar = nil
end

function LuaBaseUnit:checkToAddEnflamedEffect(damage_packet)
	local name = self.owner.label

	local flammable = chunks.load.getStat("units", name, "flammable")
	if flammable == false then
		return
	end


	if damage_packet.damage_info.flame_damage_per_second <= 0 then
		return
	end

	if self.m_enflamed_effect ~= nil then
		return
	end

	local render = PartRenderIndex(self.owner:getObjectIndex()):get()
	if render == nil then
		return nil
	end

	local location = render.render_actor:getWorldLocation()
	self.m_enflamed_effect = render.render_actor:spawn(SpriteActor())
	chunks.assets.setTexture(self.m_enflamed_effect, chunks.assets.textures.enflamed_effect)

	self.m_enflamed_effect.location = pool.get_v3f(0.0, (self.m_enflamed_effect.size.y * 0.5) + 10.0, -1.0)
--	self.m_enflamed_effect.location = pool.get_v3f(0.0, (self.m_enflamed_effect.size.y * 0.5) + 10.0, 10.0)

--	self.m_enflamed_effect.color = pool.get_colorf(1, 1, 1, 0.6)
	self.m_enflamed_effect.scale = pool.get_v3f(0, 0, 1)

	local sa = SplitAction()
	sa:addAction(ScaleAction(0.5, InterpAction.Ramp, pool.get_v3f(1.25, 1.75, 1)))
	sa:addAction(MoveOffsetAction(0.25, InterpAction.Ramp, pool.get_v3f(0, 20, 0)))
	self.m_enflamed_effect:addAction(sa)

	sa = SplitAction()
	sa:addAction(ScaleAction(0.5, InterpAction.Fall, pool.get_v3f(1, 1, 1)))
	sa:addAction(MoveOffsetAction(0.5, InterpAction.Fall, pool.get_v3f(0, -20, 0)))
	self.m_enflamed_effect:addAction(sa)
end

-- Callbacks from C++
function LuaBaseUnit:onTakeDamage(damage_packet, died)
	local object_index = self.owner:getObjectIndex()
	local object = object_index:get()

	-- Damage bubble text and health bars
	util.showAndScaleHealthBarForObject(object)

	self:checkToAddEnflamedEffect(damage_packet)

	achievements.unitDamaged(object, damage_packet.damage_info.damage)
end

function LuaBaseUnit:onDeath(object)
	self:removeHealthBar()

	if self.m_enflamed_effect ~= nil then
		self.m_enflamed_effect:destroy()
		self.m_enflamed_effect = nil
	end
end
