module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local util      = chunks.util
local achievements = chunks.achievements

LuaBoomstickWeapon = chunks.class.new
{
}

bindToObject = chunks.class.gameObjectBinder(LuaBoomstickWeapon)

-- Callbacks from C++
function LuaBoomstickWeapon:onWeaponFired(weapon_part, target)	
	achievements.weaponFired(weapon_part)

	local game = self.owner:getGame()
	local scene = game:getScene()
	local location = self.owner:getWorldLocation()
	location.z = 400	
	location.y = location.y + 1
	
	local muzzle_flash = scene:spawn(SpriteActor())
	assets.setTexture(muzzle_flash, assets.textures.muzzle_flash)
	muzzle_flash.location = location + pool.get_v3f(0.0, 0.0, 1.0)
	muzzle_flash.scale = pool.get_v3f(0.0, 0.0, 1.0)
	
	do  -- Muzzle flash sequence
		local fade_time = 0.15
		local sequence = SequenceAction()
		local scale_in_split = SplitAction()
		
		scale_in_split:addAction(ScaleOffsetAction(fade_time, InterpAction.Lerp, pool.get_v3f(0.0, 1.0, 0.0)))
		scale_in_split:addAction(ScaleOffsetAction(fade_time, InterpAction.Lerp, pool.get_v3f(1.0, 0.0, 0.0)))
		scale_in_split:addAction(MoveOffsetAction(fade_time, InterpAction.Lerp, pool.get_v3f(muzzle_flash.size.x * 0.5, 0.0, 0.0)))
		sequence:addAction(scale_in_split)
		
		local scale_out_split = SplitAction()
		scale_out_split:addAction(ScaleOffsetAction(fade_time, InterpAction.Lerp, pool.get_v3f(0.0, -1.0, 0.0)))
		scale_out_split:addAction(ScaleOffsetAction(fade_time, InterpAction.Lerp, pool.get_v3f(-1.0, 0.0, 0.0)))
		scale_out_split:addAction(MoveOffsetAction(fade_time, InterpAction.Lerp, pool.get_v3f(muzzle_flash.size.x * -0.5, 0.0, 0.0)))	
		scale_out_split:addAction(AlphaAction(fade_time, InterpAction.Lerp, 0.0))
		sequence:addAction(scale_out_split)
		sequence:addAction(DestroyAction(0.0))
		
		muzzle_flash:addAction(sequence)
	end
	
	do -- Smoke puff sequence
		local fade_in_time = 0.15
		local smoke_puff = scene:spawn(SpriteActor())
		assets.setTexture(smoke_puff, assets.textures.smoke_puff)
		smoke_puff.location = location + pool.get_v3f(0.0, 0.0, 0.0)
		smoke_puff.scale = pool.get_v3f(0.0, 0.0, 1.0)
		
		local sequence = SequenceAction()
		local scale_in_split = SplitAction()
		
		scale_in_split:addAction(ScaleOffsetAction(fade_in_time, InterpAction.Lerp, pool.get_v3f(1.0, 1.0, 0.0)))
		scale_in_split:addAction(MoveOffsetAction(fade_in_time, InterpAction.Lerp, pool.get_v3f(smoke_puff.size.x * 0.5, 0.0, 0.0)))
		scale_in_split:addAction(AlphaAction(fade_in_time, InterpAction.Lerp, 1.0))
		sequence:addAction(scale_in_split)
		
		local fade_out_time = fade_in_time * 8
		local scale_out_split = SplitAction()
		--scale_out_split:addAction(ScaleOffsetAction(fade_out_time, InterpAction.Lerp, pool.get_v3f(-1.0, -1.0, 0.0)))
		scale_out_split:addAction(MoveOffsetAction(fade_out_time, InterpAction.Fall, pool.get_v3f(0.0, smoke_puff.size.y * 0.5, 0.0)))
		scale_out_split:addAction(AlphaAction(fade_out_time, InterpAction.Lerp, 0.0))
		sequence:addAction(scale_out_split)
		sequence:addAction(DestroyAction(0.0))		
		
		smoke_puff:addAction(sequence)
	end
end
