module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local load      = chunks.load
local util      = chunks.util
local pool      = chunks.pool

local counter = 0

LuaCatapultAmmo = chunks.class.new
{	
	m_hit_target_function = nil
}

bindToObject = chunks.class.gameObjectBinder(LuaCatapultAmmo)

function LuaCatapultAmmo:setHitTargetFunction(func)
	self.m_hit_target_function = func
end


function LuaCatapultAmmo:onAmmoHitTarget(ammo_part, damage_info, target)
	local owner = self.owner
	local game = owner:getMainGame()
	local scene = game:getScene()
	
	if self.m_hit_target_function then
		self.m_hit_target_function(ammo_part, damage_info)
	end
	
	local splash_range = load.getStat("ammo", owner.label, "splash_range")
	if splash_range and splash_range ~= 0 then
		local victims = owner:getMainGame():getObjectsInRange(GameObject.TEAM_ENEMY, owner.location.x-splash_range, owner.location.x+splash_range).values
		for i,v in ipairs(victims) do
			local object = v:get()
			if object then
				local can_be_knocked_back = load.getStat("units", object.label, "can_be_knocked_back")
				if can_be_knocked_back == nil then
					can_be_knocked_back = true
				end
				if can_be_knocked_back then
					local orders = PartOrdersIndex(v):get()
					if orders and orders:isBeingKnockedBack()==false then
						orders:addOrder(KnockbackOrder(50.0, 0.0), PartOrders.QUEUE_FIRST)
					end
				end
			end
		end
	end
	
	do -- Spawn Explosion Effect
		local model = "mesh/objects/ammo/effects/catapult_explosion_effect.gr2"
		local texture = "texture/objects/ammo/effects/catapult_explosion_effect"
		local animation_name = "explode"
		local explosion = util.createGrannyEffect(scene, model, texture, animation_name)
		
		local render_part = PartRenderIndex(target):get()
		if render_part == nil then
			render_part = PartRenderIndex(owner:getObjectIndex()):get()
		end
		
		if render_part ~= nil and render_part.render_actor ~= nil then
			explosion.location = render_part.render_actor:getWorldLocation()			
		else
			explosion.location = owner:getWorldLocation()
		end	
		
		explosion.location = explosion.location + pool.get_v3f(0.0, 0.0, 1.0)
	end
	
	do -- Camera Shake		
		if game.lua:isObjectOnScreen(ammo_part:get():getObjectIndex()) == true then		
			local x_offset = 2
			counter = counter + 1
			if counter % 2 == 0 then
				x_offset = -2
			end
			
			local y_offset = -4
		
			game.lua:doCameraShake(x_offset, y_offset)
		end
	end
end