module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local util      = chunks.util

function cast(caster, type_name, location)
	local game = caster:get():getMainGame()
	local rock_separation_x = 70
	local location_offset = v2f(-300.0, 400.0)
	local level = UserData.instance():getTypeLevel(type_name)
	local ammo_name = chunks.load.getStat("spells", type_name, "ammo", level)
	local range = chunks.load.getStat("spells", type_name, "range", level)
	local start_location = location --- v3f(range, 0.0, 0.0)
	
	local num_rocks = math.floor((range * 2) / rock_separation_x)
	
	local ammo_hit_target_function = function (ammo_part, damage_info)
		local victims = game:getObjectsInRange(GameObject.TEAM_ENEMY, location.x, location.x + (range * 2)).values
		for i,enemy_index in ipairs(victims) do
			local enemy_health = PartHealthIndex(enemy_index):get()
			if enemy_health then				
				damage_info.damage = damage_info.damage / num_rocks
				local damage_packet = DamagePacket(ammo_part:get():getObjectIndex(), damage_info)
				enemy_health:takeDamage(damage_packet)
			end
		end
	end
	
	local rock_y_locs = {}
	for i=1, num_rocks do
		rock_y_locs[i] = i * 75
	end
	
	for i=0, num_rocks-1 do
		local rock = game:createGameObject("ammo", ammo_name, level)		
		local ammo_part = PartAmmoIndex(rock):get()
		if ammo_part == nil then
			rock:get():destroy()
			return false
		end	
		
		local movement_part = PartMovementIndex(rock):get()
		if movement_part == nil then
			arrow:get():destroy()
			return false
		end
		
		ammo_part.trajectory = PartAmmo.STRAIGHT
		movement_part.rotation_speed = 120
		
		local render_part = PartRenderIndex(rock):get()
		if render_part ~= nil and render_part.render_actor ~= nil then
			local sprite = SpriteActor.fast_cast(render_part.render_actor)
			if sprite ~= nil then
				sprite.location = sprite.location - pool.get_v3f(sprite.size.x * 0.5, 0.0, 0.0) -- put the center point for the rock at the tip so they look better when they hit 
			end
		end
		
		rock:get().lua:setHitTargetFunction(ammo_hit_target_function)
		
		local random_index = math.random(1, #rock_y_locs)
		
		local target_location = start_location + v3f(i * rock_separation_x, 0.0, 0.0)
		target_location.y = Map.get():getYGivenX(GameObject.TEAM_NEUTRAL, target_location.x)
		target_location.z = 450.0
		rock:get().location = target_location + location_offset + pool.get_v3f(0.0, rock_y_locs[random_index], 0.0)
		ammo_part:setTarget(target_location)
		ammo_part:fire()	
		
		table.remove(rock_y_locs, random_index)
	end
	
	return true
end