-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local pool      = chunks.pool
local util      = chunks.util
local load      = chunks.load
local globals   = chunks.globals

local white = colorf(1, 1, 1, 1)
local active_color = colorf(1.0, 0.89, 0.28, 1.0)
local inactive_color = colorf(0.75, 0.75, 0.75, 1.0)
local current_wave = 0
local scroll_view_inset = v2f(1, 4)

local text_scale = 0.55
local stat_string = lstring("stat")
local purchases = globals.purchases

LuaCoinStore = chunks.class.new
{
	m_selection_box = nil,
	m_popup_time = 0.2,
	m_iaps = {
		{id = "gold_tier1", gold = 5000},
		{id = "gold_tier2", gold = 15000},
		{id = "gold_tier3", gold = 50000},
		{id = "gold_tier4", gold = 150000},
		{id = "gold_tier5", gold = 500000}
	},
	m_upgrade_menu = nil,
	m_can_dismiss = true
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaCoinStore )

function LuaCoinStore:created(window)
	print("LuaCoinStore:created")
	
	local game_service = GameService.get()
	
	--For the Android Back Button
	window:getGame().lua.m_dismissable = window	

	self.m_asset_group = ResourceGroup("texture")
	Resources.get():addResourceGroup(self.m_asset_group)
	
	window.size = constants.main_menu_camera.window.world_size
	window.location = window.location + pool.get_v3f(0, 0, 450)
	window.color = pool.get_colorf(0, 0, 0, 0)
	window:addAction(AlphaAction(self.m_popup_time, InterpAction.Smooth, 0.75))

	--The background window
 	local upgrade_window = util.createWindowWithTexture(window, assets.textures.coinstore_popup)
	
	upgrade_window.force_color = true
	upgrade_window.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, 0, 1)}
	upgrade_window.scale = pool.get_v3f(0, 0, 1)
	upgrade_window:addAction(ScaleAction(self.m_popup_time, InterpAction.Smooth, pool.get_v3f(1, 1, 1)))
	self.m_upgrade_window = upgrade_window

	local label = upgrade_window:spawn( Text() )
	label.anchor = {Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0, -2.5, 1)}
	label.text = lstring("coins_store")
	label.style = "Georgia Bold"
	label.options = {height = 20.0, max_width = 140.0, align = TextAlign.Center, char_spacing = 0}      
	label.color = pool.get_colorf(0.97, 0.93, 0.34, 1.0)

	--Back button
	local text = lstring("options_back")
	local format =  {"Georgia Bold", 15.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 45.0, TextAlign.Center, 0.0, 3.0, 0, false}
	local back_button = util.createButtonWithDefaultOrLocale(upgrade_window, text, format, assets.textures.blank_button_locale, 
		assets.textures.back_button)

	back_button.anchor = {Anchor.Anchor_B, Anchor.Anchor_B, pool.get_v3f(0, 9, 1)}
	back_button.on_release = function(owner)
		assets.playSound(assets.sounds.button_click_wav)
		self:dismiss()
	end
	
	local coin_spin_window = nil
	local coin_spin_background_window = nil
	local old_coins = 0
	local iap_success_callback = function()

		self.m_can_dismiss = true
		self:dismiss()

		-- tell the upgrade unit menu to upgrade the coin total text field in case the window is closing because the user just bought something
		local upgrade_unit_menu_window = engine:getUpgradeUnitMenuWindow()
		if upgrade_unit_menu_window then
			upgrade_unit_menu_window:updateText(true, old_coins)
		end

	end

	local iap_failure_callback = function()
	
		self.m_can_dismiss = true
		self:dismiss()
	end

	-- create a button for each IAP
	for key, value in ipairs(self.m_iaps) do
		local id = value.id
		local gold = value.gold
		local price = chunks.iap.getPriceString(id)
		local asset = assets.textures.coinstore_button
		local button = util.createButtonWithTexture(upgrade_window, asset)
		local gold_icon = util.createWindowWithTexture(button, assets.textures.upgrade_gold_coin)
		local gold_label = gold_icon:spawn(Text())
		local format = {"Georgia Bold", 32.0, pool.get_colorf(1.0, 0.89, 0.28, 1.0)}
		local price_label = button:spawn(Text())
		local format = {"Georgia Bold", 32.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0)}
		
		button.anchor = {Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0, -35-asset.size.y*(key-1), 1)}
		
		gold_icon.anchor = {Anchor.Anchor_L, Anchor.Anchor_L, pool.get_v3f(6, 0, 1)}
		gold_icon.size = gold_icon.size * 0.666
		
		gold_label.anchor = {Anchor.Anchor_L, Anchor.Anchor_R, pool.get_v3f(-2, 8, 1)}
		gold_label.style = format[1]
		gold_label.options = {height = format[2]}
		gold_label.color = format[3]
		
		stat_string:arg(1, self:addCommas( tostring(gold) ))
		
		gold_label.text = stat_string
		gold_label.scale = pool.get_v3f(text_scale, 1, 1)
		price_label.anchor = {Anchor.Anchor_R, Anchor.Anchor_R, pool.get_v3f(-11, 8, 1)}

		price_label.style = format[1]
		price_label.options = {height = format[2]}
		price_label.color = format[3]
		price_label.text = price
		price_label.scale = pool.get_v3f(text_scale, 1, 1)

		button.on_release = function()

			self.m_can_dismiss = false
			old_coins = UserData.instance():getGold()

			game_service:purchaseProduct(id, iap_success_callback, iap_failure_callback)

			if( coin_spin_background_window == nil ) then
				-- Setup the background to block taps
				coin_spin_background_window = upgrade_window:spawn(Window())
				coin_spin_background_window.size = constants.main_menu_camera.window.world_size
				coin_spin_background_window.color = pool.get_colorf(0, 0, 0, 0.33)
				coin_spin_background_window.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, 0, 10)}
			end
			
			if( coin_spin_window == nil ) then
				-- Start the spinning coin
				coin_spin_window = util.createWindowWithTexture( upgrade_window, assets.textures.upgrade_gold_coin )
				coin_spin_window.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, 0, 20)}
				coin_spin_window.scale = pool.get_v3f(2, 2, 1)
				coin_spin_window.rotation = pool.get_v3f( 0, 0, 90 )
				coin_spin_window.color = pool.get_colorf(1, 1, 1, 0)


				coin_spin_window:addAction(SleepAction(1))
				coin_spin_window:addAction(AlphaAction(0.25, InterpAction.Smooth, 1))

				local spin_loop = LoopAction()
				spin_loop:addAction(RotateAction(1, InterpAction.Lerp, pool.get_v3f(0, 0, 360)))
				spin_loop:addAction(RotateAction(0, InterpAction.Lerp, pool.get_v3f(0, 0, 0)))
				coin_spin_window:addAction(spin_loop)
			end

		end
	end
end

-- only used for adding commas to the strings on the IAP buttons indicating number of coins awarded
function LuaCoinStore:addCommas(input)
	local length = input:len()
	if length > 3 then
		local left = input:sub(1, length - 3)
		local right = input:sub(length - 2, length)
		left = self:addCommas(left)
		return left .. "," .. right
	end
	
	return input
end

function LuaCoinStore:dismiss(delay)
	
	-- Prevent dismissing if there is a pending transaction.
	if( not self.m_can_dismiss ) then
		return
	end

	--For the Android Back Button
	self.owner:getGame().lua.m_dismissable = nil
	
	delay = delay or 0
	
	self.m_upgrade_window:addAction(SleepAction(delay))
	self.m_upgrade_window:addAction(ScaleAction(self.m_popup_time, InterpAction.Smooth, pool.get_v3f(0, 0, 1)))
	
	self.owner:addAction(SleepAction(delay))
	self.owner:addAction(AlphaAction(self.m_popup_time, InterpAction.Smooth, 0))
	self.owner:addAction(DestroyAction(0))
end

function LuaCoinStore:destroy()
	Resources.get():removeResourceGroup(self.m_asset_group)
	Resources.get():unload(self.m_asset_group)
end


--[[
			-- Example querying product prices. There is no garuntee of success so the values may be empty
			-- after the call (note: when building for the simulator hard-coded values will likely be used).
			local products = { gold_tier5="", gold_tier4="", gold_tier3="", gold_tier2="", gold_tier1="" }
			events.on_purchase_price_request(window:getGame(), products)
			for key,value in pairs(products) do 
				print("Key=" .. key .. " Value=" .. value)
			end
			
			-- Example purchasing a product. The callback is only executed if the purchase is successful.
			events.on_purchase_coins(window:getGame(), "gold_tier5", 
				function()
					UserData.instance():setGold(UserData.instance():getGold() + 500000)
					self:updateText()
				end
			)
			]]
