module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local util      = chunks.util
local assets    = chunks.assets
local achievements = chunks.achievements
local analytics = chunks.analytics

local playRootSound = assets.playSound

LuaCooldownButton = chunks.class.new
{
	--m_test_num = 0,
	m_iron_cost = 0,
	m_cooldown_time = 0,
	m_unit_type = nil,
	m_spell_type = nil,
	m_enabled = true,
	m_callback = nil,
	m_border = nil,
	m_label = nil,
	m_unique = false,
	m_cooling_down = false,
	m_icon_asset = nil,
}

-- The spawner.
spawn = chunks.class.spawner( Button, LuaCooldownButton )

function LuaCooldownButton:created( owner )
	owner.on_enter = function(owner) self:on_enter(owner) end
	owner.on_exit = function(owner) self:on_exit(owner) end
	owner.on_release = function(owner) self:on_release(owner) end
end

--Don't use tick unless you really have to
--[[
function LuaCooldownButton:tick( owner, delta_time )
	--self.m_test_num = self.m_test_num + 1
	--print("LuaCooldownButton has been ticked " .. self.m_test_num .. " times")
end
]]

function LuaCooldownButton:destroy(owner)

end

function LuaCooldownButton:on_enter(owner)
	if self.m_enabled then
		owner.color = pool.get_colorf( 0.7, 0.7, 0.7, 1.0 )
	end
end

function LuaCooldownButton:on_exit(owner)
	if self.m_enabled then
		owner.color = pool.get_colorf( 1.0, 1.0, 1.0, 1.0 )
	end
end

function LuaCooldownButton:on_release(owner)
	if self.m_enabled then
		
		local game = MainGame.fast_cast(owner:getGame())
		if self.m_unit_type then
			local unit_iron_cost = self.m_iron_cost			
			if game.lua:spendIron(unit_iron_cost) then
				local object_index = util.createUnitOnPath(owner:getGame(), self.m_unit_type)
				local object = object_index:get()
				--local unit_level = object:getLevel()
				--self.m_cooldown_time = chunks.load.getStat("units", self.m_unit_type, "cooldown") or 0
				if object ~= nil then
					achievements.unitSpawned(self.m_unit_type, object.team)
					analytics.sendUnitSpawn(self.m_unit_type)
					self:showIronFromSmithyAnimation()
				end
			else
				--playRootSound(assets.sounds.error)
				util.playSoundAtCamera("?://"..assets.sounds.error.path)
				return
			end
			
		end
		
		local dont_cooldown = nil
		
		if self.m_callback then			
			local result
			result, dont_cooldown = self.m_callback()
			if result == false then
				--playRootSound(assets.sounds.error)
				util.playSoundAtCamera("?://"..assets.sounds.error.path)
				return
			end
		end
		
		local cooldown = not dont_cooldown
		
		if cooldown then
			if self.m_cooldown_time > 0 then
				if self.m_icon_asset then
					self:spellCooldown()
				else
					self:unitCooldown()
				end
			end
			
			if self.m_unique then
				self:makeDisabled(owner)
				owner:getGame().lua.m_buttons_unique[self.m_unit_type] = self
			end
		
		end
	
		--playRootSound(assets.sounds.button_click_wav)
		util.playSoundAtCamera("?://"..assets.sounds.button_click_wav.path)
	else
		--playRootSound(assets.sounds.error)
		util.playSoundAtCamera("?://"..assets.sounds.error.path)
	end
end

function LuaCooldownButton:unitCooldown()
	local owner = self.owner
	local window = Window()
	owner:spawn(window)
	window.size = owner.size
	window.anchor = { local_anchor=Anchor.Anchor_B, owner_anchor=Anchor.Anchor_B, offset=pool.get_v3f(0, window.size.y*0.06, 1) }
	window.texture = "builtin://white"
	window.color = pool.get_colorf(0.0, 0.0, 0.0, 0.5)
	window.scale = pool.get_v3f(0.90, 0.88, 1)
	
	self:makeDisabled(owner)
	self.m_cooling_down = true
	
	window:addAction(ScaleAction(self.m_cooldown_time, InterpAction.Lerp, pool.get_v3f(0.90, 0.0, 1)))
	window:addAction(LuaAction(0, function() self.m_cooling_down = false; self:makeReady(owner) end))
	window:addAction(DestroyAction(0))
end

function LuaCooldownButton:spellCooldown()
	local owner = self.owner
	local window = Window()
	owner:spawn(window)
	assets.setTexture(window, self.m_icon_asset)
	window.size = owner.size
	window.anchor = { local_anchor=Anchor.Anchor_B, owner_anchor=Anchor.Anchor_B, offset=pool.get_v3f(0, 0, 1) }
	--window.force_color = true
	window.color = pool.get_colorf(0.0, 0.0, 0.0, 0.6)
	window.scale = pool.get_v3f(1.0, 0.95, 1)
	
	self:makeDisabled(owner)
	self.m_cooling_down = true
	
	local split = SplitAction()
	split:addAction(ScaleAction(self.m_cooldown_time, InterpAction.Lerp, pool.get_v3f(1.0, 0.06, 1)))
	local uv = UVAction(self.m_cooldown_time, InterpAction.Lerp)
	uv:setScale(v2f(1.0, 0.95), v2f(1.0, 0.06))
	split:addAction(uv)
	window:addAction(split)
	
	
	window:addAction(LuaAction(0, function() self.m_cooling_down = false; self:makeReady(owner) end))
	window:addAction(DestroyAction(0))
	
	return window
end



function LuaCooldownButton:makeDisabled(owner, time)
	owner = owner or self.owner
	self.m_enabled = false
	--owner.color = pool.get_colorf(0.5, 0.5, 0.5, 1)
	owner:addAction(ColorAction((time or 0.2), InterpAction.Lerp, pool.get_colorf(0.5, 0.5, 0.5, 1)))
end

function LuaCooldownButton:makeReady(owner, dont_pulse)
	owner = owner or self.owner
	
	if owner:getGame().lua.m_iron >= self.m_iron_cost and not self.m_cooling_down and not owner:getGame().lua.m_buttons_unique[self.m_unit_type] then
		self.m_enabled = true
		--owner.color = pool.get_colorf(1, 1, 1, 1)
		owner:addAction(ColorAction(0.2, InterpAction.Lerp, pool.get_colorf(1, 1, 1, 1)))
		if not dont_pulse then
			owner:addAction(ScaleOffsetAction(0.1, InterpAction.Smooth, pool.get_v3f(0.1, 0.1, 0)))
			owner:addAction(ScaleOffsetAction(0.1, InterpAction.Smooth, pool.get_v3f(-0.1, -0.1, 0)))
		end
	end
end

--[[function LuaCooldownButton:makeReadyWithoutPulse(owner)
	if not self.m_cooling_down then
		owner = owner or self.owner
		self.m_enabled = true
		--owner.color = pool.get_colorf(1, 1, 1, 1)
		owner:addAction(ColorAction(0.2, InterpAction.Lerp, pool.get_colorf(1,1,1,1)))
	end
end]]

function LuaCooldownButton:setUnit(unit_type)
	local owner = self.owner
	
	self.m_unit_type = unit_type
	self.m_cooldown_time = engine.g_god_mode and 1 or chunks.load.getStat("units", unit_type, "cooldown") or 0
	self.m_iron_cost = chunks.load.getStat("units", unit_type, "iron_cost") or 49
	self.m_unique = chunks.load.getStat("units", unit_type, "unique") or false

	assets.setTexture(owner, assets.findTextureByPathName( chunks.load.getStat("units", unit_type, "icon_texture") ) or assets.textures.temp_spawn_button)

	if owner:getGame().lua.m_iron < self.m_iron_cost then
		self:makeDisabled(owner, 0)
	end
	
	
	--[[
	if not self.m_border then
		local border = owner:spawn(Window())
		assets.setTexture(border, assets.textures.icon_border)
		border.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, 0.0, 1.0)}
		border.receive_input = false
		if owner:getGame().lua.m_iron >= self.m_iron_cost then
			border.hidden = true
			border.color = pool.get_colorf(1, 1, 1, 0)
		end
		
		self.m_border = border
	end
	]]
	
	if self.m_label then
		self.m_label:destroy()
	end
	
	local label = owner:spawn(Text())
	local format = {"Helvetica-Bold", 14.0}
	label.style = format[1]
	label.options = {height = format[2]}
	local stat_string = lstring("stat")
	stat_string:arg(1, self.m_iron_cost)
	label.text = stat_string
	label.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(13,-19+4,1)}
	self.m_label = label
end

function LuaCooldownButton:setSpell(spell_type, callback)
	local owner = self.owner
	
	self.m_spell_type = spell_type
	self.m_cooldown_time = engine.g_god_mode and 1 or chunks.load.getStat("spells", spell_type, "cooldown") or 0
	self.m_iron_cost = chunks.load.getStat("spells", spell_type, "iron_cost") or 0
	
	self.m_icon_asset = assets.findTextureByPathName( chunks.load.getStat("spells", spell_type, "icon_texture") ) or assets.textures.icon_generic
	assets.setTexture(owner, self.m_icon_asset)

	self.m_callback = callback
	
	local window = self:spellCooldown()
	--window.delta_time_factor = 2.0

end

function LuaCooldownButton:darkenBorder()
	local border = self.m_border
	--print("darkening border", self.m_unit_type)
	border:clearActions()
	--border.hidden = false
	border:addAction(LuaAction(0, function() border.hidden = false end))
	border:addAction(AlphaAction(0.25, InterpAction.Lerp, 1))
end

function LuaCooldownButton:lightenBorder()
	local border = self.m_border
	--print("lightening border", self.m_unit_type)
	border:clearActions()
	border:addAction(AlphaAction(0.25, InterpAction.Lerp, 0))
	border:addAction(LuaAction(0, function() border.hidden = true end))
	--border.hidden = true
end

function LuaCooldownButton:showIronFromSmithyAnimation(num)
	num = num or 3
	if num <= 0 then
		return
	end

	local game = MainGame.fast_cast(self.owner:getGame())
	if game == nil then
		return
	end
	
	local game_window = game.lua.m_game_window
	if game_window == nil then
		return
	end
	
	local smithy_button = game_window.lua.m_smithy_button
	if smithy_button == nil then
		return
	end
	
	local start_location = smithy_button.location + pool.get_v3f(-15.0, 0.0, 1.0)
	local end_location = self.owner.location + pool.get_v3f(0.0, 0.0, 1.0)
	
	local delay_time = 0.15
	local units_per_second = 300
	local distance_vector = v2f(end_location) - v2f(start_location)
	local move_time = distance_vector:length() / units_per_second
	
	for i=1, num do
		local iron = game_window:spawn(SpriteActor())
		assets.setTexture(iron, assets.textures.iron_bar_icon)
		iron.location = start_location 
		
		local sequence = SequenceAction()
		sequence:addAction(SleepAction((i - 1) * delay_time))
		sequence:addAction(MoveAction(move_time, InterpAction.Smooth, end_location))
		sequence:addAction(DestroyAction(0.0))
		
		iron:addAction(sequence)
	end
end

