module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local assets    = chunks.assets

LuaDeathcoasterUnit = chunks.class.inherit(chunks.baseunit.LuaBaseUnit)
{
	m_endpoint = nil,
	m_victims = nil,
	m_victims_orig_size = 0,
	m_previous_victims,
	m_sound_handle = nil,
}

bindToObject = chunks.class.gameObjectBinder(LuaDeathcoasterUnit)

function LuaDeathcoasterUnit:onBindToObject(object)	
	LuaDeathcoasterUnit.super.onBindToObject(self, object)	
	
	--self.m_victims = object:get():getMainGame():getObjectsInRange(GameObject.TEAM_ENEMY, -1000000, 1000000).values
	--self.m_victims_orig_size = #self.m_victims
	
	self.m_previous_victims = {}
	
	--self.m_sound_handle = assets.playLoopingSound(assets.sounds.deathcoaster_loop)
	self.m_sound_handle = MainGame.playLooping3DSound(assets.sounds.deathcoaster_loop.path, object:get())
	--self:setSoundLocation()
	
	self.m_damage_remaining = chunks.load.getStat("spells", "deathcoaster_spell", "damage")
end

-- Callbacks from C++
function LuaDeathcoasterUnit:processPartLogic(part, dt)	
	local object_index = part:get():getObjectIndex()
	local object = object_index:get()
	local orders = PartOrdersIndex(object_index)
	
	self:setSoundLocation()
	
	--Destroy it when we get to the end
	if self.m_endpoint and math.abs(self.m_endpoint.x - object.location.x)<60 then
		object:die()
		return
	end
	
	
	local range = 150
	local victims = object:getMainGame():getObjectsInRange(GameObject.TEAM_ENEMY, object.location.x-range, object.location.x+range).values
	for i,enemy_index in ipairs(victims) do
		local enemy = enemy_index:get()
		local enemy_health = PartHealthIndex(enemy_index):get()
		if enemy and enemy_health and not self.m_previous_victims[enemy.id] then
			self.m_previous_victims[enemy.id] = true
			local unique = chunks.load.getStat("units", enemy.label, "unique")
			local health_remaining = enemy_health:getHealthRemaining()

			local damage_amount = unique and 0.20*enemy_health.health or health_remaining
			local damage_info = DamageInfo(damage_amount, DamageInfo.PHYSICAL_DAMAGE)
			local damage_packet = DamagePacket(object_index, damage_info)
			enemy_health:takeDamage(damage_packet)
		
			self.m_damage_remaining = self.m_damage_remaining - damage_amount
			--print("#################")
			--print("dealt", damage_amount, "remaining", self.m_damage_remaining, tostring(unique))
			--print("#################")
			
			if self.m_damage_remaining <= 0.0 then
				object:die()
				return
			end



		end --if enemy, enemy_health, previous_victims
		
	end --for loop
	
	
	
	
	if orders:get():hasOrders() then
		part:get():requestProcess(1/60)
		return
	end 
	
	self.m_endpoint = Map.get():getLastPoint(GameObject.TEAM_ALLY)
	--orders:get():clearOrders()
	orders:get():addOrder(MoveAlongPathOrder(self.m_endpoint), PartOrders.QUEUE_END)
	
	part:get():requestProcess(1/60)	
end

function LuaDeathcoasterUnit:onDeath(object)
	LuaDeathcoasterUnit.super.onDeath(self, object)

	if self.m_sound_handle then
		self.m_sound_handle:stop()
	end
end

function LuaDeathcoasterUnit:setSoundLocation()
	if not self.m_sound_handle then
		return
	end
	
	MainGame.setSoundLocation(self.m_sound_handle, self.owner)
end