-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local pool      = chunks.pool
local util      = chunks.util
local events    = chunks.events
local stat_string = lstring("stat")
LuaDebugOptionsMenuWindow = chunks.class.new
{
	m_current_wave_label = nil,
	m_god_mode_label = nil,
	m_castle_units_label = nil,
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaDebugOptionsMenuWindow )

function LuaDebugOptionsMenuWindow:created(window)			
	window.hidden = true
	window.size = constants.main_menu_camera.window.world_size

	-- Background
	local background = window:spawn(SpriteActor())
	assets.setTexture(background , assets.textures.castle_front_background)
	
	-- Options Label
	do
		local options_label = window:spawn(Text())
		local format = {"Debug Font", 50.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0)}
		options_label.style = format[1]
		options_label.options = {height = format[2]}
		options_label.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f(10.0, -10.0, 2.0)}
		
		stat_string:arg(1, "Debug Options")
		options_label.text = stat_string
	end
	
	-- Menu Button
	do
		local back_button = util.createButtonWithTexture(window, assets.textures.back_button)
		back_button.anchor = {Anchor.Anchor_TR, Anchor.Anchor_TR, pool.get_v3f(0.0, 0.0, 1.0)}
		
		-- Quit the game
		back_button.on_release = function(owner)
			engine:goToPreviousGame()
		end
	end
	
	-- Debug Upgrade button
	do
		local upgrade_menu_button = util.createButtonWithText(window, "Upgrade Units")
		upgrade_menu_button.location = pool.get_v3f(-140.0, 50.0, 1.0)
		upgrade_menu_button.on_release = function (button)
			engine:spawnUpgradeMenu()
		end
	end
	
	
	-- Wave changing buttons
	do
		self.m_current_wave_label = window:spawn(Text())
		local format = {"Debug Font", 30.0, pool.get_colorf(1.0, 0.0, 0.0, 1.0)}
		self.m_current_wave_label.anchor = {Anchor.Anchor_L, Anchor.Anchor_L, pool.get_v3f(30, 0.0, 2.0)}
		self.m_current_wave_label.style = format[1]
		self.m_current_wave_label.options = {height = format[2]}
		local inc_wave_button = util.createButtonWithText(self.m_current_wave_label, "   +   ")
		inc_wave_button.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TR, pool.get_v3f(5.0, 0.0, 2.0)}
		inc_wave_button.on_release = function (button)
			local current_wave = UserData.instance():getCurrentWave()
			UserData.instance():setCurrentWave(current_wave + 1)
			self:updateUI()
		end
		
		local dec_wave_button = util.createButtonWithText(self.m_current_wave_label, "   -   ")
		dec_wave_button.anchor = {Anchor.Anchor_TR, Anchor.Anchor_TL, pool.get_v3f(-5.0, 0.0, 2.0)}
		dec_wave_button.on_release = function (button)
			local current_wave = UserData.instance():getCurrentWave()
			if current_wave > 1 then
				UserData.instance():setCurrentWave(current_wave - 1)
				self:updateUI()
			end
		end
	end
	
	do
		local god_mode_button
		god_mode_button, god_mode_label = util.createButtonWithText(window, "God Mode: OFF")
		self.m_god_mode_label = god_mode_label
		god_mode_button.anchor = {Anchor.Anchor_L, Anchor.Anchor_L, pool.get_v3f(5.0, -40.0, 1.0)}
		god_mode_button.on_release = function (button)
			engine.g_god_mode = not engine.g_god_mode
			UserData.instance():setGold( UserData.instance():getGold() + 25000)
			self:updateUI()
		end
	end
	
	 --Play NOW Button
    do
        local button, label = util.createButtonWithText(window, "PLAY NOW")
        button.anchor = {Anchor.Anchor_L, Anchor.Anchor_L, pool.get_v3f(5.0, -90.0, 1.0)}
        self.m_castle_units_label = label
        button.on_release = function() engine:spawnMainGame() end
    end
	
	--Iron Fist Button
	do
		local button, label = util.createButtonWithText(window, "Iron Fist: Unlocked")
		button.anchor = {Anchor.Anchor_TR, Anchor.Anchor_TR, pool.get_v3f(-20.0, -100.0, 1.0)}
		self.m_iron_fist_label = label
		button.on_release = function() UserData.instance():setLocked("iron_fist", not UserData.instance():getLocked("iron_fist")); self:updateUI() end
	end
	
	--Chainsaw Button
	do
		local button, label = util.createButtonWithText(window, "Chainsaw: Unlocked")
		button.anchor = {Anchor.Anchor_TR, Anchor.Anchor_TR, pool.get_v3f(-20.0, -150.0, 1.0)}
		self.m_chainsaw_label = label
		button.on_release = function() UserData.instance():setLocked("chainsaw", not UserData.instance():getLocked("chainsaw")); self:updateUI() end
	end
    
    -- Crash Button
    do
		local button, label = util.createButtonWithText(window, "Crash")
		button.anchor = {Anchor.Anchor_TR, Anchor.Anchor_TR, pool.get_v3f(-20.0, -200.0, 1.0)}
		button.on_release = function() GameService.get():throwException() end
	end

     --[[ Resources Button
    do
        local button, label = util.createButtonWithText(window, "Resources")
        button.anchor = {Anchor.Anchor_TR, Anchor.Anchor_TR, pool.get_v3f(-20.0, -250.0, 1.0)}
        button.on_release = function() print(Resources.get():description()) end
    end
    ]]
    
	-- Endless Wave Button
    do
        local button, label = util.createButtonWithText(window, "Endless Wave")
        button.anchor = {Anchor.Anchor_TR, Anchor.Anchor_TR, pool.get_v3f(-20.0, -300.0, 1.0)}
        self.m_castle_units_label = label
        button.on_release = function()
			util.setEndlessModeUnlocked()
			UserData.instance():setCurrentWave(51)
			engine:spawnUpgradeUnitMenu()
		end
    end
	
	--Rest Data Button
	do
		local format = {"Debug Font", 12.0, pool.get_colorf(1.0, 0.0, 0.0, 1.0)}
		local button = util.createButtonWithText(window, "***RESET ALL DATA***", format)
		button.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f(0.0, 0.0, 1.0)}
		button.on_release = function() 
			engine:resetData()
		end
	end
	
	self:updateUI()
end

function LuaDebugOptionsMenuWindow:updateUI()
	local window = self.owner
	
	stat_string:arg(1, "Wave " .. UserData.instance():getCurrentWave())
	self.m_current_wave_label.text = stat_string
	stat_string:arg(1, "God Mode: "..(engine.g_god_mode and "ON" or "OFF"))
	self.m_god_mode_label.text = stat_string
	stat_string:arg(1, "Iron Fist: " .. (UserData.instance():getLocked("iron_fist") and "Locked" or "Unlocked"))
	self.m_iron_fist_label.text = stat_string
	stat_string:arg(1, "Chainsaw: " .. (UserData.instance():getLocked("chainsaw") and "Locked" or "Unlocked"))
	self.m_chainsaw_label.text = stat_string
	
	--self.m_castle_units_label.text = "Castle Units: " .. (engine.g_castle_units and "ON" or "OFF")
	
	
end
