module(...,package.seeall)
-- includes --
local engine	= engine.instance
local chunks	= engine.chunks
local util		= chunks.util
local assets	= chunks.assets
local pool		= chunks.pool
local tutorial	= chunks.tutorial

DefaultWaveActor = chunks.class.new
{
	owner = nil,
	game = nil,
	game_window = nil,
	action_actor = nil,
	tutorial_window_action_actor = nil,
	tutorial_window = nil,
	fade_time = 0.25,
	iron_trigger_count = chunks.globals.iron_level_cap_upgrade_price[1],
	iron_dummy_actor = nil,
}

local function adjustZOffset(a, o)
	a.location = a.location + pool.get_v3f(0, 0, o)
end

function bindToObject(actor)
	DefaultWaveActor:new(actor) -- bind the script to the object
end

function DefaultWaveActor:created( o )
	self.owner = o
	self.owner.label = "Wave actor"
	self.game = self.owner:getGame()
	self.game_window = self.game.lua.m_game_window

	local ud = UserData.instance()
	if #ud:getValue("tutorial_iron_upgrade_shown") < 1 then
		tutorial.watchForIronCountOver(self.owner, self.game, self.iron_trigger_count, function() self:showSmithyUpgradeTutorial() end)
	end
end

function DefaultWaveActor:unitSpawned(i, game_object_index, tag)
	--Spawn hero unit
	if tag then
		tag = tostring(tag)
		if tag:sub(1, 6) == "spawn_" then
			local unit_type = tag:sub(7)

			local far_left = Map.get():getFirstPoint(GameObject.TEAM_ALLY).x
			local spawn_point = math.max(self.game.lua.m_ash:get().location.x - 300, far_left - 100)

			local text_box = tutorial.createWindow( self.game_window, "top" )
			adjustZOffset(text_box, 100)
			text_box.color = pool.get_colorf(1, 1, 1, 0)
			local label = text_box:spawn(Text())
			label.anchor = { Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, 0+5, 1) }
			local format = {"Helvetica-Bold", 22.0}
			label.style = format[1]
			label.options = {height = format[2]}
			label.text = lstring("hero_appeared") --"A hero has appeared!"

			text_box:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1))
			text_box:addAction(LuaAction(0, function() util.createUnitOnPath(self.game, unit_type, 1, spawn_point) end))
			text_box:addAction(SleepAction(3))
			text_box:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
			text_box:addAction(DestroyAction(0))
		end
	end
end

function DefaultWaveActor:unitDied(game_object_index)
end

function DefaultWaveActor:gameOver(team)
end

function DefaultWaveActor:showSmithyUpgradeTutorial()
	local text_box = tutorial.createWindow( self.game_window, "top" )
	adjustZOffset(text_box, 100)

	--add text to the window
	local label = text_box:spawn(Text())
	label.anchor = { Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0, -24+5, 1) }
	local format = {"Helvetica-Bold", 22.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0)}
	label.style = format[1]
	label.options = {height = format[2]}
	label.text = lstring("upgrade_smithy") --"Upgrade your smithy"
	label = label:spawn(Text())
	label.anchor = { Anchor.Anchor_T, Anchor.Anchor_B, pool.get_v3f(0, 0-5, 0) }
	local format = {"Helvetica", 18.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0)}
	label.style = format[1]
	label.options = {height = format[2]}
	label.text = lstring("so_he_works_faster") --"so he works faster"

	local smithy = self.game_window.lua.m_smithy_button

	-- spawn bouncing arrow
	local pointer_window = smithy:spawn(Window())
	pointer_window.anchor = {Anchor.Anchor_B, Anchor.Anchor_T, pool.get_v3f(0, 0, 101)}
	pointer_window.size = assets.textures.tutorial_pointer.size
	pointer_window.hidden = true
	local pointer = tutorial.createBouncingArrow(pointer_window, "down")

	--draw a circle around the peasant icon
	local circle_button = util.createButtonWithTexture(smithy, assets.textures.tutorial_circle)
	circle_button.size = smithy.size

	--animate circle
	local circle_loop = LoopAction()
	circle_loop:addAction(ScaleAction(0.4, InterpAction.Fall, pool.get_v3f(1.25, 1.25, 1)))
	circle_loop:addAction(ScaleAction(0.4, InterpAction.Ramp, pool.get_v3f(1, 1, 1)))
	circle_button:addAction(circle_loop)

	text_box.color = pool.get_colorf(1, 1, 1, 0)
	text_box:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1))

	--Used to watch for spawn button presses
	local dummy_actor = self.owner:spawn(ActorBase())

	local function clear_tut()
		if self.has_smithy_tut then
			self.has_smithy_tut = false
			circle_button:clearActions()
			circle_button:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
			circle_button:addAction(DestroyAction(0))
			pointer_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
			pointer_window:addAction(DestroyAction(0))
			text_box:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
			text_box:addAction(DestroyAction(0))
			dummy_actor:destroy()
		end
	end

	circle_button.on_release = function()
		clear_tut()
		UserData.instance():setValue("tutorial_iron_upgrade_shown", "yes")
		smithy:on_release()
	end

	--Dummy actor which loops looking to see if iron was spent spawning units
	local loop = LoopAction()
	loop:addAction(LuaAction(chunks.globals.tutorial_interval, function()
		if self.game.lua.m_iron < self.iron_trigger_count then
			clear_tut()
			tutorial.watchForIronCountOver(self.owner, self.game, self.iron_trigger_count, function() self:showSmithyUpgradeTutorial() end)
		end
	end))
	dummy_actor:addAction(loop)

	self.has_smithy_tut = true
end
