module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local util		= chunks.util
local pool		= chunks.pool
require	"common.BigNum"

local stat_string = lstring("stat")

EndlessWave = chunks.class.new {
	m_owner = nil,
	m_game = nil,
	m_game_window = nil,
	m_wave = nil,
	m_should_evaluate = false,
	m_allies = nil,
	m_enemies = nil,
	m_tagged_unit = nil,
	m_last_unit_spawned = nil,
	m_xp = 0,
	m_challenge = 1,
	m_ally_count = 0,
	m_enemy_count = 0,
	m_last_group = nil,
	m_boss = nil,
	m_should_spawn_boss = false,
	m_spawn_timer = nil,		-- timer to check if it's been too long since a group was spawned
	m_total_spawn_time = nil,	-- timer to track how long until the next spawn set
	m_ally_list = nil,
	m_enemy_list = nil,
	m_label_actor = nil,
	m_min = nil,
	m_sec = nil,
	m_dead = false,
	m_score = nil,
	m_score_label = nil,
	m_score_shadow = nil,
	m_score_window = nil,
	m_score_timer = nil,
	m_pushing_back = false,
}

function bindToObject(actor)
	EndlessWave:new(actor) -- bind the script to the object
end

function EndlessWave:created( owner )
	owner.label = "Wave actor"
	self.m_owner = owner
	self.m_game = owner:getGame()
	self.m_game_window = self.m_game.lua.m_game_window
	
	self.m_game_window.lua.m_enemy_progress_bar_container.hidden_recursive = true
	self.m_ally_list = {}
	self.m_enemy_list = {}

	self.m_spawn_timer = 0
	self.m_total_spawn_time = 0
	self.m_min = 0
	self.m_sec = 0
	self.m_score_timer = 0
	self.m_score = BigNum.new(0)
	self.m_score_window = util.createWindowWithTexture(self.m_game_window, assets.textures.coin_box, false)
	self.m_score_window.anchor = {Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0, 0, 1)}
	self.m_score_window.size = self.m_score_window.size - pool.get_v2f(96, 6)
	--self.m_score_shadow = self.m_score_window:spawn(Label())
	--self.m_score_shadow.format = TextFormat("font://Helvetica-Bold", 18.0, pool.get_colorf(0.0, 0.0, 0.0, 0.5))
	--self.m_score_shadow.location = self.m_score_shadow.location + pool.get_v3f(4, -4, 0)
	self.m_score_label = self.m_score_window:spawn(Text())
	local format = {"Helvetica-Bold", 18+2.0}
	--format.shadow_color = pool.get_colorf(0.0, 0.0, 0.0, 1.0)
	--format.shadow_offset = pool.get_v2f(5.0, 5.0)
	local fx = {drop_shadow = true, shadow_color = colorf( 0, 0, 0, 1.0 ), shadow_offset = v3f( 2, -2, 0), shadow_blur_amount = 100000}
	--self.m_score_label.fx = fx
	self.m_score_label.style = format[1]
	self.m_score_label.options = {height = format[2]}
	self.m_score_label.location = self.m_score_label.location + pool.get_v3f(2, -2+6, 2)
	self:updateScore()
end

function EndlessWave:updateScore()
	local score_string = string.format("Score: %09s", tostring(self.m_score))	
	-- local score_string = string.format("Score:  %09u", math.floor(self.m_score))
	score_string = string.gsub(score_string, "(%d%d%d)", function(s) return s.." " end)
	stat_string:arg(1, score_string)
	self.m_score_label.text = stat_string
end

function EndlessWave:spawnWave(game, wave)
	print("wave start called")
	self.m_game = game
	self.m_wave = wave
	self.m_should_evaluate = true
end

function EndlessWave:gameOver(team)
	if team == GameObject.TEAM_ALLY then
		print("THE GOOD GUYS HAVE WON!")
		return false -- the good guys can never win an endles wave
	elseif team == GameObject.TEAM_ENEMY then
		print("THE BAD GUYS HAVE WON! YOU LOST!")
		self.m_dead = true
	end
	
	local previous = BigNum.new(UserData.instance():getEndlessWaveScore())
	if self.m_score > previous then
		print("score is ", tostring(self.m_score))
		UserData.instance():setEndlessWaveScore(tostring(self.m_score))
	end
	
	return true
end

EndlessWave.levels = {
	250,	-- 2
	590,	-- 3
	1310,	-- 4
	2270,	-- 5
	3460,	-- 6
	4840,	-- 7
	6400,	-- 8
	8120,	-- 9
	10000,	-- 10
}

function EndlessWave:checkLevelUp()
	if self.m_challenge > 9 then
		return
	end
	if self.m_xp > self.levels[self.m_challenge] then
		self.m_challenge = self.m_challenge + 1
	end
	print("EndlessWave: LEVEL "..self.m_challenge)
end

--=============================
-- TICK
-- Every tick, check if we should reevaluate the player's strength to send another group after them
function EndlessWave:tick( owner, delta_time )
	if self.m_dead then return end
	-- add the delkta time to their score
	-- self.m_score = self.m_score + delta_time * 10 * self.m_challenge
	self.m_score_timer = self.m_score_timer + delta_time * 10 * self.m_challenge
	if self.m_score_timer > 1 then
		local n_score = math.floor(self.m_score_timer)
		local score_tick = BigNum.new( n_score )
		self.m_score = self.m_score + score_tick

		self.m_score_timer = self.m_score_timer - n_score
	end
	self:updateScore()

	self.m_spawn_timer = self.m_spawn_timer + delta_time

	if self.m_should_evaluate then
		self.m_should_evaluate = false
		print("EndlessWave: evaluating player strength...")
		if self.m_last_group == nil then
			self:spawnBaseSmattering()
			self.m_wave:addAction( SleepAction(5) )
			self:spawnTankAndSpank()
		else
			-- pick a random grouping to send in
			-- should evaluate player's strength instead, btw
			-- including average balance of units, etc...
			self:spawnBaseSmattering()
		end
	end

	if self.m_ally_count > self.m_enemy_count then
		--print("EndlessWave: Too many allies, should I send in more enemies?")
		local line_ratio = self:checkForGateCamping()
		if line_ratio > 0.8 then
			self:spawnTankAndSpank(self.m_challenge + 2)
		elseif line_ratio > 0.4 then
			self:spawnTankAndSpank()
		end
		local start_push_ratio = 0.6 + 0.4*(1-self.m_challenge/10)
		if line_ratio > start_push_ratio and not self.m_pushing_back then
			print("EndlessWave: STARTING PUSH BACK!")
			self.m_pushing_back = true
			self:spawnPushBack()
		end
	end

	if self.m_should_spawn_boss then
		if self.m_boss == nil then
			-- spawn a boss
			self.m_should_spawn_boss = false
			local r = math.random(15)
			if (10-r) > self.m_challenge then
				-- spawn sheila
				self.m_last_unit_spawned = util.createUnitOnPath(self.m_game, "evil_sheila", 1)
			else
				self.m_last_unit_spawned = util.createUnitOnPath(self.m_game, "evil_ash", 1)
			end
			self.m_boss = self.m_last_unit_spawned
		else
			self.m_should_spawn_boss = false
		end
	end

	if self.m_spawn_timer - self.m_total_spawn_time > 4 then
		-- spawn some filler dudes
		--print(".\n.\n.\nEndlessWave: SPAWNING FILLER DUDES\n.\n.\n.")
		self:spawnFillerWave()
	end
end

function EndlessWave:checkForGateCamping()
	local game = self.m_game
	local far_right = Map.get():getLastPoint(GameObject.TEAM_ALLY).x

	local closest_enemy = game:getClosestObjectToAttack(GameObject.TEAM_ENEMY, 0):get()
	local farthest_ally = game:getClosestObjectToAttack(GameObject.TEAM_ALLY, far_right):get()

	if closest_enemy == nil then
		print("EndlessWave: no enemy to check for distance ):")
	end
	if farthest_ally == nil then
		print("EndlessWave: no allies exist for attacking")
	end
	
	local ratio = 0.0
	
	if closest_enemy and farthest_ally then
		local scrimmage = (closest_enemy.location.x + farthest_ally.location.x)/2

		ratio = scrimmage / far_right
		if ratio > 0.95 then -- someone is camping the gate
			print("EndlessWave:checkForGateCamping: camping the gate! spawn boss!")
			self.m_should_spawn_boss = true
		end
	end
	
	return ratio
end

-- Object Spawned
-- figure out if the object spawned was an ally or enemy; the do some
-- processing to see if we need to send some units after the player
function EndlessWave:objectSpawned(object_index, category, type_name)
	if category ~= "unit" then
		return
	end
	
	local object = object_index:get()
	-- don't worry about checking, this will always be a legitimate object
	
	local team = object.team
	-- GameObject.TEAM_ENEMY | GameObject.TEAM_ALLY
	
	if team == GameObject.TEAM_ALLY and self.allies[type_name] ~= nil then
		-- print("EndlessWave: ally spawned")
		-- add xp for ally units
		self.m_xp = self.m_xp + self.allies[type_name].xp
		self.m_ally_count = self.m_ally_count + 1
		-- print("EndlessWave: xp added = "..self.allies[type_name].xp)
	elseif team == GameObject.TEAM_ENEMY then
		-- print("EndlessWave: enemy spawned")
		-- enemies are added when they are queued up to avoid obvious
		-- pitfalls of rocket-spamming enemies on the player
		self.m_xp = self.m_xp + self.units[type_name].xp
		-- print("EndlessWave: xp added = "..self.units[type_name].xp)
	else
		return
	end

	-- check if we need to challenge up
	self:checkLevelUp()
end

-- Unit Died
-- if they killed the "key" unit, send more units in, otherwise see if we
-- NEED to send more units after them
function EndlessWave:unitDied(gameObjectIndex)
	local object = gameObjectIndex:get()
	-- print("EndlessWave: unit died -- "..object.team)
	-- print("EndlessWave: object.label = "..object.label)
	-- print("EndlessWave: object.level = "..object:getLevel())

	if gameObjectIndex == self.m_tagged_unit then
		-- print("EndlessWave: tagged unit was killed")
		self.m_should_evaluate = true
	end
	if gameObjectIndex == self.m_boss then
		-- add an actor to set the m_boss value to nil, to prevent a boss from spawning immediately
		-- after you kill one; this probably never occurs, but just in case...
		local temp_actor = self.m_owner:spawn(ActorBase())
		temp_actor:addAction( LuaAction( 5, function()
			self.m_boss = nil
			temp_actor:addAction( DestroyAction(0.1) )
		end) )
	end
	--local object = gameObjectIndex:get()
	if object.team == GameObject.TEAM_ALLY and self.allies[type_name] ~= nil then
		self.m_ally_count = self.m_ally_count - 1
		-- print("EndlessWave: new allies count = "..self.m_ally_count)
	elseif object.team == GameObject.TEAM_ENEMY then
		self.m_enemy_count = self.m_enemy_count - 1
		-- print("EndlessWave: new enemies count = "..self.m_enemy_count)
		local name = object.label
		local level = object:getLevel()
		if self.units[name] ~= nil then
			local points = self.units[name].points * level * self.m_challenge
			self.m_score = self.m_score + BigNum.new(points)
		else
			-- print("EndlessWave: NIL INDEX FOR ENEMY KILLED")
			-- add a mid-range score value for enemies sucked into the pit
			self.m_score = self.m_score + BigNum.new(200 * level * self.m_challenge)
		end
	end
end

EndlessWave.units = {
	crossbow_skeleton = {
		levels = {
			{2,2,3}, --1
			{2,3}, --2
			{2,2,3,3,3}, --3
			{2,3,3}, --4
			{2,3,3,3}, --5
			{2,3,3,3,3}, --6
			{2,3,3,3,3,3}, --7
			{2,3,3,3,3,3,3}, --8
			{2,3,3,3,3,3,3,3}, --9
			{3}, --10
		},
		xp = 10,
		points = 300,
	},
	drummer_skeleton = {
		levels = {
			{1}, --1
			{1}, --2
			{1}, --3
			{1}, --4
			{1}, --5
			{1}, --6
			{1}, --7
			{1}, --8
			{1}, --9
			{1}, --10
		},
		xp = 5,
		points = 200,
	},
	dryad = {
		levels = {
			{1}, --1
			{1}, --2
			{1}, --3
			{1}, --4
			{1}, --5
			{1}, --6
			{1}, --7
			{1}, --8
			{1}, --9
			{1}, --10
		},
		xp = 20,
		points = 1200,
	},
	evil_ash = {
		levels = {
			{1}, --1
			{1}, --2
			{1}, --3
			{1}, --4
			{1}, --5
			{1}, --6
			{1}, --7
			{1}, --8
			{1}, --9
			{1}, --10
		},
		xp = 75,
		points = 10000,
	},
	evil_sheila = {
		levels = {
			{1}, --1
			{1}, --2
			{1}, --3
			{1}, --4
			{1}, --5
			{1}, --6
			{1}, --7
			{1}, --8
			{1}, --9
			{1}, --10
		},
		xp = 35,
		points = 5000,
	},
	flute_skeleton = {
		levels = {
			{1}, --1
			{1}, --2
			{1}, --3
			{1}, --4
			{1}, --5
			{1}, --6
			{1}, --7
			{1}, --8
			{1}, --9
			{1}, --10
		},
		xp = 5,
		points = 100,
	},
	horseback_skeleton = {
		levels = {
			{1}, --1
			{1}, --2
			{1,1,1,2}, --3
			{1,1,1,2}, --4
			{1,2}, --5
			{1,2}, --6
			{1,2,2,2}, --7
			{1,2,2,2}, --8
			{2}, --9
			{2}, --10
		},
		xp = 15,
		points = 600,
	},
	mini_ash = {
		levels = {
			{1}, --1
			{1}, --2
			{1}, --3
			{1}, --4
			{1}, --5
			{1}, --6
			{1}, --7
			{1}, --8
			{1}, --9
			{1}, --10
		},
		xp = 5,
		points = 250,
	},
	pit_monster = {
		levels = {
			{2}, --1
			{2}, --2
			{2,2,2,3}, --3
			{2,2,2,3,3}, --4
			{2,3}, --5
			{2,2,3,3,3}, --6
			{2,3,3,3}, --7
			{2,3,3,3,3}, --8
			{3}, --9
			{3}, --10
		},
		xp = 15,
		points = 300,
	},
	shield_skeleton = {
		levels = {
			{2}, --1
			{2}, --2
			{2}, --3
			{2,2,2,2,3}, --4
			{2,2,2,3,3}, --5
			{2,2,3,3,3}, --6
			{2,3,3,3,3}, --7
			{2,3,3,3,3,3}, --8
			{2,3,3,3,3,3,3}, --9
			{3}, --10
		},
		xp = 20,
		points = 400,
	},
	spear_skeleton = {
		levels = {
			{2,2,3,3,4}, --1
			{2,3,4}, --2
			{2,2,3,3,4,4,4}, --3
			{2,3,3,4,4}, --4
			{2,2,3,3,4,4,4}, --5
			{2,3,3,3,4,4,4}, --6
			{3,4}, --7
			{3,3,4,4,4}, --8
			{3,4,4}, --9
			{3,4,4,4}, --10
		},
		xp = 10,
		points = 175,
	},
	sword_skeleton = {
		levels = {
			{3,3,4,4,4,5,6}, --1
			{3,4,4,4,4,5,6}, --2
			{3,4,4,4,5,5,6}, --3
			{3,4,4,5,5,5,6}, --4
			{3,4,4,5,5,5,6}, --5
			{3,4,4,5,5,6,6}, --6
			{3,4,5,5,6,6}, --7
			{3,4,5,5,6,6,6}, --8
			{3,4,5,6,6,6}, --9
			{3,4,5,6,6,6,6,6}, --10
		},
		xp = 15,
		points = 100,
	},
	witch = {
		levels = {
			{1}, --1
			{1}, --2
			{1}, --3
			{1}, --4
			{1}, --5
			{1}, --6
			{1}, --7
			{1}, --8
			{1}, --9
			{1}, --10
		},
		xp = 5,
		points = 150,
	},
}

EndlessWave.allies = {
	archer = {
		xp = 3,
	},
	armored_guard = {
		xp = 7,
	},
	arthur = {
		xp = 6,
	},

	henry = {
		xp = 9,
	},

	horseman = {
		xp = 4,
	},

	peasant = {
		xp = 1,
	},

	spearman = {
		xp = 3,
	},

	swordboy = {
		xp = 1,
	},

	swordsman = {
		xp = 2,
	},

	torchboy = {
		xp = 1,
	},

	wiseman = {
		xp = 3,
	},

	blacksmith = {
		xp = 10,
	},
}

function EndlessWave:generateUnitLevel(unit, challenge)
	if challenge == nil then challenge = self.m_challenge end
	if challenge < 1 then challenge = 1 end
	if challenge > 10 then challenge = 10 end
	local set = self.units[unit].levels[challenge]
	--print("EndlessWave: set found "..#(set).." permutations")

	local choice = math.random(1, #(set))
	--print("EndlessWave: choice = "..choice)
	return set[choice]
end

--=================================================
-- Spawn Unit Sets
--=================================================

-- Spawn Test Skeleton
-- simple test skeleton, spawns a level 6 test skeleton and marks it as
-- the tagged unit so that whenever it dies it spawns another
--function EndlessWave:spawnTestSkeleton()
--	print("SPAWNING TEST SKELETON")
--	self.m_wave:addAction( LuaAction(0, function ()
--		local enemy = "sword_skeleton"
--		local level = 6
--		local index = util.createUnitOnPath(self.m_game, enemy, level)
--		self.m_tagged_unit = index
--	end) )
--end

function EndlessWave:checkSpawnTimer()
	-- reset spawn timer if greater than total spawn time, since there are no more
	-- guys left, may as well be zero
	if self.m_spawn_timer > self.m_total_spawn_time then
		self.m_spawn_timer = 0
		self.m_total_spawn_time = 0
	end
end

function EndlessWave:spawnBaseSmattering()
	self:checkSpawnTimer()
	-- challenge is an integer from 1 to 10
	print("EndlessWave: spawning base smattering group")
	self.m_last_group = "smattering"
	local unit_count = math.random(2, 5) + self.m_challenge
	self.m_enemy_count = self.m_enemy_count + unit_count
	print("EndlessWave: unit count is "..unit_count)
	local tag_number = math.floor(unit_count / 2)-1
	local avg_unit_time = 2 - (self.m_challenge/10) * 1.5

	local i=0
	local unit_to_spawn = "sword_skeleton"
	local is_melee = true
	local spawned_count = 0
	while i < unit_count do
		-- see if should change to ranged unit
		local switch_rand = math.random(unit_count)
		--print("EndlessWave: switch_rand = "..switch_rand)
		if switch_rand < spawned_count then
			-- switch unit type
			print("EndlessWave: switching unit type")
			if is_melee then
				is_melee = false
				local which = math.random(1,10)
				--print("EndlessWave: ranged unit choice = "..which)
				if which < 6 then
					unit_to_spawn = "spear_skeleton"
				else
					unit_to_spawn = "crossbow_skeleton"
				end
			else
				is_melee = true
				unit_to_spawn = "sword_skeleton"
			end
		end

		local unit_level = self:generateUnitLevel(unit_to_spawn)
		local unit = unit_to_spawn
		local time = avg_unit_time + math.random()-0.5
		self.m_wave:addAction( LuaAction(time, function()
			self.m_last_unit_spawned = util.createUnitOnPath(self.m_game, unit, unit_level)
		end))
		if i == tag_number then
			print("EndlessWave: assigning unit to be tagged")
			self.m_wave:addAction(LuaAction(0, function()
				self.m_tagged_unit = self.m_last_unit_spawned
			end))
		end
		self.m_total_spawn_time = self.m_total_spawn_time + time -- increment the total spawn time

		-- cycle count up
		spawned_count = spawned_count + 1
		i = i+1
	end
end

function EndlessWave:findATank(challenge)
	if challenge == nil then challenge = self.m_challenge end
	if challenge < 1 then challenge = 1 end
	if challenge > 10 then challenge = 10 end
	-- determine between pit monster and shield skeleton
	local r = math.random(10)

	if challenge < 8 then
		if r > 5 then
			return "shield_skeleton"
		else return "pit_monster" end
	elseif challenge < 10 then
		if r > 8 then
			return "shield_skeleton"
		else return "pit_monster" end
	else -- at challenge 10, ONLY spawn shield skeletons
		return "shield_skeleton"
	end
end

function EndlessWave:spawnTankAndSpank(challenge)
	self:checkSpawnTimer()
	if challenge == nil then challenge = self.m_challenge end
	print("EndlessWave: spawning tank 'n spank group")
	self.m_last_group = "tank-n-spank"

	local unit_count = 1 + math.ceil(challenge / 2) -- units to spawn after the tank
	self.m_enemy_count = self.m_enemy_count + unit_count + 1 -- add one for at least one tank
	local unit = self:findATank()
	local unit_level = self:generateUnitLevel(unit, challenge)
	local average_spawn_time = 1.5 - (challenge/10)

	-- spawn one tank
	self.m_last_unit_spawned = util.createUnitOnPath(self.m_game, unit, unit_level)

	-- loop to spawn support units
	local i = 0
	unit = "spear_skeleton"
	local has_changed = false
	local half = math.floor(unit_count/2)
	while i < unit_count do
		-- see if we should change units partway through
		if unit_count > half and not has_changed then
			--can change
			local r = math.random(1, 10)
			if r > 4 then
				if r > 7 then
					unit = "sword_skeleton"
				else
					unit = "crossbow_skeleton"
				end
				has_changed = true
			end
		end -- end change support unit randomly
		
		unit_level = self:generateUnitLevel(unit)
		local time = average_spawn_time + math.random() - 0.5
		self.m_wave:addAction( LuaAction(time, function()
			self.m_last_unit_spawned = util.createUnitOnPath(self.m_game, unit, unit_level)
		end) )
		self.m_total_spawn_time = self.m_total_spawn_time + time

		-- increment unit count
		i = i + 1
	end
end

function EndlessWave:spawnFillerWave()
	--local text = self.m_game_window:spawn(Label())
	--text.text = "spawning filler wave"
	--text.format = TextFormat("font://Helvetica", 16.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0))
	--text:addAction( DestroyAction(5) )

	self:checkSpawnTimer()
	print("EndlessWave: spawning a filler wave")
	-- challenge is an integer from 1 to 10
	self.m_last_group = "filler"
	local unit_count = math.random(3, 5)
	self.m_enemy_count = self.m_enemy_count + unit_count
	print("EndlessWave: unit count is "..unit_count)
	local avg_unit_time = 2.2 - (self.m_challenge/10) * 1.5

	local i=0
	local unit_to_spawn = "sword_skeleton"
	local is_melee = true
	local spawned_count = 0

	local challenge = self.m_challenge - 3
	if challenge < 1 then challenge = 1 end

	while i < unit_count do
		-- see if should change to ranged unit
		local switch_rand = math.random(unit_count)
		--print("EndlessWave: switch_rand = "..switch_rand)
		if switch_rand < spawned_count then
			-- switch unit type
			print("EndlessWave: switching unit type")
			if is_melee then
				is_melee = false
				local which = math.random(1,10)
				--print("EndlessWave: ranged unit choice = "..which)
				if which < 6 then
					unit_to_spawn = "spear_skeleton"
				else
					unit_to_spawn = "crossbow_skeleton"
				end
			else
				is_melee = true
				unit_to_spawn = "sword_skeleton"
			end
		end

		local unit_level = self:generateUnitLevel(unit_to_spawn, challenge)
		local unit = unit_to_spawn
		local time = avg_unit_time + math.random()-0.5
		self.m_wave:addAction( LuaAction(time, function()
			self.m_last_unit_spawned = util.createUnitOnPath(self.m_game, unit, unit_level)
		end))
		self.m_total_spawn_time = self.m_total_spawn_time + time

		-- cycle count up
		spawned_count = spawned_count + 1
		i = i+1
	end

	-- randomly spawn a support unit with the filler wave
	local r = math.random(10)
	if r > 4 then -- spawn a support unit
		if r > 7 then -- spawn a witch
			self.m_wave:addAction( LuaAction(1, function()
				self.m_last_unit_spawned = util.createUnitOnPath(self.m_game, "witch", 1)
			end))
			self.m_total_spawn_time = self.m_total_spawn_time + 1
		elseif r > 8 then -- spawn a flute skeleton
			self.m_wave:addAction( LuaAction(1, function()
				self.m_last_unit_spawned = util.createUnitOnPath(self.m_game, "flute_skeleton", 1)
			end))
			self.m_total_spawn_time = self.m_total_spawn_time + 1
		else -- spawn a drummer skeleton
			self.m_wave:addAction( LuaAction(1, function()
				self.m_last_unit_spawned = util.createUnitOnPath(self.m_game, "drummer_skeleton", 1)
			end))
			self.m_total_spawn_time = self.m_total_spawn_time + 1
		end
	end
end

function EndlessWave:spawnPushBack()
	-- local text = self.m_game_window:spawn(Label())
	-- text.text = "Pushing back"
	-- text.format = TextFormat("font://Helvetica-Bold", 20.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0))
	-- text:addAction( DestroyAction(3) )
	
	-- like misc. wave only can randomly throw in buffs and dryad
	self:checkSpawnTimer()
	print("EndlessWave: spawning to push back")

	self.m_last_group = "pushback"
	local unit_count = math.random(2, 5)
	self.m_enemy_count = self.m_enemy_count + unit_count
	print("EndlessWave: unit count is "..unit_count)
	local avg_unit_time = 1.5 - (self.m_challenge/10)

	local i=0
	local unit_to_spawn = "sword_skeleton"
	local is_melee = true
	local spawned_count = 0

	local challenge = self.m_challenge

	while i < unit_count do
		-- see if should change to ranged unit
		local switch_rand = math.random(unit_count)
		--print("EndlessWave: switch_rand = "..switch_rand)
		if switch_rand < spawned_count then
			-- switch unit type
			print("EndlessWave: switching unit type")
			if is_melee then
				is_melee = false
				local which = math.random(1,10)
				--print("EndlessWave: ranged unit choice = "..which)
				if which < 6 then
					unit_to_spawn = "spear_skeleton"
				else
					unit_to_spawn = "crossbow_skeleton"
				end
			else
				is_melee = true
				unit_to_spawn = "sword_skeleton"
			end
		end

		local unit_level = self:generateUnitLevel(unit_to_spawn, challenge)
		local unit = unit_to_spawn
		local time = avg_unit_time + math.random()-0.5
		print("EndlessWave: pushback, queueing "..unit)
		self.m_wave:addAction( LuaAction(time, function()
			self.m_last_unit_spawned = util.createUnitOnPath(self.m_game, unit, unit_level)
		end))
		self.m_total_spawn_time = self.m_total_spawn_time + time

		-- cycle count up
		spawned_count = spawned_count + 1
		i = i+1
	end

	-- randomly spawn another (awesome) unit
	local r = math.random(1,5)
	if r > 2 then
		if r > 4 then
			-- spawn witch
			print("EndlessWave: pushback, queueing WITCH")
			self.m_wave:addAction( LuaAction(1, function()
				self.m_last_unit_spawned = util.createUnitOnPath(self.m_game, "witch", 1)
			end))
		elseif r > 3 then
			-- spawn drummer/flute
			print("EndlessWave: pushback, queueing DRUMMER")
			self.m_wave:addAction( LuaAction(1, function()
				self.m_last_unit_spawned = util.createUnitOnPath(self.m_game, "drummer_skeleton", 1)
			end))
		else
			-- spawn dryad
			print("EndlessWave: pushback, queueing DRYAD")
			self.m_wave:addAction( LuaAction(1, function()
				self.m_last_unit_spawned = util.createUnitOnPath(self.m_game, "dryad", 1)
			end))
		end
	else
		print("EndlessWave: pushback, not spawning extra")
	end

	-- add one last action to check if we need to release the push-back state
	self.m_wave:addAction( LuaAction(1, function()
		local ratio = self:checkForGateCamping()
		local let_off_ratio = 0.5 - 0.3*(1-self.m_challenge/10)
		if ratio < let_off_ratio then
			self.m_pushing_back = false	
			-- local text = self.m_game_window:spawn(Label())
			-- text.text = "Ending push back"
			-- text.format = TextFormat("font://Helvetica-Bold", 20.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0))
			-- text:addAction( DestroyAction(3) )
		else
			self:spawnPushBack()
		end
	end) )
end


