module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local util      = chunks.util
local load      = chunks.load

LuaEnemyUnit = chunks.class.inherit(chunks.baseunit.LuaBaseUnit)
{
	m_last_attack_object = nil,
}

bindToObject = chunks.class.gameObjectBinder(LuaEnemyUnit)

-- Callbacks from C++
function LuaEnemyUnit:processPartLogic(part, dt)	
	local object = part:get():getObjectIndex()
	local orders = PartOrdersIndex(object)
	if orders:get():hasOrders() then
		return
	end 
	
	local attack_object = object:get():getMainGame():getClosestObjectToAttack(GameObject.TEAM_ALLY, object:get().location.x)	
	if attack_object:get() then
		orders:get():addOrder(AttackOrder(attack_object), PartOrders.QUEUE_END)
	else
		local render = PartRenderIndex(object):get()
		if render then
			local options = {
								loop_count = 0,
								start_offset = math.random(),
								ease_out_current_animation_time = 0.3,
								ease_in_time = 0.1,
								start_new_animation_if_the_same = false,
							}
			render:playAnimation("win", options)
		end
	end
end

function LuaEnemyUnit:onTakeDamage(damage_packet, died)
	LuaEnemyUnit.super.onTakeDamage(self, damage_packet, died)
	
	local object_index = self.owner:getObjectIndex()
	local object = object_index:get()
	
	-- Blood
	local render = PartRenderIndex(object_index):get()
	if damage_packet.damage_info.damage > 0 and render then
		util.createDustEffect(object)
	end
	
	util.tryToPopOffShieldsAndHelmets(object_index)
end

function LuaEnemyUnit:onKnockedBack(distance)
    local owner = self.owner
    local game = owner:getMainGame()
    local pit = game.lua.m_pit
    if pit and pit:get() then
        pit:get().lua:throwIntoPit(owner)
    end
end

function LuaEnemyUnit:onDeath(owner)
	LuaEnemyUnit.super.onDeath(self, owner)

	util.tryToSpawnCoin(owner)
	
	local bonus = load.getStat( "units", owner.label, "ash_kill_bonus", owner:getLevel() ) or 0
	local bonus = math.floor( bonus * 0.1 + 0.5 )
	self.owner:getMainGame().lua:addGold(bonus)
	--print("AWARDED ", bonus)
end
