--| module |--
module(...,package.seeall)

--| includes |--
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants

PRODUCTS = chunks.globals.products

function purchasingAvailable()
    local iap = InAppPurchaseService.get()
    return (iap ~=nil and iap:canPurchase())
end

function displayPurchasingUnavailable()
    local ui = PlatformUIService.get()

    if ui then
        local title = lstring("purchasing_unavailable_title")
        local text = lstring("purchasing_unavailable_text")
        ui:showDialog(title, text, {lstring("ok")}, function() end)
    end
end

function getPriceString(id)
    local iap = InAppPurchaseService.get()

    if iap then
        local product = iap:getProduct(id)
        if product then
            return product:getPriceString()
        end
    end
    return lstring.format("")
end

function getProduct(product_id)
    for _,v in ipairs(PRODUCTS) do
        if v.id == product_id then
            return v
        end
    end
end

function restorePurchases(request_complete_callback)
    local handle = nil

    local iap = InAppPurchaseService.get();
    --NOTE: If purchasing is unavailable, then we do nothing.
    if iap and iap:canPurchase() then
        local callback = function(handle, status, response)
            local restore_table = {}
            local count = 0

            function runRequestCompleteCallback()
                if request_complete_callback then
                    request_complete_callback(restore_table, count)
                end
            end

            print("Restore result: " .. tostring(status))
            if status == IAPRequestResponseCode.ResponseCode_Success then

                local product_info_table = iap:consumeOutstandingPurchases();

                --TODO: TYE: Re-write this...  no need to have the restore table.
                for _,product_info in ipairs(product_info_table) do
                    restore_table[product_info.productName] = (restore_table[product_info.productName] or 0) + 1
                end

                for name, number in pairs(restore_table) do
                    if name == PRODUCTS.ad_buyout.id then
                        count = count + number

                        awardProduct(name, number)
                        print("Product: " .. name .. " (" .. tostring(number) .. ")")
                    else    
                        print("Products Restored")

                        local product = chunks.iap.PRODUCTS[name]

                        if product then
                            local points = product.points * number
                            chunks.iap.awardProduct(name, number)

                            count = count + number
                            print("Restoring " .. points .. " paper points")
                        else
                            print("Unable to restore unknown product: " .. name)
                        end
                    end
                end
            end

            print("Total products restored: " .. tostring(count))

            local ui = PlatformUIService.get()
            if ui then
                local title = lstring("restore_purchases_result_title")
                local button = lstring("ok")
                local message = nil

                if count > 0 then
                    message = lstring("restore_purchases_result_message")
                    message:arg(1, count)
                    message:arg(2, iap:getRestoreKey())
                else
                    message = lstring("restore_purchases_result_no_products_restored")
                    message:arg(1, iap:getRestoreKey())
                end

                ui:showDialog(title, message, button, runRequestCompleteCallback)
            else 
                -- Execute request complete callback.
                runRequestCompleteCallback()
            end
        end
        
        handle = iap:requestRestorePurchases(callback);
    else
        print("iAP:restorePurchases() - iAP is unavailable, ignoring request.")
    end

    return handle
end
