--| module |--
module(...,package.seeall)

show_handle = nil
preload_handle = {}
ad_ready_callback = nil
local incent_ad_provider = "google" --"fuse", "google"

function maintainPreloadCycle()
    if(incent_ad_provider == "google") then
        --this method is called whenever the user hits "play again" it double-checks the preload state to prevent
        --an edge case where the user cancels an ad in progress, breaking the preload cycle
        if(engine.instance.m_google_ad_is_loaded == false) then
            print("+++++ ad is not preloaded at the cycle check")
            local gv_ads = GoogleVideoAdsService.get()
        
            local callback = function(handle, response)
                platform_error = response.error
                print("+++++ preload cycle check callback")
                if not platform_error then
                    print("+++++ artificially injecting ad readiness")
                    engine.instance.m_google_ad_is_loaded = true
                end
                ad_ready_callback = nil
            end
        
            ad_ready_callback = googleVideoAds:preloadAd(getGoogleAdUnit(), callback)
        else
            print("+++++ ad IS preloaded at the cycle check, skipping")
        end
    end
end


function videoAvailable(tag)
    print("*** CALLED videoAvailable")
    if(incent_ad_provider == "fuse")then
        local fuse = FuseService.get()
        if(fuse ~= nil) then
            local available_according_to_fuse = fuse:checkForAvailableAds(tag)
            --print("available_according_to_fuse ", available_according_to_fuse)
            local info     = DeviceInfo.get()
            local network_connection_exists = info:query( "net.connected" )
            --print("network_connection_exists ", network_connection_exists)
            return (available_according_to_fuse and network_connection_exists)
        else
            return false
        end
    elseif(incent_ad_provider == "google") then
        local gv_ads = GoogleVideoAdsService.get()
        local empty_callback = function() end
        if(gv_ads ~= nil) then
            print("available_according_to_GVA ", engine.instance.m_google_ad_is_loaded)
            return (engine.instance.m_google_ad_is_loaded)-- and network_connection_exists)
        else
            print("GV ADS service not found")
            return false
        end
    end
end

function preloadAd(tag, on_load_callback, on_error_callback )
    print("*** CALLED preloadAd")
    if(incent_ad_provider == "fuse")then
        local fuse = FuseService.get()
        if(fuse ~= nil) then
            print("Starting preload")
            preload_handle[tag] = fuse:preloadAd(tag, function() print("Ad preloaded") end)
        end
    elseif(incent_ad_provider == "google") then
        if(engine.instance.m_google_ad_is_loaded)then
            print("Ad is already loaded, aborting preload")

            if( on_load_callback ) then
                on_load_callback()
            end
            
            return
        end
        
        googleVideoAds = GoogleVideoAdsService.get();
        if googleVideoAds ~= nil then
            local callback = function(handle, response)
            print("+*+*+*+*+*+*+*+*+*+*+*+*++*++*+*+*+*+**+*+*+ PRELOAD callback fired")
                platform_error = response.error
                if platform_error then
                    print("ERROR in preload, marking ad as NOT AVAILABLE")
                    engine.instance.m_google_ad_is_loaded = false
                    if( on_error_callback ) then
                        on_error_callback()
                    end
                else
                    print("Marking ad as AVAILABLE")
                    engine.instance.m_google_ad_is_loaded = true
                    if( on_load_callback ) then
                        on_load_callback()
                    end
                end
                engine.instance.m_ad_preload_handle = nil
            end
            print("Firing the preloadAd call")
            engine.instance.m_ad_preload_handle = googleVideoAds:preloadAd(getGoogleAdUnit(tag), callback)
        end
    end
end

function showIncentVideo(tag, success_callback, fail_callback)
    print("*** CALLED showIncentVideo")
    if(incent_ad_provider == "fuse")then
        local fuse = FuseService.get()
        Audio.get().resetEnabled = false;
        local final_callback = function(handle, response)
            --Some newer versions of iOS are not properly muting audio during the video ad. Doing so manually
            local music_on = GameOptions.instance().music_on
            Audio.get():getPlaylistGroup().volume = music_on and 1.0 or 0.0
            Audio.get().paused = false
            Audio.get().resetEnabled = true;
        
            if(response.error)then
                fail_callback()
            else
                local reward_value = response.float
                if((reward_value > 0)) then
                    success_callback()
                else
                    fail_callback()
                end
            end
        end
                
        if (fuse ~= nil) then
            --last second check to availability, could possible save us when connection is lost after button is created(and the check at creation)
            if(videoAvailable(tag)) then
                print("Ad is still available at the last second check...showing for zone " .. tag)
                --See muting comment above
                Audio.get():getPlaylistGroup().volume = 0.0
                show_handle = fuse:showIncentivizedVideoAd(tag, final_callback)
            else
                print("No fuse ad available, aborting request")
                fail_callback()
            end
        end
    elseif (incent_ad_provider == "google") then
        googleVideoAds = GoogleVideoAdsService.get()
        if googleVideoAds ~= nil then
            local callback = function(handle, response)
                --Some newer versions of iOS are not properly muting audio during the video ad. Doing so manually
                local music_on = GameOptions.instance().music_on
                Audio.get():getPlaylistGroup().volume = music_on and 1.0 or 0.0
                Audio.get().paused = false
                Audio.get().resetEnabled = true;
            
                platform_error = response.error
                messsage = nil
                if platform_error then
                    print("Error when attmpting to show video ad: ", message)
                    fail_callback()
                else
                    local reward_count = response:rewardItem():getAmount()

                    if(reward_count > 0)then
                        success_callback()
                    else
                        fail_callback()
                    end
                end
                engine.instance.m_ad_preload_handle = nil
                
                
            end
            
            Audio.get():getPlaylistGroup().volume = 0.0
            engine.instance.m_google_ad_is_loaded = false
            engine.instance.m_ad_preload_handle = googleVideoAds:showIncentivizedVideoAd(getGoogleAdUnit(), callback)
        end
    end
end

function getGoogleAdUnit()
    local info     = DeviceInfo.get()
    local platform = info:query( "platform.type" )

    if platform == "ios" then
        if info:is( "ios.ipad_ui_idiom" ) then
            return "ca-app-pub-1375302447008666/2859248633"
        else
            return "ca-app-pub-1375302447008666/8840233433"
        end
    elseif platform == "android" then
        return "ca-app-pub-1375302447008666/1242914636"
    end
    
    error("No tag specified for Ad unit!")
    return ""
end
