-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local pool      = chunks.pool
local util      = chunks.util
local load      = chunks.load
local globals   = chunks.globals
local analytics = chunks.analytics

local white = colorf(1, 1, 1, 1)
local active_color = colorf(1.0, 0.89, 0.28, 1.0)
local inactive_color = colorf(0.75, 0.75, 0.75, 1.0)
local current_wave = 0
local scroll_view_inset = v2f(1, 4)

local text_scale = 0.55
local stat_string = lstring("stat")
local purchases = globals.purchases
local incent_video_handle = nil

LuaIncentAdMenu = chunks.class.new
{
    m_selection_box = nil,
    m_popup_time = 0.2,
    m_upgrade_menu = nil
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaIncentAdMenu )

function LuaIncentAdMenu:created(window)
    print("LuaIncentAdMenu:created")
    
    local game_service = GameService.get()
    
    --For the Android Back Button
    window:getGame().lua.m_dismissable = window 

    self.m_asset_group = ResourceGroup("texture")
    Resources.get():addResourceGroup(self.m_asset_group)
    
    window.size = constants.main_menu_camera.window.world_size
    window.location = window.location + pool.get_v3f(0, 0, 450)
    window.color = pool.get_colorf(0, 0, 0, 0)
    window:addAction(AlphaAction(self.m_popup_time, InterpAction.Smooth, 0.75))

    --The background window
    local upgrade_window = util.createWindowWithTexture(window, assets.textures.incent_ad_background)
    
    upgrade_window.force_color = true
    upgrade_window.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, 0, 1)}
    upgrade_window.scale = pool.get_v3f(0, 0, 1)
    upgrade_window:addAction(ScaleAction(self.m_popup_time, InterpAction.Smooth, pool.get_v3f(1, 1, 1)))
    self.m_upgrade_window = upgrade_window

        -- Icon
    --local video_icon = util.createWindowWithTexture(window, assets.textures.video_icon)
    --video_icon.anchor = {Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0.0, -82.0, 1.0)}
    --video_icon.color = pool.get_colorf(1.0, 1.0, 1.0, 1.0)

    local label = upgrade_window:spawn( Text() )
    label.anchor = {Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0, -70, 1)}
    label.text = lstring("watch_video")
    label.style = "Georgia Bold"
    label.options = {height = 20.0, max_width = 250.0, wrapping = 250.0, align = TextAlign.Center, char_spacing = 0}      
    label.color = pool.get_colorf(1.0, 1.0, 1.0, 1.0)

    do
        --Yes button
        local text = lstring("yes")
        local format =  {"Georgia Bold", 25.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 45.0, TextAlign.Center, 0.0, 5.0, 0, false}
        local yes_button = util.createButtonWithTextAndTexture(upgrade_window, text, format, assets.textures.incent_ad_yes)
        yes_button.anchor = {Anchor.Anchor_B, Anchor.Anchor_B, pool.get_v3f(-75, 30, 1)}
        yes_button.on_release = function(owner)

            assets.playSound(assets.sounds.button_click_wav)
            engine:getUpgradeUnitMenuWindow().m_is_watching_ad = true

            incent_video_handle = chunks.incent_ads.showIncentVideo("postroll", function()
                -- Give the coins and refresh the display.
                local old_coins = UserData.instance():getGold()
                game_service:giveIncentCoins()
                local upgrade_unit_menu_window = engine:getUpgradeUnitMenuWindow()
                if upgrade_unit_menu_window then
                    upgrade_unit_menu_window.m_is_watching_ad = false
                    upgrade_unit_menu_window:updateText(true, old_coins)
                    upgrade_unit_menu_window:updateAdDisplay();
                end
                incent_video_handle = nil

            end, 
            function() 
                local upgrade_unit_menu_window = engine:getUpgradeUnitMenuWindow()
                if upgrade_unit_menu_window then
                    upgrade_unit_menu_window.m_is_watching_ad = false
                    upgrade_unit_menu_window:updateText()
                    upgrade_unit_menu_window:updateAdDisplay();
                end
                incent_video_handle = nil
                
            end)
            
            self:dismiss()
            analytics.sendIncentMenuDecision(true);
        end
    end

    do
        --No button
        local text = lstring("no")
        local format =  {"Georgia Bold", 25.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 45.0, TextAlign.Center, 0.0, 5.0, 0, false}
        local no_button = util.createButtonWithTextAndTexture(upgrade_window, text, format, assets.textures.incent_ad_no)
        no_button.anchor = {Anchor.Anchor_B, Anchor.Anchor_B, pool.get_v3f(75, 30, 1)}
        no_button.on_release = function(owner)
            assets.playSound(assets.sounds.button_click_wav)
            self:dismiss()
            analytics.sendIncentMenuDecision(false);
        end
    end
end

function LuaIncentAdMenu:dismiss(delay)
    --For the Android Back Button
    self.owner:getGame().lua.m_dismissable = nil
    
    delay = delay or 0
    
    self.m_upgrade_window:addAction(SleepAction(delay))
    self.m_upgrade_window:addAction(ScaleAction(self.m_popup_time, InterpAction.Smooth, pool.get_v3f(0, 0, 1)))
    
    self.owner:addAction(SleepAction(delay))
    self.owner:addAction(AlphaAction(self.m_popup_time, InterpAction.Smooth, 0))
    self.owner:addAction(DestroyAction(0))
end

function LuaIncentAdMenu:destroy()
    Resources.get():removeResourceGroup(self.m_asset_group)
    Resources.get():unload(self.m_asset_group)
end
