-- module --
module(...,package.seeall)

-- includes --
local engine = engine.instance
local chunks = engine.chunks
local pool   = chunks.pool
local util   = chunks.util

-- LuaInfoPopup --
LuaInfoPopup = chunks.class.inherit( chunks.popup.LuaPopup )
{
	m_title_label = nil,
	m_message_labels = nil,
	m_confirmation_button = nil,
	m_button_label = nil,
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaInfoPopup )

-- The created method.
function LuaInfoPopup:created( owner )
	-- Super.
	LuaInfoPopup.super.created( self, owner )

	local popup = self.m_popup

	-- Settings.
	assets.setTexture(popup, assets.textures.selection_box)
	popup.size = pool.get_v2f(202, 177)	

	-- Title label.
	do
		local label = popup:spawn( Text() )

		local format = {"Helvetica-Bold", 22, 170}
		label.style = format[1]
		label.options = {height = format[2], max_width = format[3]}
		label.anchor = { Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f( 0.0, -18.0, 1.0 ) }

		self.m_title_label = label
	end

	do -- Confirmation button
		local button = util.createButtonWithTexture(popup, assets.textures.upgrade_button_blank)
		button.anchor = { Anchor.Anchor_C, Anchor.Anchor_B, pool.get_v3f( 0.0, 0.00, 1.0 ) }
		button.size = button.size / 1.1

		do -- Ok Text
			local label = button:spawn( Text() )

			local format = {"Helvetica-Bold", 20, 50}
			format.width_constraint = button.size.x - 10
			
			label.style = format[1]
			--TODO shadow blur doesnt seem to be working
			local fx = {drop_shadow = true, shadow_color = colorf( 0, 0, 0, 1.0 ), shadow_offset = v3f( 2, -2, 0), shadow_blur_amount = 100000}
			label.fx = fx
			label.options = {height = format[2], max_width = format[3], align = TextAlign.Center}

			label.anchor = { Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0.0, -2.5+7, 1.0 ) }

			self.m_button_label = label
		end

		button.on_release = function ()
			assets.playSound(assets.sounds.button_click_wav)
			self:transitionOut(nil)
		end

		self.m_confirmation_button = button
	end

	self:setButtonText("OK")

	popup.scale = pool.get_v3f(0.0, 0.0, 1.0)
end

function LuaInfoPopup:setTitle(text)
	self.m_title_label.text = lstring(text)
end

function LuaInfoPopup:setMessage(message_table)
	if self.m_message_labels then
		for k,v in pairs(self.m_message_labels) do
			v:destroy()
		end
	end

	self.m_message_labels = {}

	local start_y = -45
	local line_spacing = 5
	local popup = self.m_popup
	local format = {"Helvetica-Bold", 12}
	format.width_constraint = popup.size.x - 20
	for i,message in ipairs(message_table) do
		local label = popup:spawn( Text() )
		if #message == 0 then
			label.text = lstring("")
		else
			label.text = lstring(message)
		end
		label.style = format[1]
		label.options = {height = format[2], max_width = format.width_constraint, wrapping = format.width_constraint }
		label.anchor = { Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f( 0.0, start_y - (i * label.size.y * 0.5) - ((i - 1) * line_spacing), 1.0 ) }

		table.insert(self.m_message_labels, label)
	end
end

function LuaInfoPopup:setButtonText(text)
	self.m_button_label.text = lstring(text)
end

function LuaInfoPopup:show(callback)
	self:transitionIn()

	self.m_confirmation_button.on_release = function ()
		assets.playSound(assets.sounds.button_click_wav)
		self:transitionOut(callback)
	end
end
