-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local pool      = chunks.pool
local util      = chunks.util

LuaLoadingGame = chunks.class.new
{	
	m_window_camera = nil,
	m_loading_actor = nil,
	m_fade_time = 0.125,
	m_game_window = nil,
}

-- The spawner.
spawn = chunks.class.spawner( WindowGame, LuaLoadingGame )

function LuaLoadingGame:created(game)
	print("LuaLoadingGame:created")
	
	local game_window = game:getWindow()	
	
	-- Set clearing info.	
	-- Window camera.
	do
		local camera = game:getWindow():spawn( OrthographicCamera() )

		camera.viewport_offset = constants.main_menu_camera.window.viewport_offset
		camera.viewport_size   = constants.main_menu_camera.window.viewport_size
		camera.screen_size     = constants.main_menu_camera.window.screen_size
		camera.world_size      = constants.main_menu_camera.window.world_size
		camera.clipping_planes = constants.main_menu_camera.window.clipping_planes
		camera.clear_color     = constants.clear_color
		camera.clear_color_on  = constants.loading_clear_color_on
		camera.clear_depth_on  = constants.clear_depth_on
		camera.scene           = game_window
		
		camera.render_priority = -100
		game_window.camera = camera
		self.m_window_camera = camera
	end
	
	game_window.color = colorf(0.0, 0.0, 0.0, 0.0) -- hidden but still receives input
	game_window.receive_input = false
	game_window.size = constants.main_menu_camera.window.world_size
	self.m_game_window = game_window
	
	-- Create the loading Window
	do
		self.m_loading_actor = game_window:spawn(ActorBase())
		self.m_loading_actor.color = colorf(1.0, 1.0, 1.0, 0.0)
		self.m_loading_actor.force_color = true
		
		local loading_bg = self.m_loading_actor:spawn(SpriteActor())
		loading_bg.texture = "builtin://white"
		loading_bg.size = constants.main_menu_camera.window.world_size
		loading_bg.color = colorf(0.0, 0.0, 0.0, 1.0)
		loading_bg.location = pool.get_v3f(0.0, 0.0, 1.0)
		
		local loading_image = self.m_loading_actor:spawn(SpriteActor())
		loading_image.location = pool.get_v3f(0.0, 0.0, 2.0)
		loading_image.color = colorf(1.0, 1.0, 1.0, 1.0)

		if engine.m_default_language_used then
			assets.setTexture(loading_image, assets.textures.loading_image)
		else
			local format = {"Helvetica-Bold", 13.0, pool.get_colorf(0.0, 0.0, 0.0, 1.0), 53.0, TextAlign.Center, 18.0, -2.0, 0.0, false, false}
			util.createWindowWithTextAndTexture(loading_image, lstring("loading"), format, assets.textures.loading_image_locale)
		end
	end
	
	self:setHidden(true)
end

function LuaLoadingGame:setHidden(onoff)
	self.m_loading_actor.hidden_recursive = onoff
	self.m_window_camera.hidden = onoff
	
	self.m_game_window.receive_input = not onoff
end

function LuaLoadingGame:fadeIn(...)
	self.m_loading_actor:addAction(LuaAction(0.0, function()
		self:setHidden(false)
	end))
	
	self.m_loading_actor:addAction(AlphaAction(self.m_fade_time, InterpAction.Lerp, 1))
	self.m_loading_actor:addAction(LuaAction(0.0, function() self.m_loading_actor.color = pool.get_colorf(1,1,1,1) end )) -- Really make sure it's opaque
	self.m_loading_actor:addAction(WaitFrameAction())  -- Wait here so we try and render the previous frame fully (full black) for one frame
	
	for i,callback_function in ipairs(arg) do
		if callback_function then
			self.m_loading_actor:addAction(LuaAction(0.0, function()
				callback_function()
			end))
		end
	end
	
	
end

function LuaLoadingGame:fadeOut(...)
	self.m_loading_actor:addAction(AlphaAction(self.m_fade_time, InterpAction.Lerp, 0.0))
	
	self.m_loading_actor:addAction(LuaAction(0.0, function()
		self:setHidden(true)
	end))
	
	for i,callback_function in ipairs(arg) do
		if callback_function then
			self.m_loading_actor:addAction(LuaAction(0.0, function()
				callback_function()
			end))
		end
	end
end
