module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local assets    = chunks.assets
local util      = chunks.util
local load      = chunks.load
local achievements = chunks.achievements

function cast(caster, type_name, location)
	local level = UserData.instance():getTypeLevel(type_name)
	local necktie_chance = chunks.load.getStat("spells", type_name, "necktie_chance", level)

	local rand_number = math.random(1, 100) * 0.01

	if rand_number <= necktie_chance then
		return castNecktie(caster, type_name, location)
	else
		return castSpawnUnit(caster, type_name, location)
	end

	return false
end

function spawnCastEffect(caster, animation_name)	
	local model = "mesh/objects/spells/magic_words.gr2"
	local texture = "texture/objects/spells/magic_words"	
	local effect = util.createGrannyEffect(caster:get(), model, texture, animation_name)  -- Spawn on the game object so it doesn't rotate with the render actor.
	
	local render_part = PartRenderIndex(caster):get()
	if render_part ~= nil and render_part.render_actor ~= nil then
		effect.location = render_part.render_actor.location + pool.get_v3f(0.0, 0.0, -1.0)
	end	
end

function castNecktie(caster, type_name, location)
	local function doCastNecktie()
		local game = caster:get():getMainGame()
		local ash = game.lua.m_ash

		if ash ~= nil and ash:get() ~= nil then
			ash:get().lua:showNecktie(0.0)
		end

		castHeal(caster, type_name, location)
		
		achievements.magicWordsResult("necktie")
	end

	util.playSoundAtCamera("?://" .. assets.sounds.words_fail.path, {priority=10.0})
	spawnCastEffect(caster, "necktie")

	local game = caster:get():getMainGame()
	local scene = game:getScene()
	scene:addAction(LuaAction(1.65, doCastNecktie))
	
	return true
end

function castHeal(caster, type_name, location)
	local game = caster:get():getMainGame()
	local level = UserData.instance():getTypeLevel(type_name)
	local heal_percent = chunks.load.getStat("spells", type_name, "heal_percent", level)

	local patients = game:getObjectsOnTeam(caster:get().team).values
	for i,patient_index in ipairs(patients) do
		local patient_health = PartHealthIndex(patient_index):get()
		if patient_health ~= nil then
			local heal_amount = -heal_percent * patient_health.health
			local heal_info = DamageInfo(heal_amount, DamageInfo.MAGICAL_DAMAGE)
			local heal_packet = DamagePacket(caster, heal_info)

			--print("\t\t\tHEALING", patient_index:get().label, "for", -heal_amount)
			patient_health:takeDamage(heal_packet)
			chunks.util.showHealSign(patient_index:get())
		end
	end

	return true
end

function castSpawnUnit(caster, type_name, location)
	local function doCastSpawnUnit()
		local game = caster:get():getMainGame()
		local scene = game:getScene()
		local level = UserData.instance():getTypeLevel(type_name)
		local spawn_unit_list = util.splitString(chunks.load.getStat("spells", type_name, "spawn_unit", level), ";")
		local spawn_num_list = util.splitString(chunks.load.getStat("spells", type_name, "spawn_num", level), ";")
		local unit_index = math.random(1, #spawn_unit_list)
		local spawn_unit = spawn_unit_list[unit_index]
		local spawn_num = spawn_num_list[unit_index] or 1

		local unit_spacing = 30
		local caster_location = caster:get().location

		local poof_effect_model = "mesh/objects/spells/poof.gr2"
		local poof_effect_texture = "texture/objects/spells/poof"
		local poof_effect_animation_name = "poof"

		for i=1, spawn_num do
			local unit_index = util.createUnitOnPath(game, spawn_unit, nil, caster_location.x - (unit_spacing * i), true) --the true is so we don't play the spawn sound
			local unit_object = unit_index:get()

			if unit_object ~= nil then
				local stun_time = i * 0.065
				local orders = PartOrdersIndex(unit_index):get()
				if orders ~= nil then
					orders:addOrder(KnockbackOrder(0.0, stun_time), PartOrders.QUEUE_FIRST)  -- stun them for a bit so their animations aren't in sync
				end

				local poof = util.createGrannyEffect(scene, poof_effect_model, poof_effect_texture, poof_effect_animation_name)
				local render_part = PartRenderIndex(unit_index):get()
				if render_part ~= nil and render_part.render_actor ~= nil then
					poof.location = render_part.render_actor:getWorldLocation()

					local fade_in_sequence = SequenceAction()
					fade_in_sequence:addAction(SleepAction(0.25))
					fade_in_sequence:addAction(AlphaAction(0.5, InterpAction.Lerp, render_part.render_actor.color.a))
					render_part.render_actor:addAction(fade_in_sequence)
					render_part.render_actor.color = render_part.render_actor.color * pool.get_coloroffsetf(0.75, 0.75, 1.0, 0.0) -- Color dudes summoned from spells a little blue
				else
					poof.location = unit_object:getWorldLocation()
				end

				poof.location = poof.location + pool.get_v3f(0.0, 0.0, 5.0)
			end
		end
		
		achievements.magicWordsResult("spawn")
	end

	util.playSoundAtCamera("?://" .. assets.sounds.words_success.path, {priority=10.0})
	spawnCastEffect(caster, "nicto")

	local game = caster:get():getMainGame()
	local scene = game:getScene()
	scene:addAction(LuaAction(1.65, doCastSpawnUnit))

	return true
end
