-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local pool      = chunks.pool
local events	= chunks.events

LuaMainMenuGame = chunks.class.new
{
	m_asset_group = nil,
}

-- The spawner.
spawn = chunks.class.spawner( SingleTouchWindowGame, LuaMainMenuGame )

function LuaMainMenuGame:created(game)
	print("LuaMainMenuGame:created")
	
	local game_window = game:getWindow()
	
	-- Set clearing info.
		-- Window camera.
	do
		local camera = game:getWindow():spawn( OrthographicCamera() )

		camera.viewport_offset = constants.main_menu_camera.window.viewport_offset
		camera.viewport_size   = constants.main_menu_camera.window.viewport_size
		camera.screen_size     = constants.main_menu_camera.window.screen_size
		camera.world_size      = constants.main_menu_camera.window.world_size
		camera.clipping_planes = constants.main_menu_camera.window.clipping_planes
		camera.clear_color     = constants.clear_color
		camera.clear_color_on  = constants.clear_color_on
		camera.clear_depth_on  = constants.clear_depth_on
		camera.scene           = game_window

		game_window.camera = camera
	end
	
	self.m_asset_group = ResourceGroup("texture")
	
	self:preloadAssets()
	
	-- Startup music.
	do
		Audio.get():getSoundPoolGroup().volume = GameOptions.instance().sound_on and 1.0 or 0.0
		Audio.get():getPlaylistGroup().volume = GameOptions.instance().music_on and 1.0 or 0.0
		
		chunks.music.checkOrPlaySong( assets.music.loop )
	end

	Resources.get():addResourceGroup(self.m_asset_group)
	game_window:spawn(chunks.main_menu_window.spawn())
	Resources.get():removeResourceGroup(self.m_asset_group)

	events.on_main_menu_enter()
end


function LuaMainMenuGame:destroy()
	events.on_main_menu_exit()
	self:unloadAssets()
end


function LuaMainMenuGame:preloadAssets()
	--TODO: FIXME local sound_manager = Audio.get().sound
	
	-- sounds
	--sound_manager:precache(URI(assets.getSound(assets.sounds.button_click)))
end

function LuaMainMenuGame:unloadAssets()
	Resources.get():unload(self.m_asset_group)
end

function LuaMainMenuGame:onInputEvents( game, events )
	if engine.platform.device == "android" then
		for event in events:all() do
			local e = AndroidEvent.fast_cast( event )
			if e and e.type == AndroidEvent.AndroidEvent_BackButton then
				chunks.events.onShouldExit()
			end
		end
	end
end
