-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local pool      = chunks.pool
local util      = chunks.util
local events    = chunks.events
local analytics = chunks.analytics

local ui_handle = nil

LuaMainMenuWindow = chunks.class.new
{
    m_horse = nil,
    m_play_button = nil, -- can be normal play button or endless button
    m_options_button = nil,
    m_video_handle = nil
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaMainMenuWindow )

function LuaMainMenuWindow:created(window)

    Audio.get():restoreState()

    window.hidden = true
    window.size = constants.main_menu_camera.window.world_size

    local text_controls = {}

    --Hack demo code
    if GameEngineLuaTable.BuildInfo.isUnlockedDemo then
        local ud = UserData.instance()

        ud:setCurrentWave(math.max(ud:getCurrentWave(), 20))
        ud:setLocked("iron_fist", false)
        ud:setLocked("chainsaw", false)
        ud:setGold(math.max(ud:getGold(), 10000))
        ud:setTypeLevel("ash", math.max(ud:getTypeLevel("ash"), 4))

        for _, unit in ipairs(util.getValidUnits()) do
            ud:setTypeLevel(unit.name, math.max(ud:getTypeLevel(unit.name), 4))
            chunks.units[unit.name].unlock_wave = math.floor(chunks.units[unit.name].unlock_wave / 2)
        end

        for _, spell in ipairs(util.getValidSpells()) do
            ud:setTypeLevel(spell.name, math.max(ud:getTypeLevel(spell.name), 4))
            chunks.spells[spell.name].unlock_wave = math.floor(chunks.spells[spell.name].unlock_wave / 2)
        end

        chunks.globals.auto_select_max_wave = 50
        chunks.util.tryToAutoSelectObjects()
    end

    local background_size = { x = assets.textures.castle_front_background.size.x, y = assets.textures.castle_front_background.size.y }

    -- Background
    do
        local background = window:spawn(Window())
        assets.setTexture(background, assets.textures.castle_front_background)
        background.anchor = { Anchor.Anchor_L, Anchor.Anchor_L, pool.get_v3f(0,0,-150) }

        -- Gate Mask
        do
            local mask = background:spawn(Window())
            assets.setTexture(mask, assets.textures.castle_front_mask)
            local offset = pool.get_v3f(0, background_size.y * -0.12, 250.0)
            mask.anchor = {Anchor.Anchor_L, Anchor.Anchor_L, offset}
            mask.receive_input = false
        end

        do -- Create animated flames on background
            local create_fire = function()

                local anchor_window = background:spawn(Window())
                local fire = background:spawn(SpriteActor())
                assets.setTexture(fire, assets.textures.fire_sprite)
                fire.color = pool.get_colorf(1.0, 1.0, 1.0, 0.75)

                return fire
            end

            do -- Closest torch
                local anchor_window = background:spawn(Window())
                local offset = pool.get_v3f(background_size.x * 0.15, 
                    background_size.y * -0.01, 
                    251.0)
                anchor_window.anchor = {Anchor.Anchor_L, Anchor.Anchor_L, offset}

                local fire = anchor_window:spawn(SpriteActor())
                assets.setTexture(fire, assets.textures.fire_sprite)
                fire.color = pool.get_colorf(1.0, 1.0, 1.0, 0.75)
                fire.scale = pool.get_v3f(2.5, 2.5, 1.0)
            end

            do -- Furthest torch
                local anchor_window = background:spawn(Window())
                local offset = pool.get_v3f(background_size.x * 0.275, 
                    background_size.y * 0.175, 
                    1.0)
                anchor_window.anchor = {Anchor.Anchor_L, Anchor.Anchor_L, offset}

                local fire = anchor_window:spawn(SpriteActor())
                assets.setTexture(fire, assets.textures.fire_sprite)
                fire.color = pool.get_colorf(1.0, 1.0, 1.0, 0.75)
                fire.scale = pool.get_v3f(2.0, 2.0, 1.0)
            end
        end
    end

    -- Logo
    do
        local logo = util.createWindowWithTexture(window, assets.textures.logo)
        logo.anchor = {Anchor.Anchor_TL, Anchor.Anchor_L, pool.get_v3f(-7.0, 175.0, 200.0)}
    end

    local snap_right_offset = 5;
    local button_spacing = 5
    local world_size = constants.main_menu_camera.window.world_size
    
    -- Privacy Policy Button
    local privacy_button
    do
        privacy_button = util.createButtonWithTexture(window, assets.textures.privacy_button)
        privacy_button.location = pool.get_v3f(0.0, 0.0, 200.0)
        
        util.snapBottom(privacy_button, world_size, 5)
        util.snapRight(privacy_button, world_size, 7)
        
        privacy_button.on_release = function(owner)
            local ui = PlatformUIService.get()
            if ui then
                local callback = function()
                    print("web view disappeared")
                    self.m_platform_handle = nil
                end
                self.m_platform_handle = ui:showWebView("http://www.backflipstudios.com/help/privacy-policy-3/", "", UIOrientation.Landscape, callback)
            end
        end
    end
    
    local sprite = privacy_button:spawn(SpriteActor())
    assets.setTexture(sprite, assets.textures.privacy_button_text)
    sprite.location = pool.get_v3f(0, 0, 1)

     -- More Games Button
    local more_games_button
    do
        --if (GameBridge.instance():query("moregames.supported")) then
        if (false) then

            -- If English is being used just use the original texture
            if engine.m_default_language_used then
                more_games_button = util.createButtonWithTexture(window, assets.textures.more_games_button)
            else    -- Not English, so we use the alternate image and text
                more_games_button = util.createButtonWithTexture(window, assets.textures.more_games_button_locale)

                local fx = {drop_shadow = true, shadow_color = colorf( 0, 0, 0, 1.0 ), shadow_offset = v3f( 2, -2, 0), shadow_blur_amount = 100000}

                local more_label = more_games_button:spawn(Text())
                more_label.text = lstring("more")
                more_label.style = "Gothic"
                more_label.options = {height = 24.0, font_color = pool.get_colorf(1.0, 1.0, 1.0, 1.0), max_width = 47.0, align = TextAlign.Left}
                more_label.anchor = { Anchor.Anchor_L, Anchor.Anchor_L, pool.get_v3f(15.0, 12.0, 0.1) }
                more_label.fx = fx;

                local games_label = more_games_button:spawn(Text())
                games_label.text = lstring("games")
                games_label.style = "Gothic"
                games_label.options = {height = 24.0, font_color = pool.get_colorf(1.0, 1.0, 1.0, 1.0), max_width = 55.0, align = TextAlign.Right}
                games_label.anchor = { Anchor.Anchor_R, Anchor.Anchor_R, pool.get_v3f(-16.0, -4.0, 0.1) }
                games_label.fx = fx;


                --Add to the list of text controls so we can unify the sizes at the end
                table.insert(text_controls, more_label)
                table.insert(text_controls, games_label)
            end

            more_games_button.location = pool.get_v3f(0.0, 0.0, 200.0)
            util.snapAboveOther(more_games_button, privacy_button, 0);
            util.snapRight(more_games_button, world_size, snap_right_offset - 6);

            more_games_button.on_release = function (button)
                assets.playSound(assets.sounds.button_click_wav)
                events.on_more_games()
            end
        end
    end

    -- Options Button
    local options_button
    do
        local text = lstring("options")
        local format =  {"Gothic_Large", 30.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 75.0, TextAlign.Right, -14.0, 1.0}

        options_button, options_label = util.createButtonWithDefaultOrLocale(window, text, format, assets.textures.options_button_locale, assets.textures.options_button)
        options_button.location = pool.get_v3f(0.0, 0.0, 200);
        util.snapRight(options_button, world_size, snap_right_offset - 6)
        if more_games_button ~= nil then
            util.snapAboveOther(options_button, more_games_button, -6);
        else
            util.snapAboveOther(options_button, privacy_button, -6);
        end
        options_button.on_release = function (button)
            if GameEngineLuaTable.BuildInfo.isDemo == false then
                assets.playSound(assets.sounds.button_click_wav)
                engine:spawnOptionsMenu()
                --print("options")
            end
        end
        self.m_options_button = options_button

        if options_label then
            table.insert(text_controls, options_label)
        end
    end

    if util.isEndlessModeUnlocked() == true then
        -- Endless Button
        local text = lstring("endless")
        local format =  {"Gothic_Large", 35.0, pool.get_colorf(0.63, 0.27, 1.0, 1.0), 115.0, TextAlign.Center, 0.0, 5.0, 0.8}

        local endless_button, endless_label = util.createButtonWithDefaultOrLocale(window, text, format, assets.textures.endless_button_locale, assets.textures.endless_button)
        endless_button.location = pool.get_v3f(0.0, 0.0, 200);
        util.snapRight(endless_button, world_size, snap_right_offset - 6)
        util.snapAboveOther(endless_button, options_button, -6);

        endless_button.on_release = function (button)
            UserData.instance():setCurrentWave(globals.endless_wave)

            assets.playSound(assets.sounds.button_click_wav)
            engine:spawnUpgradeUnitMenu()
        end

        local scale_offset = 0.05
        local scale_time = 0.6
        local pulse_loop = LoopAction()
        pulse_loop:addAction(ScaleOffsetAction(scale_time, InterpAction.Smooth, pool.get_v3f(scale_offset, scale_offset, 0.0)))
        pulse_loop:addAction(ScaleOffsetAction(scale_time, InterpAction.Smooth, pool.get_v3f(-scale_offset, -scale_offset, 0.0)))
        endless_button:addAction(pulse_loop)

        self.m_play_button = endless_button

        if endless_label then
            table.insert(text_controls, endless_label)
        end
    else
        -- Play Button
        local text = lstring("play")
        local format =  {"Gothic_Large", 45.0, pool.get_colorf(1.0, 0.0, 0.0, 1.0), 62.0, TextAlign.Left, 15.0, 3.0, 1.0}

        local play_button, play_label = util.createButtonWithDefaultOrLocale(window, text, format, assets.textures.play_button_locale, assets.textures.play_button)
        play_button.location = pool.get_v3f(0.0, 0.0, 200);
        util.snapRight(play_button, world_size, snap_right_offset - 6)
        util.snapAboveOther(play_button, options_button, -6);

        play_button.on_release = function (button)
            if UserData.instance():getCurrentWave() > globals.final_wave then
                UserData.instance():setCurrentWave(globals.final_wave)
            end
        
            if util.getBooleanValue("has_seen_intro", false) then
                engine:spawnUpgradeUnitMenu()
            else

                -- Pause the horse sounds to stop them from playing during video
                local gallop_handle = self.m_horse.lua.gallop_sound
                if gallop_handle then
                    gallop_handle:stop()
                end

                local scount_handle = self.m_horse.lua.scout_sound
                if scount_handle then
                    scount_handle:stop()
                end
                
                util.playIntroVideo( self.spawnUpgradeMenu );
            end
            assets.playSound(assets.sounds.button_click_wav)
        end
        self.m_play_button = play_button

        if play_label then
            table.insert(text_controls, play_label)
        end
    end


    -- Leaderboards Button
    local gc =  GameCenterService.get()
    if gc then
        local leaderboards_button = util.createButtonWithTexture(window, assets.textures.leaderboards_button)
        leaderboards_button.anchor = {Anchor.Anchor_L, Anchor.Anchor_C, pool.get_v3f(0.0, -137.0, 200.0)}
        leaderboards_button.on_release = function (button)

            util.requireGCLogin( function()

                assets.playSound(assets.sounds.button_click_wav)
                gc:displayLeaderboardsUI("", function() end)
                analytics.sendLeaderboardUIOpened()

            end, function()

                --Tell user to go login to GC
                local callback = function (handle, response, result)
                    m_platform_ui_prompt_handle = nil
                end
                platform_ui = PlatformUIService.get();
                if platform_ui then
                    local title = lstring("gc_relogin_title")
                    local text = lstring("gc_relogin_text")
                    local ok = {lstring("gc_relogin_okay")}
                    m_platform_ui_prompt_handle = platform_ui:showDialog(title, text, ok, callback);
                end

            end ) 
        end
    end

    -- Achievements Button
    if gc then
        local achievements_button = util.createButtonWithTexture(window, assets.textures.achievements_button)
        achievements_button.anchor = {Anchor.Anchor_L, Anchor.Anchor_C, pool.get_v3f(45.0, -137.0, 200.0)}
        achievements_button.on_release = function (button)
        
            util.requireGCLogin( function()

                gc:displayAchievementsUI(function() end)        -- no action is necessary on display so pass an empty function
                assets.playSound(assets.sounds.button_click_wav)
                analytics.sendAchievementsUIOpened()

            end, function()

                local callback = function (handle, response, result)
                    m_platform_ui_prompt_handle = nil
                end
                platform_ui = PlatformUIService.get();
                if platform_ui then
                    local title = lstring("gc_relogin_title")
                    local text = lstring("gc_relogin_text")
                    local ok = {lstring("gc_relogin_okay")}
                    m_platform_ui_prompt_handle = platform_ui:showDialog(title, text, ok, callback);
                end

            end )
        end
    end

    -- Unify the text heights
    util.unifyTextHeight(text_controls)

    -- Spawn background horse with actions
    do
        local horse = util.loadObjectRenderDataFromStats(window, "units", "horseman", 1)
        if horse ~= nil then
            local options = {
                        loop_count = 0,
                        ease_out_current_animation_time = 0.3,
                        ease_in_time = 0.1,
                    }

            horse.lua = horse.lua or {}

            local move_time_one_way = 5.0
            local distance = 800.0

            horse.scale = pool.get_v3f(0.9, 0.9, 1.0)
            horse.location = pool.get_v3f(distance * -0.5, -105.0, 50.0)
            self.m_horse = horse

            -- The move sequence
            do
                function create_random_sequecne ()
                    local sequence = SequenceAction()

                    -- Pause offscreen
                    sequence:addAction(SleepAction(math.random() * 2))

                    -- Walk all the way out
                    sequence:addAction(LuaAction(0.0, function (inner)
                        inner.scale = pool.get_v3f(math.abs(inner.scale.x), inner.scale.y, inner.scale.z)
                        inner.lua.gallop_sound = assets.playLoopingSound(assets.sounds.horse_gallop)
                        horse:playAnimation("walk", options)
                    end))
                    sequence:addAction(MoveOffsetAction(move_time_one_way, InterpAction.Lerp, pool.get_v3f(distance, 0.0, 0.0)))

                    -- Pause offscreen
                    sequence:addAction(LuaAction(0.0, function (inner)
                        inner.lua.gallop_sound:stop()
                    end))
                    sequence:addAction(SleepAction(math.random() * 10))

                    -- Play the scout approaching sound
                    if math.random() <= 0.4 then
                        sequence:addAction(LuaAction(0.0, function (inner)
                            inner.lua.scout_sound = assets.playSound(assets.sounds.soldier_a_scout_approaches)
                        end))

                        sequence:addAction(SleepAction(1.0))
                    end

                    -- Move half way back
                    sequence:addAction(LuaAction(0.0, function (inner)
                        inner.scale = pool.get_v3f(math.abs(inner.scale.x) * -1.0, inner.scale.y, inner.scale.z)
                        inner.lua.gallop_sound = assets.playLoopingSound(assets.sounds.horse_gallop)
                    end))
                    sequence:addAction(MoveOffsetAction(move_time_one_way * 0.5, InterpAction.Lerp, pool.get_v3f(distance * -0.5, 0.0, 0.0)))

                    -- Pause and look back?
                    local pause_random = math.random()
                    if pause_random <= 0.3 then
                        sequence:addAction(LuaAction(0.0, function (inner)
                            inner.lua.gallop_sound:stop()
                            inner:playAnimation("idle", options)
                        end))

                        sequence:addAction(LuaAction(0.5, function (inner)
                            assets.playSound(assets.sounds.horse_snort)
                            inner.scale = pool.get_v3f(math.abs(inner.scale.x), inner.scale.y, inner.scale.z)
                        end))
                        sequence:addAction(SleepAction(math.random() * 3))
                        sequence:addAction(LuaAction(0.0, function (inner)
                            inner.scale = pool.get_v3f(math.abs(inner.scale.x) * -1.0, inner.scale.y, inner.scale.z)
                        end))
                        sequence:addAction(LuaAction(0.0, function (inner)
                            inner.lua.gallop_sound = assets.playLoopingSound(assets.sounds.horse_gallop)
                            inner:playAnimation("walk", options)
                        end))
                    -- Rear up and buck
                    elseif pause_random <= 0.6 then
                        sequence:addAction(LuaAction(0.0, function (inner)
                            inner.lua.gallop_sound:stop()
                            inner:playAnimation("idle", options)
                        end))
                        sequence:addAction(SleepAction(math.random() * 2))
                        sequence:addAction(LuaAction(0.0, function (inner)
                            assets.playSound(assets.sounds.horse_whinny)
                            inner:playAnimation("attack_b", options)
                        end))
                        sequence:addAction(LuaAction(1.0, function (inner)
                            inner.lua.gallop_sound = assets.playLoopingSound(assets.sounds.horse_gallop)
                            inner:playAnimation("walk", options)
                        end))
                    end

                    -- Move the rest of the way back
                    sequence:addAction(MoveOffsetAction(move_time_one_way * 0.5, InterpAction.Lerp, pool.get_v3f(distance * -0.5, 0.0, 0.0)))
                    sequence:addAction(LuaAction(0.0, function (inner)
                        inner.lua.gallop_sound:stop()
                    end))

                    -- Start a new action in a bit
                    sequence:addAction(LuaAction(math.random() * 10, function (inner)
                        inner:addAction(create_random_sequecne())
                    end))

                    return sequence
                end

                horse:addAction(create_random_sequecne())
            end
        end
    end

     -- IAP Warning
    local has_seen_iap_warning = UserData.instance().iap_warning
    if not has_seen_iap_warning then
        print("Running IAP warning task")
        local ui = PlatformUIService.get()
        if ui ~= nil then
            local title = lstring("iap_warning_title")
            local body = lstring("iap_warning_body")
            local okay =  { lstring("ok") }
            ui_handle = ui:showDialog(title, body, okay, function(handle, responseCode, response)
                ui_handle = nil
                print("IAP warning task completed: " .. tostring(responseCode))
                UserData.instance().iap_warning = true
            end)
        end
    end

    if( not ui_handle ) then
        print("Running app update check task")
        local ui = PlatformUIService.get()
        if ui ~= nil then
            local message = lstring("application_update_message");
            local yes_button = lstring("application_update_yes");
            local no_button = lstring("application_update_no");
            local later_button = lstring("application_update_later");

            local callback = function(handle, responseCode, response)
                print("App update check task completed: " .. tostring(responseCode))
                ui_handle = nil
            end

            ui_handle = ui:promptForApplicationUpdate(message, lstring(), yes_button, no_button, later_button, callback)
        end
    end
end

function LuaMainMenuWindow:spawnUpgradeMenu()
    engine:spawnUpgradeUnitMenu()
end

function LuaMainMenuWindow:destroy(window)
    local gallop_handle = self.m_horse.lua.gallop_sound
    if gallop_handle then
        gallop_handle:stop()
        --TODO: FIXME Audio.get().sound:unloadSounds("sound", ".*gallop.*")
    end
    
    if self.m_video_handle then
        self.m_video_handle:cancel()
    end
end
