module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local util      = chunks.util

LuaNecronomicon = chunks.class.inherit(chunks.baseunit.LuaBaseUnit)
{
	m_last_damage_time = 0.0,
	m_flash_sprite = nil,
}

bindToObject = chunks.class.gameObjectBinder(LuaNecronomicon)

function LuaNecronomicon:onBindToObject(object)	
	LuaNecronomicon.super.onBindToObject(self, object)	
	local actual_object = self.owner
	
	local render_actor = PartRenderIndex(actual_object:getObjectIndex()):get().render_actor
	
	do	-- Create the flash effect and hide it
		local flash_sprite = render_actor:spawn(SpriteActor())
		assets.setTexture(flash_sprite, assets.textures.necro_gate_flash)
		flash_sprite.location = pool.get_v3f(0.0, 0.0, 1.0)
		flash_sprite.hidden = true
		self.m_flash_sprite = flash_sprite
	end
end

-- Overidding LuaBaseUnit:addHealthBar - The Necronomicon gets a custom health bar
function LuaNecronomicon:addHealthBar()	
	if self.m_health_bar_container ~= nil then
		return
	end

	local object = self.owner:getObjectIndex()
	local part_health = PartHealthIndex(object):get()
	if part_health == nil then
		return
	end
	
	-- Create Health Bar
	local render_actor = PartRenderIndex(object):get().render_actor
	
	local health_bar_size = v2f(50, 8)
	self.m_health_bar, self.m_health_bar_container = util.createHealthBarSprite(render_actor, health_bar_size, colorf(0.0, 0.7, 0.0, 1.0))
	self.m_health_bar_container.location = self.m_health_bar_container.location + pool.get_v3f(0.0, 55.0, 0.0)
		
	local overlay = self.m_health_bar_container:spawn(SpriteActor())
	assets.setTexture(overlay, assets.textures.enemy_progress_bar_front)
	overlay.location = pool.get_v3f(0.0, 0.0, 2.0)
	overlay.scale = v3f(health_bar_size / overlay.size) + pool.get_v3f(0.0325, 0.2, 1.0)	
end

-- Callbacks from C++
function LuaNecronomicon:onTakeDamage(damage_packet, died)	
	LuaNecronomicon.super.onTakeDamage(self, damage_packet, died)

	local object = self.owner:getObjectIndex():get()
	local game = MainGame.fast_cast(object:getGame())
	
	util.scaleHealthBarForObject(object)
	
	if damage_packet.damage_info.damage > 0.0 then
		self.m_last_damage_time = game:getGameTime()
		
		-- Flash the gate
		if self.m_flash_sprite:hasActions() == false then
			local sequence = SequenceAction()
			sequence:addAction(LuaAction(0.0, function (inner)
				inner.color = pool.get_colorf(1.0, 1.0, 1.0, 0.0)
				inner.hidden = false
			end))
			sequence:addAction(AlphaAction(0.1, InterpAction.Ramp, 1.0))
			sequence:addAction(AlphaAction(0.1, InterpAction.Ramp, 0.0))
			sequence:addAction(LuaAction(0.0, function (inner)
				inner.hidden = true
			end))
			
			self.m_flash_sprite:addAction(sequence)
		end
	end
end
