-- module --
module(...,package.seeall)

-- includes --
local engine = engine.instance
local chunks = engine.chunks
local pool   = chunks.pool
local util   = chunks.util

-- LuaOptionsResetPopup --
LuaOptionsResetPopup = chunks.class.inherit( chunks.popup.LuaPopup )
{
	m_title_label = nil,
	m_message_labels = nil,
	m_confirmation_button = nil,
	m_cancel_button = nil,
	m_button_label = nil,
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaOptionsResetPopup )

-- The created method.
function LuaOptionsResetPopup:created( owner )
	-- Super.
	LuaOptionsResetPopup.super.created( self, owner )

	local popup = self.m_popup

	-- Settings.
	assets.setTexture(popup, assets.textures.selection_box)
	popup.size = pool.get_v2f(202, 177)	

	-- Title label.
	do
		local label = popup:spawn( Text() )

		local format = {"Helvetica-Bold", 20, TextAlign.Center}
		label.style = format[1]
		label.options = {height = format[2], max_width = 180, align = TextAlign.Center}
		label.anchor = { Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f( 0.0, -14.0, 1.0 ) }
		label.text = lstring("RESET_GAME")

		self.m_title_label = label
	end
	
	-- Message label.
	do
		local label = popup:spawn( Text() )

		label.style = "Helvetica-Bold"
		label.options = {height = 16, wrapping = 170, max_width = 170, align = TextAlign.Center, max_lines = 6}
		label.anchor = { Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f( 0.0, -43.0, 1.0 ) }
		label.text = lstring("RESET_MESSAGE")

		self.m_message_label = label
	end
	
	

	do -- Confirmation button
		local button = util.createButtonWithTexture(popup, assets.textures.upgrade_button_blank)
		button.anchor = { Anchor.Anchor_C, Anchor.Anchor_B, pool.get_v3f( 0.25*popup.size.x, 0.00, 1.0 ) }

		do -- Ok Text
			local label = button:spawn( Text() )

			local format = {"Helvetica-Bold", 16}
			format.width_constraint = button.size.x - 10
			
			label.style = format[1]

			local fx = {drop_shadow = true, shadow_color = colorf( 0, 0, 0, 1.0 ), shadow_offset = v3f( 2, -2, 0), shadow_blur_amount = 100000}
			label.fx = fx
			label.options = {height = format[2], max_width = 50, align = TextAlign.Center}

			label.anchor = { Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0.0, -2.5+7, 1.0 ) }
			
			label.text = lstring("RESET")

			self.m_button_label = label
		end

		button.on_release = function ()
			assets.playSound(assets.sounds.button_click_wav)
			self:transitionOut(nil)
		end

		self.m_confirmation_button = button
	end
	
	
	do -- Cancel button
		local button = util.createButtonWithTexture(popup, assets.textures.upgrade_button_blank)
		button.anchor = { Anchor.Anchor_C, Anchor.Anchor_B, pool.get_v3f( -0.25*popup.size.x, 0.00, 1.0 ) }

		do -- Cancel Text
			local label = button:spawn( Text() )

			local format = {"Helvetica-Bold", 16}
			format.width_constraint = button.size.x - 10
			
			label.style = format[1]

			local fx = {drop_shadow = true, shadow_color = colorf( 0, 0, 0, 1.0 ), shadow_offset = v3f( 2, -2, 0), shadow_blur_amount = 100000}
			label.fx = fx
			label.options = {height = format[2], max_width = 50, align = TextAlign.Center}

			label.anchor = { Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f( 0.0, -2.5+7, 1.0 ) }
			
			label.text = lstring("CANCEL")

			self.m_button_label = label
		end

		button.on_release = function ()
			assets.playSound(assets.sounds.button_click_wav)
			self:transitionOut(nil)
		end

		self.m_cancel_button = button
	end
	
	
	

	popup.scale = pool.get_v3f(0.0, 0.0, 1.0)
end

function LuaOptionsResetPopup:show(callback)
	self:transitionIn()

	self.m_confirmation_button.on_release = function ()
		assets.playSound(assets.sounds.button_click_wav)
		self:transitionOut(callback)
	end
end
