-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local pool      = chunks.pool
local util      = chunks.util
local globals   = chunks.globals
local achievements = chunks.achievements

local tally = nil

LuaPauseWindow = chunks.class.new
{
	m_asset_group = nil,
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaPauseWindow )

function LuaPauseWindow:created(window)	
	local game = window:getGame()
	game.paused = true	
	
	window.hidden = true
	window.receive_input = false
	
	self.m_asset_group = ResourceGroup("texture")
	Resources.get():addResourceGroup(self.m_asset_group)
	
	-- Background Window
	do
		local bg = window:spawn(Window())
		bg.size = constants.game_camera.window.world_size
		bg.color = pool.get_colorf(0, 0, 0, 0.5)
	end
	
	-- Current Level
	do
		local format = {"Georgia Bold", 32.0, pool.get_colorf(0.93, 0.72, 0.26, 1.0)}
		
		local label = window:spawn(Text())
		label.style = format[1]
		--TODO shadow blur doesnt seem to be working
		local fx = {drop_shadow = true, shadow_color = colorf( 0, 0, 0, 1.0 ), shadow_offset = v3f( 2, -2, 0), shadow_blur_amount = 100000}
		label.fx = fx
		label.options = {height = format[2], font_color = format[3]}
		label.anchor = {Anchor.Anchor_T, Anchor.Anchor_C, pool.get_v3f(0.0, 135.0, 1.0)}
		local wave = util.getCurrentWave()
		local wave_string = lstring("wave")
		wave_string:arg(1, wave)
		label.text = wave_string
		
	end
	
	-- Paused Image/Text
	do
		if ( engine.m_default_language_used ) then
			local window_anchor = { Anchor.Anchor_T, Anchor.Anchor_C, pool.get_v3f(0.0, 95.0, 2.0) }
			local paused_label = window:spawn(Window())
			paused_label.anchor = window_anchor
		
			assets.setTexture(paused_label, assets.textures.paused_label)

		else
			local text_options = 	{
										height = 50,
									  	max_width = 400,
									  	align = TextAlign.Center,
									  	--outline_color = pool.get_colorf(0,0,0,1),
									  	font_color = colorf(1.0, 0.1, 0.1, 1),
									  	spacing = 1.0
								 	}
			util.createText(window, "paused", "Gothic_HD", text_options, {x = 0.0, y = 80.0}, true)
		end
	end
	
    local button_text_controls = {}

	-- Resume Button (says "Unpause")    
	do
		local format = {"Gothic", 28.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 90.0, TextAlign.Center, -1.0, 3.0, 1.2}

        -- HACK: Sorry, the localization code won't use both the localization_scaling_limit and line wrapping together.
        if engine.m_language == "it" then
            format[2] = 25
            format.wrapping = 90
            format.max_lines = 2
        else
            format.localization_scaling_limit = 0.01
        end

		local button, text = util.createButtonWithDefaultOrLocale(window, lstring("unpause"), format, assets.textures.resume_button_locale, assets.textures.resume_button)

		button.anchor = { Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0.0, -5.0, 2.0) }

		-- Resume the game
		button.on_release = function(owner)
			util.playSoundAtCamera(assets.sounds.button_click_wav)
			--game.paused = false
			--window:destroy()
			self.owner:getGame().lua.m_game_window.lua:showHidePauseWindow()
		end

        -- Only add unification to text sizes if there isn't multiple lines.
        if not format.wrapping and text then
            table.insert(button_text_controls, text)
        end 
	end
	
	--Restart Button
	do
		local button = util.createButtonWithTexture(window, assets.textures.restart_button)
		button.anchor = {Anchor.Anchor_R, Anchor.Anchor_C, pool.get_v3f(-20.0, -75.0, 20.0)}
		
		button.on_release = function(owner)
			achievements.waveQuit(UserData.instance():getCurrentWave())
			game.lua:sendWaveEndedAnalytics(globals.wave_ended_state.QUIT)
			engine:spawnMainGame()
		end
	end
	
	-- Quit Button
	do
		local button = window:spawn(Button())

		local format = {"Gothic", 28.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 68.0, TextAlign.Center, 0.0, 3, 1.4}
        format.localization_scaling_limit = 0.01
		local button, text = util.createButtonWithDefaultOrLocale(window, lstring("quit"), format, assets.textures.quit_button_locale, assets.textures.quit_button)

		button.anchor = { Anchor.Anchor_L, Anchor.Anchor_C, pool.get_v3f(20.0, -75.0, 2.0) }

		-- Quit the game
		button.on_release = function(owner)
			--assets.playSound(assets.sounds.button_click_wav)
			--TODO: FIXME Audio.get().sound:unloadSounds("sound", ".*")
			util.playSoundAtCamera(assets.sounds.button_click_wav)
			achievements.waveQuit(UserData.instance():getCurrentWave())
			game.lua:sendWaveEndedAnalytics(globals.wave_ended_state.QUIT)
			engine:spawnMainMenu()
		end

        if text then
            table.insert(button_text_controls, text)
        end
	end

    if #button_text_controls > 1 then
        util.unifyTextHeight(button_text_controls)
    end
	
	Resources.get():removeResourceGroup(self.m_asset_group)

--[[
	local game = MainGame.fast_cast(window:getGame())
	
	local num_enemies = game:getNumObjectsOnTeam(GameObject.TEAM_ENEMY)
	local num_allies = game:getNumObjectsOnTeam(GameObject.TEAM_ALLY)
	local num_neutral = game:getNumObjectsOnTeam(GameObject.TEAM_NEUTRAL)
	local num_coins = game:getNumObjectsOnTeam(GameObject.TEAM_COIN)
	
	print("num enemies", num_enemies)
	print("num allies", num_allies)
	print("num neutral", num_neutral)
	print("num coins", num_coins)
	
	tally = {}
	
	self:tallyChildren(game:getScene(), "scene")
	
	local total = 0
	for k,v in pairs(tally) do
		print(v, k)
		total = total + v
		
	end
	print("total", total)
]]

end


function LuaPauseWindow:tallyChildren(owner, parent_name)

	local size = owner:getNumOwned()
	for i=0,size-1 do
		local actor = owner:getOwned(i)
		
		local label = actor.label
		if #label == 0 then
			label = "unlabled"
		end
		
		local type = ""
		local granny = GrannyActor.fast_cast(actor)
		if GrannyActor.fast_cast(actor) then
			type = "granny"
		elseif SpriteActor.fast_cast(actor) then
			type = "sprite"
		--elseif GrannyBoneActor.fast_cast(actor) then
			--type = "granny_bone"
		elseif GameObject.fast_cast(actor) then
			type = "game_object"
		elseif ParticleSystem.fast_cast(actor)then
			type = "particle_system"
		elseif ActorBase.fast_cast(actor) then
			type = "actor_base"
		else
			type = "unknown"
		end
		
		local key = type .. "___" .. label .. "____p:" .. parent_name
		local value = tally[key] or 0
		tally[key] = value + 1
		
		if label == "unlabled" then
			self:tallyChildren(actor, parent_name)
		else
			self:tallyChildren(actor, label)
		end
		
	end

end
		



function LuaPauseWindow:destroy()
	Resources.get():unload(self.m_asset_group)
end
