module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local util      = chunks.util
local load      = chunks.load
local achievements = chunks.achievements

LuaPit = chunks.class.inherit(chunks.baseunit.LuaBaseUnit)
{
    m_object_count = 0,
    m_main_game = nil,
}

bindToObject = chunks.class.gameObjectBinder(LuaPit)

function LuaPit:onBindToObject(object)  
    LuaPit.super.onBindToObject(self, object)
    local actual_object = self.owner	
    
    self.m_main_game_lua = actual_object:getMainGame().lua
end


function LuaPit:throwIntoPit(object)
    local owner = self.owner
	print(object.label, "was thrown into pit")

    local odds = 1 --load.getStat("units", owner.label, "odds")
    local r = math.random()
    if r < odds then
        
        local pit_loc = owner:getWorldLocation()
        local object_loc = object:getWorldLocation()
        local range = 150 --load.getStat("units", owner.label, "range")
        local dist_x = pit_loc.x - object_loc.x
        if math.abs(dist_x) < range then
            local render = PartRenderIndex(object:getObjectIndex()):get()
            if render then
                local actor = render.render_actor
                local loc = actor:getWorldLocation()
                object.location = loc
                object_loc = loc
                actor.location = pool.get_v3f(0, 0, 0)
                --object_loc = actor:getWorldLocation()
                --actor:addAction(MoveAction(0.5, InterpAction.Lerp, pool.get_v3f(0, 0, 0)))
                actor:clearActions()
                object:clearActions()
            end

            local pit_render = PartRenderIndex(self.owner:getObjectIndex()):get()
            local sprite_actor = SpriteActor.fast_cast(pit_render.render_actor)

            local height = 75
			local mid_x = (pit_loc.x + object_loc.x) * 0.5
			local mid_point = v3f(mid_x, object_loc.y + height, self.owner.location.z-5) -- minus 5 because the put should be 300 and the opening should be 290

            local first_offset = mid_point - object_loc
			local second_offset = (pit_loc - pool.get_v3f(0, sprite_actor.size.y*0.5, 0)) - mid_point

			local rotate_direction = 1
			if (object_loc.x < pit_loc.x) then
			   rotate_direction = -1 
			end
			
			local rotation_amount = 90 * math.random(1, 3)
			local full_anim = SplitAction()
			local full_time = 0.9
			local up_time = 0.55
			local down_time = full_time - up_time

			full_anim:addAction(RotateAction(full_time, InterpAction.Smooth, v3f( 0.0, 0.0, rotation_amount * rotate_direction)))
			full_anim:addAction(MoveOffsetAction(full_time, InterpAction.Smooth, pool.get_v3f(pit_loc.x - object_loc.x, 0.0, 0.0)))
			local up_and_down = SequenceAction() 

			local first_split = SplitAction()
			first_split:addAction(MoveOffsetAction(up_time, InterpAction.Fall, pool.get_v3f(0.0, first_offset.y, first_offset.z)))
			up_and_down:addAction(first_split)
	
			object:addAction(LuaAction(0.0, function() object.team = GameObject.TEAM_NEUTRAL; self:incrementCount() end ))
	
			local second_split = SplitAction()
			second_split:addAction(MoveOffsetAction(down_time, InterpAction.Ramp, pool.get_v3f(0.0, second_offset.y, 0.0)))
			second_split:addAction(ScaleAction(down_time, InterpAction.Ramp, pool.get_v3f(0.3, 0.3, 0.3)))
			up_and_down:addAction(second_split)

			full_anim:addAction(up_and_down)
			object:addAction(full_anim)
			
            --object:addAction(DestroyAction(0))
            object:addAction(LuaAction(0.0, function()
                object.label="";
                object.color = pool.get_colorf(0,0,0,0);
                object.team = GameObject.TEAM_ENEMY;
                object:die();
                self:decrementCount();
                end ))
			
			local orders = PartOrdersIndex(object:getObjectIndex()):get()
			if orders then
				orders:addOrder(KnockbackOrder(0, 2.0), PartOrders.QUEUE_FIRST)
			end
            
            achievements.unitKnockedIntoPit(object.label)
        end

    end
end

function LuaPit:incrementCount()
    if self.m_object_count == 0 then
        self.m_main_game_lua.m_pit_closed.hidden = true
        self.m_main_game_lua.m_pit_open.hidden = false
    end
    self.m_object_count = self.m_object_count + 1
end

function LuaPit:decrementCount()
    self.m_object_count = self.m_object_count - 1
    if self.m_object_count == 0 then
        self.m_main_game_lua.m_pit_closed.hidden = false
        self.m_main_game_lua.m_pit_open.hidden = true
    end

end

