-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local pool      = chunks.pool

-- LuaPopup --
LuaPopup = chunks.class.new
{
	-----------
	-- logic --
	-----------
	
	m_pause_game = false,
	
	-------------
	-- visuals --
	-------------
	
	m_popup = nil,
	
	---------------------
	-- visual settings --
	---------------------
	
	m_transition_time = 0.2
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaPopup )

-- The created method.
function LuaPopup:created( owner )
	local game = owner:getGame()
	
	-- Settings.
	owner.size     = constants.main_menu_camera.window.world_size
	owner.color    = pool.get_colorf( 0.0, 0.0, 0.0, 0.5 )
	owner.location = pool.get_v3f( 0.0, 0.0, 10.0 )
	
	-- Spawn the popup.
	do
		local actor = owner:spawn( Window() )
		
		actor.force_color = true
		actor.location    = pool.get_v3f( 0.0, 0.0, 0.0 )
		self.m_popup      = actor
	end
end

-- Transition in.
function LuaPopup:transitionIn()
	local popup = self.m_popup
	local game  = popup:getGame()
	local s     = SequenceAction()
	local split = SplitAction()
	local s1    = SequenceAction()
	local s2    = SequenceAction()
	
	-- Setup.
	s:addAction( LuaAction( 0.0, function( inner ) 
		-- Pause game?
		if self.m_pause_game then
			game.paused = true
		end
	end ) )
	
	-- Rotation.
	--[[s1:addAction( RotateOffsetAction( 0.0, InterpAction.Smooth, 
	              pool.get_v3f( 0.0, 0.0, -180.0 ) ) )
	s1:addAction( RotateOffsetAction( self.m_transition_time, InterpAction.Smooth, 
	              pool.get_v3f( 0.0, 0.0, 180.0 ) ) )]]
	
	-- Scaling.
	s2:addAction( ScaleAction( 0.0, InterpAction.Smooth,
	              pool.get_v3f( 0.0, 0.0, 1.0 ) ) )
	s2:addAction( ScaleAction( self.m_transition_time, InterpAction.Smooth,
	              pool.get_v3f( 1.0, 1.0, 1.0 ) ) )
	              
	split:addAction( s1 )
	split:addAction( s2 )
	s:addAction( split )
	             
	popup:addAction( s )
end

-- Transition out.
function LuaPopup:transitionOut( callback )
	local owner = self.owner
	local popup = self.m_popup
	local game  = popup:getGame()
	local s     = SequenceAction()
	local split = SplitAction()
	local s1    = SequenceAction()
	local s2    = SequenceAction()
	
	-- Rotation.
	--[[s1:addAction( RotateOffsetAction( self.m_transition_time, InterpAction.Smooth, 
	              pool.get_v3f( 0.0, 0.0, -180.0 ) ) )]]
	
	-- Scaling.
	s2:addAction( ScaleAction( self.m_transition_time, InterpAction.Smooth,
	              pool.get_v3f( 0.0, 0.0, 1.0 ) ) )
	              
	split:addAction( s1 )
	split:addAction( s2 )
	s:addAction( split )
	
	-- Teardown.
	s:addAction( LuaAction( 0.0, function( inner ) 
		-- Unpause game?
		if self.m_pause_game then
			game.paused = true
		end
		
		-- Callback.
		if callback then
			callback()
		end
		
		owner:destroy()
	end ) )
	 
	popup:addAction( s )
end