------------------------
-- Kevin Depue (2010) --
-- Backflip Studios   --
--                    --
-- Sounds             --
------------------------

-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local assets    = chunks.assets
local spool     = Audio.get():createSoundPool()

-- Function that generates a getMesh method.
function get_sound_spawner( root_path )
	return function( sound )
		return "?://" .. root_path .. sound.path
	end
end



function play_sound_spawner( root_path )
	return function( sound )
		return spool:play("?://" .. root_path .. sound.path, false)
	end
end

function play_looping_sound_spawner( root_path )
	return function( sound )
		local handle = spool:play("?://" .. root_path .. sound.path, false)
		if handle then
			handle.loop_count = -1
		end
		
		return handle
	end
end

-- Precache a sound.
function precache( sound )
   if sound.path then
      spool:precache( sound.path )
   end
end

-- Play a sound.
function play( sound, paused, options )

	local channel = nil


	if sound then
		channel = spool:play( sound, true )
	end


	if channel then
		-- Loops?
		if options and options.loops then
			channel.loop_count = -1
		end

		-- Volume?
		if options and options.volume then
			channel.volume = sound.volume
		end

		-- Pitch?
		if options and options.pitch then
			channel.frequency = utility.frandom( sound.pitch.min, sound.pitch.max ) * channel.frequency
		end

		-- Unpause.
		channel.paused = false
	end

end

function pauseSound( sound )
	spool:play(sound, false)
end
