module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local util      = chunks.util

LuaSuperBoomstickWeapon = chunks.class.new
{
}

bindToObject = chunks.class.gameObjectBinder(LuaSuperBoomstickWeapon)

-- Callbacks from C++
function LuaSuperBoomstickWeapon:onWeaponFired(weapon_part, target)	
	local game = self.owner:getGame()
	local scene = game:getScene()
	local location = self.owner:getWorldLocation()
	location.z = 400
	location.y = location.y + 4.5
	
	local muzzle_flash = scene:spawn(SpriteActor())
	assets.setTexture(muzzle_flash, assets.textures.muzzle_flash)
	muzzle_flash.location = location + pool.get_v3f(0.0, 0.0, 1.0)
	muzzle_flash.scale = pool.get_v3f(0.0, 0.0, 1.0)
	
	do  -- Muzzle flash sequence
		local fade_time = 0.15
		local scale_offset = v3f(3.5, 3.5, 0.0)
		local sequence = SequenceAction()
		local scale_in_split = SplitAction()
		
		scale_in_split:addAction(ScaleOffsetAction(fade_time, InterpAction.Lerp, pool.get_v3f(0.0, scale_offset.y, 0.0)))
		scale_in_split:addAction(ScaleOffsetAction(fade_time, InterpAction.Lerp, pool.get_v3f(scale_offset.x, 0.0, 0.0)))
		scale_in_split:addAction(MoveOffsetAction(fade_time, InterpAction.Lerp, pool.get_v3f((muzzle_flash.size.x * 0.5) * scale_offset.x, 0.0, 0.0)))
		sequence:addAction(scale_in_split)
		
		local scale_out_split = SplitAction()
		scale_out_split:addAction(ScaleOffsetAction(fade_time, InterpAction.Lerp, pool.get_v3f(0.0, -scale_offset.y, 0.0)))
		scale_out_split:addAction(ScaleOffsetAction(fade_time, InterpAction.Lerp, pool.get_v3f(-scale_offset.x, 0.0, 0.0)))
		scale_out_split:addAction(MoveOffsetAction(fade_time, InterpAction.Lerp, pool.get_v3f((muzzle_flash.size.x * -0.5) * scale_offset.x, 0.0, 0.0)))	
		scale_out_split:addAction(AlphaAction(fade_time, InterpAction.Lerp, 0.0))
		sequence:addAction(scale_out_split)
		sequence:addAction(DestroyAction(0.0))
		
		muzzle_flash:addAction(sequence)
	end
	
	do -- Smoke puff sequence
		local fade_in_time = 0.15
		local smoke_puff = scene:spawn(SpriteActor())
		
		-- This effect has a locale version because the texture says 'BOOM' on the English version
		util.setLocaleTexture(smoke_puff, assets.textures.boom_stick_blast_effect_locale, assets.textures.boom_stick_blast_effect)
		
		if not engine.m_default_language_used then
			local text_options = 	{
										height = 70,
									  	max_width = 130,
									  	align = TextAlign.Center,
									  	outline_color = pool.get_colorf(1,1,1,1),
									  	font_color = colorf(0,0,0,1)
								 	}
			util.createText(smoke_puff, "super_boom_effect", "Gothic_Large", text_options, {x = -12.0, y = 6.0})
		end

		smoke_puff.location = location + pool.get_v3f(0.0, 0.0, 0.0)
		smoke_puff.scale = pool.get_v3f(0.0, 0.0, 1.0)
		
		local sequence = SequenceAction()
		local scale_in_split = SplitAction()
		
		scale_in_split:addAction(ScaleOffsetAction(fade_in_time, InterpAction.Lerp, pool.get_v3f(1.0, 1.0, 0.0)))
		scale_in_split:addAction(MoveOffsetAction(fade_in_time, InterpAction.Lerp, pool.get_v3f(smoke_puff.size.x * 0.5, 0.0, 0.0)))
		scale_in_split:addAction(AlphaAction(fade_in_time, InterpAction.Lerp, 0.75))
		sequence:addAction(scale_in_split)
		
		local fade_out_time = fade_in_time * 8
		local scale_out_split = SplitAction()
		scale_out_split:addAction(AlphaAction(fade_out_time, InterpAction.Lerp, 0.0))
		sequence:addAction(scale_out_split)
		sequence:addAction(DestroyAction(0.0))		
		
		smoke_puff:addAction(sequence)
	end
end
