module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local util      = chunks.util
local load      = chunks.load
local achievements = chunks.achievements
local assets = chunks.assets

LuaTorchSwordboy = chunks.class.inherit(chunks.baseunit.LuaBaseUnit)
{
	m_previous_victims,
	m_torch_boy = false,
	m_sword_boy = false,
	m_flute_skeleton = false,
	m_range = 0,
	m_has_ever_been_unblocked = false,
}

bindToObject = chunks.class.gameObjectBinder(LuaTorchSwordboy)

function LuaTorchSwordboy:onBindToObject(object)	
	LuaTorchSwordboy.super.onBindToObject(self, object)	
	local actual_object = self.owner
	
	self.m_previous_victims = {}
	
	
	--print("actual_object.label="..actual_object.label)
	
	local label = actual_object.label:lower()
	
	--print("label="..label)
	
	self.m_torch_boy = label:find("torch")
	self.m_sword_boy = label:find("sword")
	self.m_flute_skeleton = label:find("flute") or label:find("drummer")
	
	--print(type(self.m_torch_boy))
	--print(type(self.m_sword_boy))
	
	--print((self.m_torch_boy))
	--print((self.m_sword_boy))
	
	local inventory = PartInventoryIndex(object):get()
	if not inventory then
		print("TorchSword boy couldn't get his own inventory")
		return
	end
	
	local item = inventory:getEquippedItemAtSlot(PartInventory.WEAPON_EQUIP_SLOT):get()
	if not item then
		print("TorchSwordboy could not get his own weapon item")
		return
	end


	local weapon = PartWeaponIndex(item:getObjectIndex()):get()
	if not weapon then
		print("TorchSwordboy could not get his own weapon weapon")
		return
	end
	
	self.m_range = weapon.range
	
	
	
	if self.m_flute_skeleton then
		local loop_actor = actual_object:spawn(ActorBase())
		local loop = LoopAction()
		loop:addAction(LuaAction(5.0, function()
			util.showHealSign(actual_object, chunks.assets.textures.note, 4.0)
		end ))
		loop_actor:addAction(loop)
	end
end

-- Callbacks from C++
function LuaTorchSwordboy:processPartLogic(part, dt)
	self.m_has_ever_been_unblocked = true
	self:process(part, dt)
end

function LuaTorchSwordboy:blockedPartLogic(part, dt)
	if self.m_has_ever_been_unblocked then
		self:process(part, dt)
	else
        part:get():requestProcess(0.2)
    end
end

function LuaTorchSwordboy:process(part, dt)
	local object_index = part:get():getObjectIndex()
	local object = object_index:get()
	local orders = PartOrdersIndex(object_index)
	
	
	
	local range = self.m_range
	local victims = object:getMainGame():getObjectsInRange(object.team, object.location.x-range, object.location.x+range).values
	for i,enemy_index in ipairs(victims) do
		local enemy = enemy_index:get()
		if enemy and not self.m_previous_victims[enemy.id] then
			self.m_previous_victims[enemy.id] = true
			if self.m_torch_boy then
				self:torchboy(enemy)
			elseif self.m_sword_boy then
				self:swordboy(enemy)
			elseif self.m_flute_skeleton then
				self:fluteSkeleton(enemy)
			else
				print("torch_sword_boy.lua: The label is: "..object.label)
			end
		end --if enemy, previous_victims
		
	end --for loop
	
	
	
	if not orders:get():hasOrders() then
        local attack_object = object:getMainGame():getClosestObjectToAttack(object:getOppositeTeam(), object.location.x)	
        orders:get():addOrder(AttackOrder(attack_object), PartOrders.QUEUE_END)
	end

	part:get():requestProcess(0.2)	

end

function LuaTorchSwordboy:swordboy(target)
	--print("SWORDBOY IS DOING HIS THING TO:"..target.id)
	
	local new_weapon_name = chunks.load.getStat("units", target.label, "sword_boy_weapon")
	if not new_weapon_name then
		--print("the unit " .. target.label .. " does not have a sword_boy_weapon listed")
		return
	end
	
	local new_weapon = target:getGame():createGameObject("weapon", new_weapon_name, target:getLevel()):get()
	if not new_weapon then
		print("Sword boy encountered an error when creating " .. new_weapon_name)
		return
	end
	
	local new_item_index = PartItemIndex(new_weapon:getObjectIndex())
	if not new_item_index:get() then
		print("sword boy: new weapon doesn't have an item part")
		return
	end
	
	local inventory = PartInventoryIndex(target:getObjectIndex()):get()
	if not inventory then
		print("sword boy couldn't get inventory of "..target.label)
		return
	end
	
	local item_index = inventory:getEquippedItemAtSlot(PartInventory.WEAPON_EQUIP_SLOT)
	if item_index:get() then
		inventory:unequipItem(item_index)
	end
	
	inventory:equipItem(new_item_index)
	
	--target.scale = pool.get_v3f(2, 2, 1)

	util.showBubbleSprite(target, assets.textures.buff_sword_good, 60)
	
	achievements.swordBoyUpgradedUnit(target.label)
end

function LuaTorchSwordboy:torchboy(target)
	--print("TORCHBOY IS DOING HIS THING TO:"..target.id)
	
	local inventory = PartInventoryIndex(target:getObjectIndex()):get()
	if not inventory then
		print("Torch boy couldn't get inventory of "..target.label)
		return
	end
	
	local item_index = inventory:getEquippedItemAtSlot(PartInventory.WEAPON_EQUIP_SLOT)
	item = item_index:get()
	
	if not item then
		print("Torch boy: " .. target.label .. " does not have a weapon to enflame")
		return
	end
	
	local old_weapon_name = item:getObjectIndex():get().label
	local new_weapon_name = "flaming_" .. old_weapon_name
	
	local new_weapon = target:getGame():createGameObject("weapon", new_weapon_name, target:getLevel()):get()
	if not new_weapon then
		--print("Torch boy encountered an error when creating " .. new_weapon_name)
		return
	end
	
	local new_item_index = PartItemIndex(new_weapon:getObjectIndex())
	if not new_item_index:get() then
		print("torch boy: new weapon doesn't have an item part")
		return
	end
	
	inventory:unequipItem(item_index)
	
	inventory:equipItem(new_item_index)
	

	--local old_damage = self:getWeaponDamage(old_weapon_name, target:getLevel())
	--local new_damage = self:getWeaponDamage(new_weapon_name, target:getLevel())
	--util.showBubbleText(target, "+"..(new_damage-old_damage), 80)
	
	util.showBubbleSprite(target, assets.textures.enflamed_effect, 80, 0.4)
	
	achievements.torchBoyUpgradedUnit(target.label)
end

function LuaTorchSwordboy:fluteSkeleton(target)
	--print("FLUTE SKELETON IS DOING HIS THING TO:"..target.id)
	
	local inventory = PartInventoryIndex(target:getObjectIndex()):get()
	if not inventory then
		print("Flute Skeleton couldn't get inventory of "..target.label)
		return
	end
	
	local item_index = inventory:getEquippedItemAtSlot(PartInventory.WEAPON_EQUIP_SLOT)
	item = item_index:get()
	
	if not item then
		print("Flute Skeleton: " .. target.label .. " does not have a weapon to buff")
		return
	end
	
	local weapon = PartWeaponIndex(item:getObjectIndex()):get()
	
	if not weapon then
		print("Flute Skeleton: " .. target.label .. " has a non-weapon on their weapon slot ")
		return
	end
	
	local damage_buff_factor = chunks.load.getStat("units", self.owner.label, "damage_buff_factor")
	weapon:setDamageBuff(damage_buff_factor, self.owner:getObjectIndex())

	--target.scale = pool.get_v3f(0.5,2,1)
	
	--local old_damage = self:getWeaponDamage(weapon:getObjectIndex():get().label, target:getLevel())
	--local new_damage = old_damage * damage_buff_factor
	--util.showBubbleText(target, "+"..math.floor(new_damage-old_damage+0.5))
	
	--damage buff sprite
	do
		local renderer = PartRenderIndex(target:getObjectIndex()):get()
		if not renderer then
			return
		end
		scale = scale or 1.0
		
		local asset = assets.textures.buff_sword_evil
		local render = renderer.render_actor
		local sprite = render:spawn(SpriteActor())
		assets.setTexture(sprite, asset)
		sprite.label = "damage_buff"
		sprite.location = pool.get_v3f(0, scale*asset.size.y/2, -10)
	
		local length = 1.3
		local height = (chunks.load.getHeight("units", target.label) or 55) + 10
		
		sprite.scale = pool.get_v3f(scale, scale, scale)
	
		sprite:addAction(MoveOffsetAction(length*0.5, InterpAction.Fall, pool.get_v3f(0, height, 0)))
	
		sprite:addAction(ScaleAction(length*0.25, InterpAction.Smooth, pool.get_v3f(scale, scale, scale)*2.0 ))
		sprite:addAction(ScaleAction(length*0.25, InterpAction.Smooth, pool.get_v3f(scale, scale, scale)))

		local loop = LoopAction()
			loop:addAction(ScaleAction(length*0.25, InterpAction.Smooth, pool.get_v3f(scale, scale, scale)*1.25 ))
			loop:addAction(ScaleAction(length*0.25, InterpAction.Smooth, pool.get_v3f(scale, scale, scale)*0.75 ))
		sprite:addAction(loop)

	
		--sprite:addAction(DestroyAction(0))
		
		
	end
end

function LuaTorchSwordboy:destroy(owner)
	local cooldown_button = owner:getGame().lua.m_buttons_unique[owner.label]
	if cooldown_button then
		owner:getGame().lua.m_buttons_unique[owner.label] = nil
		cooldown_button:makeReady()
	end
end

function LuaTorchSwordboy:onKnockedBack(distance)
    local owner = self.owner
    local game = owner:getMainGame()
    local pit = game.lua.m_pit
    if pit and pit:get() and owner.team == GameObject.TEAM_ENEMY then
        pit:get().lua:throwIntoPit(owner)
    end
end

function LuaTorchSwordboy:onDeath(owner)
	LuaTorchSwordboy.super.onDeath(self, owner)

	if owner.team == GameObject.TEAM_ENEMY then
		util.tryToSpawnCoin(owner)
		
		local bonus = load.getStat( "units", owner.label, "ash_kill_bonus", owner:getLevel() ) or 0
		local bonus = math.floor( bonus * 0.1 + 0.5 )
		self.owner:getMainGame().lua:addGold(bonus)
		--print("AWARDED ", bonus)
		
		if self.m_flute_skeleton then
			self:removeDamageBuffSprites()
		end
	end
end

function LuaTorchSwordboy:getWeaponDamage(weapon_name, level)
	local damage
	local ammo_name = load.getStat("weapons", weapon_name, "ammo", level)
	if ammo_name then
		damage = load.getStat("ammo", ammo_name, "damage", level)
	else
		damage = load.getStat("weapons", weapon_name, "damage", level)
	end
	return damage
end

function LuaTorchSwordboy:removeDamageBuffSprites()
	local owner = self.owner
	local game = owner:getMainGame()
	local enemies = game:getObjectsInRange(GameObject.TEAM_ENEMY, -1000000, 1000000).values
	for _,object_index in ipairs(enemies) do
		local render = PartRenderIndex(object_index):get()
		if render then
			local actor = render.render_actor
			local size = actor:getNumOwned()
			for i=0,size-1 do
				local child = actor:getOwned(i)
				print(child.id, child.label)
				if child.label == "damage_buff" then
					child:clearActions()
					child:addAction(ScaleAction(0.4, InterpAction.Lerp, pool.get_v3f(0, 0, 0)))
					child:addAction(DestroyAction(0))
					break
				end
			end
		end
	end
end