module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local util      = chunks.util
local assets    = chunks.assets
local load      = chunks.load
local achievements = chunks.achievements
local globals	= chunks.globals

local playRootSound = assets.playSound
local stat_string = lstring("stat")

LuaTroopCell = chunks.class.new
{
	m_name = nil,
	m_available = false,
	m_level_label = nil,
	m_attack_label = nil,
	m_health_label = nil,
	m_upgrade_button = nil,
	m_cost_label = nil,
	m_gold_icon = nil,
}

-- The spawner.
spawn = chunks.class.spawner( Button, LuaTroopCell )

function LuaTroopCell:created( owner )

	assets.setTexture(owner, assets.textures.upgrade_box)

	owner.on_enter = function(owner) owner.color = pool.get_colorf(0.5, 0.5, 0.5, 1.0) end
	owner.on_exit  = function(owner) owner.color = pool.get_colorf(1.0, 1.0, 1.0, 1.0) end

end


function LuaTroopCell:setTroop(name, unlock_wave, current_wave)
	local owner = self.owner
	self.m_name = name

	local function purchase()
		local user_data_instance = UserData.instance()
		local level = user_data_instance:getTypeLevel(self.m_name)
		local upgrade_cost = load.getStat("units", name, "cost", level + 1)
		local gold = user_data_instance:getGold()
		if gold >= upgrade_cost then
			user_data_instance:setTypeLevel(self.m_name, level + 1)
			user_data_instance:setGold( gold - upgrade_cost)
			self:updateText()
			self.m_main_window.lua:updateText()
			playRootSound(assets.sounds.upgrade)
			achievements.typeUpgraded("units", self.m_name)
			achievements.goldChanged()
		else
			playRootSound(assets.sounds.error)
		end
	end

	local function unpurchase()
		local user_data_instance = UserData.instance()
		local level = user_data_instance:getTypeLevel(self.m_name)
		local upgrade_cost = load.getStat("units", name, "cost", level) * globals.downgrade_gold_return
		local gold = user_data_instance:getGold()
		if level > 1 then
			user_data_instance:setTypeLevel(self.m_name, level - 1)
			user_data_instance:setGold( gold + math.floor(upgrade_cost))
			self:updateText()
			self.m_main_window.lua:updateText()
			playRootSound(assets.sounds.upgrade)
			achievements.typeUpgraded("units", self.m_name)
			achievements.goldChanged()
		else
			playRootSound(assets.sounds.error)
		end
	end

	owner.on_release = function(owner)
		assets.playSound(assets.sounds.button_click_wav)
		local info_window = self.m_main_window:spawn( chunks.infowindow.spawn() )
		info_window.lua:setNameAndChunkName(self.m_name, "units", purchase, unpurchase)
	end


	--create Icon
	do
		local available = current_wave >= unlock_wave
		self.m_available = available

		local icon_asset = assets.findTextureByPathName( chunks.load.getStat("units", name, "icon_texture") ) or assets.textures.icon_generic
		if not available then
			icon_asset = assets.textures.icon_locked
		end

		local icon = util.createWindowWithTexture(owner, icon_asset)
		icon.anchor = {Anchor.Anchor_L, Anchor.Anchor_L, pool.get_v3f(7.0, 0.0, 1.0)}

		-- create Title
		do
			local title_label = icon:spawn(Text())
			local format = {"Georgia Bold", 18.0+2, 150.0}
			title_label.style = format[1]
			title_label.options = {height = format[2], max_width = format[3]}
			title_label.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TR, pool.get_v3f(45.0, -5.0+5, 1.0)}
			title_label.text = lstring(name .. "_title") -- load.getStat("units", name, "title")
		end

		if available then

			-- create Level Label
			do
				local level_label = icon:spawn(Text())
				level_label.style = "Georgia Bold"
				level_label.options = {height = 20.0, max_width = 150}
				level_label.anchor = {Anchor.Anchor_BL, Anchor.Anchor_BR, pool.get_v3f(45.0, 10.0, 1.0)}
				self.m_level_label = level_label
			end


			-- create Iron Cost Label
			if load.getStat("units", name, "iron_cost") then
				local iron_label = icon:spawn(Text())
				local format = {"Helvetica-Bold", 14.0}
				iron_label.style = format[1]
				iron_label.options = {height = format[2]}
				iron_label.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(13,-19+4,1)}
				stat_string:arg(1, tostring( load.getStat("units", name, "iron_cost") ))
				iron_label.text = stat_string
			end

			--create Attack Icon
			if load.getStat("units", name, "weapon") then
				local attack = nil
				if self:getDamage() < 0 then
					attack = util.createWindowWithTexture(icon, assets.textures.heal_icon)
				else
					attack = util.createWindowWithTexture(icon, assets.textures.attack_icon)
				end
				attack.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TR, pool.get_v3f(-2, -7.5, 1.0)}

				-- create Attack Label
				do
					local attack_label = icon:spawn(Text())
					local format = {"Helvetica-Bold", 12.0+1}
					attack_label.style = format[1]
					attack_label.options = {height = format[2]}
					attack_label.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TR, pool.get_v3f(12, -7.5+3, 1.0)}
					--attack_label.text = "LEVEL " .. UserData.instance():getTypeLevel(name)
					self.m_attack_label = attack_label
				end
			end

			--create Health Icon
			if load.getStat("units", name, "health") then
				local health = util.createWindowWithTexture(icon, assets.textures.health_icon)
				health.anchor = {Anchor.Anchor_BL, Anchor.Anchor_BR, pool.get_v3f(-2, 7.5, 1.0)}

				-- create Health Label
				do
					local health_label = icon:spawn(Text())
					local format = {"Helvetica-Bold", 12.0+1}
					health_label.style = format[1]
					health_label.options = {height = format[2]}
					health_label.anchor = {Anchor.Anchor_BL, Anchor.Anchor_BR, pool.get_v3f(12, 7.5+3, 1.0)}
					--health_label.text = "LEVEL " .. UserData.instance():getTypeLevel(name)
					self.m_health_label = health_label
				end
			end

			--create Upgrade Button
			if load.hasNextLevel("units", name) then
				local upgrade_button = util.createButtonWithTexture(owner, assets.textures.upgrade_button_blank)
				upgrade_button.anchor = {Anchor.Anchor_R, Anchor.Anchor_R, pool.get_v3f(-7.0, 0, 1.0)}
				self.m_upgrade_button = upgrade_button

				upgrade_button.on_release = function(owner)
					assets.playSound(assets.sounds.button_click_wav)
					local info_window = self.m_main_window:spawn( chunks.infowindow.spawn() )
					info_window.lua:setNameAndChunkName(self.m_name, "units", purchase, unpurchase)
				end

				--create Next label
				do
					local next_label = upgrade_button:spawn(Text())
					local format = {"Helvetica-Bold", 10.0+3, 50.0, TextAlign.Center}
					next_label.style = format[1]
					next_label.options = {height = format[2], max_width = format[3], align = format[4]}
					next_label.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0.0, 7.5+3, 1.0)}
					next_label.text = lstring("UPGRADE")
				end

				--Create cost window
				do
					local cost_window = upgrade_button:spawn(Window())
					cost_window.anchor = {Anchor.Anchor_B, Anchor.Anchor_B, pool.get_v3f(0-1, 10, 1)}
					cost_window.hidden = true

					--create Gold Icon
					local gold_icon = util.createWindowWithTexture(cost_window, assets.textures.upgrade_gold_coin)
					gold_icon.scale = pool.get_v3f(0.5, 0.5, 1)
					gold_icon.anchor = {Anchor.Anchor_L, Anchor.Anchor_L, pool.get_v3f(0, 0, 1)}
					self.m_gold_icon = gold_icon

					--create Cost Label
					local cost_label = cost_window:spawn(Text())
					local format = {"Helvetica-Bold", 16.0+2}
					cost_label.style = format[1]
					cost_label.options = {height = format[2]}
					cost_label.anchor = {Anchor.Anchor_R, Anchor.Anchor_R, pool.get_v3f(0, 0+4, 1)}
					self.m_cost_label = cost_label
				end



			end  --if has next level

			self:updateText()

		else -- if available
			local unlocked_after = owner:spawn(Text())
			local format = {"Georgia Bold", 16.0+2, 220.0}
			unlocked_after.style = format[1]
			unlocked_after.options = {height = format[2], max_width = format[3]}
			unlocked_after.anchor = {Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f(109, 8+4, 1.0)}
			local unlocked_at = lstring("unlocked_at")
			unlocked_at:arg(1, unlock_wave)
			unlocked_after.text = unlocked_at

		end

	end



end

function LuaTroopCell:getDamage()
	local name = self.m_name
	local level = UserData.instance():getTypeLevel(name)
	local damage = 0

	local weapon_name = load.getStat("units", name, "weapon", level)
	local dps = load.getStat("weapons", weapon_name, "dps", level)
	if dps then
		damage = dps
	else
		local ammo_name = load.getStat("weapons", weapon_name, "ammo", level)
		if ammo_name then
			damage = load.getStat("ammo", ammo_name, "damage", level)
		else
			damage = load.getStat("weapons", weapon_name, "damage", level)
		end
	end

	return damage
end

function LuaTroopCell:updateText()
	if not self.m_available then
		return
	end


	local name = self.m_name
	local level = UserData.instance():getTypeLevel(name)
	local has_next_level = load.hasNextLevel("units", name)

	if has_next_level then
		label_text = lstring("level_caps")
		label_text:arg(1, tostring( level ))
		self.m_level_label.text = label_text
	else
		label_text = lstring("level_caps")
		label_text:arg(1, lstring("MAX"))
		self.m_level_label.text = label_text
	end

	if self.m_attack_label then
		local damage = self:getDamage()
		local damage_string = tostring(math.abs(damage))
		stat_string:arg(1, damage_string)
		self.m_attack_label.text = stat_string
	end

	if self.m_health_label then
		local health_string = tostring( load.getStat("units", name, "health", level) )
		stat_string:arg(1, health_string)
		self.m_health_label.text = stat_string
	end

	if self.m_upgrade_button and not has_next_level then
		self.m_upgrade_button.hidden_recursive = true
		self.m_upgrade_button.receive_input = false
	end

	if self.m_upgrade_button and self.m_cost_label then
		local cost_string = tostring( load.getStat("units", name, "cost", level + 1) )
		stat_string:arg(1, cost_string)
		self.m_cost_label.text = stat_string
		
		local cost_window = self.m_cost_label:getOwner()
		local c = 38
		if self.m_cost_label.size.x > c then
			self.m_cost_label.scale = pool.get_v3f(c / self.m_cost_label.size.x, c / self.m_cost_label.size.x, 1)
		end
		cost_window.size = pool.get_v2f(self.m_gold_icon.size.x * self.m_gold_icon.scale.x + self.m_cost_label.size.x * self.m_cost_label.scale.x, math.max(self.m_gold_icon.size.y * self.m_gold_icon.scale.y, self.m_cost_label.size.y * self.m_cost_label.scale.y))
	end

end
