-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local pool      = chunks.pool

LuaUnlockedMenuGame = chunks.class.new
{
	m_asset_group = nil,
}

-- The spawner.
spawn = chunks.class.spawner( SingleTouchWindowGame, LuaUnlockedMenuGame )

function LuaUnlockedMenuGame:created(game)
	print("LuaUnlockedMenuGame:created")
	
	local game_window = game:getWindow()
	
	-- Set clearing info.
		-- Window camera.
	do
		local camera = game:getWindow():spawn( OrthographicCamera() )

		camera.viewport_offset = constants.main_menu_camera.window.viewport_offset
		camera.viewport_size   = constants.main_menu_camera.window.viewport_size
		camera.screen_size     = constants.main_menu_camera.window.screen_size
		camera.world_size      = constants.main_menu_camera.window.world_size
		camera.clipping_planes = constants.main_menu_camera.window.clipping_planes
		camera.clear_color     = constants.clear_color
		camera.clear_color_on  = constants.clear_color_on
		camera.clear_depth_on  = constants.clear_depth_on
		camera.scene           = game_window

		game_window.camera = camera
	end
	
	self.m_asset_group = ResourceGroup("texture")
	self:preloadAssets()
	
	Resources.get():addResourceGroup(self.m_asset_group)
	game_window:spawn(chunks.unlocked_menu_window.spawn())
	Resources.get():removeResourceGroup(self.m_asset_group)
	
	 -- There is no way to get out of this menu except for going forward so just remove if from the stack so we can't come back to it
	engine:removeTopGameFromSpawnStack() 
end


function LuaUnlockedMenuGame:destroy()
	self:unloadAssets()
end


function LuaUnlockedMenuGame:preloadAssets()
	--TODO: FIXME local sound_manager = Audio.get().sound
	
	-- sounds
	--sound_manager:precache(URI(assets.getSound(assets.sounds.button_click)))
end

function LuaUnlockedMenuGame:unloadAssets()
	Resources.get():unload(self.m_asset_group)
end
