-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local pool      = chunks.pool
local util      = chunks.util
local load      = chunks.load

local stat_string = lstring("stat")

LuaUnlockedMenuWindow = chunks.class.new
{
	m_unlocked_set = nil,
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaUnlockedMenuWindow )

function LuaUnlockedMenuWindow:created(window)

	self:setUnlockedSet(getUnlockedSet())
end

function LuaUnlockedMenuWindow:setUnlockedSet(set)
	local owner = self.owner

	owner.color = pool.get_colorf(0, 0, 0, 1)
	owner.size = constants.main_menu_camera.window.world_size

	--Settings
	local padding_size = 7
	local border_height = 21
	local border_width = 7
	local unlocked_font = "Georgia Bold"
	local unlocked_font_size = 26
	local iron_cost_font = "Helvetica-Bold"
	local iron_cost_font_size = 14
	local title_font = "Georgia Bold"
	local title_font_size = 28
	local subtitle_font = "Georgia Bold"
	local subtitle_font_size = 16
	local description_font = "Georgia Bold"
	local description_font_size = 18
	local description_font_color = colorf(1, 0.92, 0.55, 1)
	local must_buy_font_color = colorf(1, 0.25, 0.25, 1)
	local flavor_font = "Helvetica Bold Oblique"
	local flavor_font_size = 14

	--"You Unlocked" label
	local unlocked_text = owner:spawn(Text())
	unlocked_text.style = unlocked_font
	unlocked_text.options = {height = unlocked_font_size}
	unlocked_text.anchor = {Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0, 0+3, 0)}
	unlocked_text.text = lstring("you_unlocked") --"You Unlocked"
	unlocked_text.force_color = true
	unlocked_text:addAction(AlphaAction(0.25, InterpAction.Smooth, 1.0))

	local function showNextInSet()
		--Are we done?
		if #set < 1 then
			owner:addAction(LuaAction(0.25, function() engine:spawnUpgradeUnitMenu() end))
			owner:addAction(DestroyAction(0))
			return
		end

		local unlocked_item = table.remove(set, 1)

		--Sound effect
		util.playSoundAtCamera("?://" .. assets.sounds.unlock.path)

		--Background window
		local unlocked_window = util.createWindowWithTexture(owner, assets.textures.selection_box)
		unlocked_window.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, 4, 1)}
		unlocked_window.force_color = true

		--Icon
		local icon_name = load.getStat(unlocked_item.chunk_name, unlocked_item.name, "upgrade_icon_texture") or
			load.getStat(unlocked_item.chunk_name, unlocked_item.name, "icon_texture") or
			"UI/game/units/icon_generic"
		local icon = util.createWindowWithTexture(unlocked_window, assets.findTextureByPathName(icon_name))
		icon.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f(padding_size + border_width, -(padding_size + border_height), 1)}

		--Iron cost on icon
		local iron_cost = load.getStat("units", unlocked_item.name, "iron_cost")
		if iron_cost then
			local iron_cost_text = icon:spawn(Text())
			iron_cost_text.style = iron_cost_font
			iron_cost_text.options = {height = iron_cost_font_size}
			iron_cost_text.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(13, -15, 1)}
			stat_string:arg(1, iron_cost)
			iron_cost_text.text = stat_string
		end

		--Title
		local title_label = icon:spawn(Text())
		title_label.style = title_font
		title_label.options = {height = title_font_size, max_width = 210}
		title_label.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TR, pool.get_v3f(padding_size, 0, 1)}
		print("unlock title is " .. unlocked_item.name .. "_title")
		title_label.text = lstring(unlocked_item.name .. "_title")

		do
			local window_half_size = (unlocked_window.size.x - (border_width + padding_size) * 2) * 0.5
			local text_half_size = title_label.size.x * 0.5
			if window_half_size - (icon.size.x + padding_size) >= text_half_size then
				title_label:destroy()
				title_label = unlocked_window:spawn(Text())
				title_label.style = title_font
				title_label.options = {height = title_font_size, max_width = 210}
				title_label.anchor = {Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(icon.size.x / 2, -(border_height + padding_size)+7, 1)}
				title_label.text = lstring(unlocked_item.name .. "_title")
			end
		end

		--Subtitle
		local subtitle_label = title_label:spawn(Text())
		subtitle_label.style = subtitle_font
		subtitle_label.options = {height = subtitle_font_size, max_width = 180, align = TextAlign.Center}
		subtitle_label.anchor = {Anchor.Anchor_T, Anchor.Anchor_B, pool.get_v3f(0, 0, 1)}
		local subtitle_label_text = unlocked_item.is_spell and lstring("skill") or lstring(load.getStat(unlocked_item.chunk_name, unlocked_item.name, "unit_type"))
		subtitle_label.text = subtitle_label_text
		--Description
		local description_text = unlocked_item.name .. "_description" --load.getStat(unlocked_item.chunk_name, unlocked_item.name, "description") or "TODO"
		local description_label_format = {description_font, description_font_size, description_font_color}
		local description_label = util.createMultiLineLabel(unlocked_window, description_text, {description_font, description_font_size, description_font_color}, 275)
		description_label.anchor = {Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0, -(icon.size.y + padding_size * 2 + border_height), 1)}

		--'Must Buy' text
		if load.getStat(unlocked_item.chunk_name, unlocked_item.name, "unit_type") == ("Castle Upgrade") then
			local must_buy_label = description_label:spawn(Text())
			local format = {description_font, description_font_size, must_buy_font_color}
			must_buy_label.style = format[1]
			must_buy_label.options = {height = format[2], max_width = 270, align = TextAlign.Center}
			must_buy_label.text = lstring("must_buy") --"*Must buy to use"
			must_buy_label.anchor = {Anchor.Anchor_T, Anchor.Anchor_B, pool.get_v3f(0, 0, 1)}
		end

		--Flavor text
		local flavor_text = lstring(unlocked_item.name .. "_flavor") --load.getStat(unlocked_item.chunk_name, unlocked_item.name, "flavor_text")
		if flavor_text then
			local quoted_flavor_text = lstring("quotations")
			quoted_flavor_text:arg(1, flavor_text)
			local flavor_label = util.createMultiLineLabel(unlocked_window, quoted_flavor_text,{flavor_font, flavor_font_size, pool.get_colorf(1, 1, 1, 1)}, 275, 275)
			flavor_label.anchor = {Anchor.Anchor_B, Anchor.Anchor_B, pool.get_v3f(0, padding_size * 2 + border_width, 1)}
		end

		--Continue button
		local format = {"Georgia Bold", 25.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 95.0, TextAlign.Center, 0.0, 4.0, 0.0, false}
		local continue_button = util.createButtonWithDefaultOrLocale(unlocked_window, lstring("continue"), format, assets.textures.continue_button_locale, assets.textures.continue_button)

		continue_button.anchor = {Anchor.Anchor_T, Anchor.Anchor_B, pool.get_v3f(0, 16, 1)}
		continue_button.on_release = function()
			assets.playSound(assets.sounds.button_click_wav)
			unlocked_window:addAction(ScaleAction(0.25, InterpAction.Smooth, pool.get_v3f(0, 0, 1)))
			unlocked_window:addAction(DestroyAction(0))
			if #set < 1 then
				unlocked_text:addAction(AlphaAction(0.25, InterpAction.Smooth, 0.0))
			end
			owner:addAction(LuaAction(0.25, showNextInSet))
		end

		--Animate in
		unlocked_window.scale = pool.get_v3f(0, 0, 1)
		unlocked_window:addAction(ScaleAction(0.25, InterpAction.Smooth, pool.get_v3f(1, 1, 1)))
	end

	--Start the chain
	showNextInSet()
end


function getUnlockedSet()
	local current_wave = UserData.instance():getCurrentWave()

	local set = {}

	for k,v in pairs(chunks.units) do
		if v.unlock_wave and v.unlock_wave == current_wave then
			table.insert(set, {name=k, chunk_name="units"})
			--print("ADDED", k)
		end
	end

	for k,v in pairs(chunks.weapons) do
		if v.unlock_wave and v.unlock_wave == current_wave then
			table.insert(set, {name=k, chunk_name="weapons"})
			--print("ADDED", k)
		end
	end

	for k,v in pairs(chunks.spells) do
		if v.unlock_wave and v.unlock_wave == current_wave then
			table.insert(set, {name=k, chunk_name="spells", is_spell=true})
			--print("ADDED", k)
		end
	end

	return set
end
