-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local pool      = chunks.pool
local util      = chunks.util

LuaUpgradeInfoWindow = chunks.class.new
{
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaUpgradeInfoWindow )

function LuaUpgradeInfoWindow:created(window)
	window.hidden = true
	window.size = constants.main_menu_camera.window.world_size
end


function LuaUpgradeInfoWindow:setData(name, type_data)
	self.owner:destroyOwned()
	
	if name == nil or type_data == nil then
		return
	end
	
	local level = UserData.instance():getTypeLevel(name)
	local level_data = chunks.load.getLevelData(type_data, level)
	
	local window = self.owner
	
	-- Name
	local name_label = window:spawn(Label())
	name_label.format = TextFormat("font://Helvetica-Bold", 42.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0))
	name_label.text = type_data.title
	name_label.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f(0.0, 0.0, 1.0)}
	
	
	local current_level_data = chunks.load.getLevelData(type_data, level)
	local next_level_data = chunks.load.getLevelData(type_data, level + 1)
	local can_upgrade = current_level_data ~= next_level_data
	
	-- Level
	local level_label = window:spawn(Label())
	local new_level = level
	if can_upgrade then
		level_label.format = TextFormat("font://Helvetica-Bold", 16.0, pool.get_colorf(0.0, 1.0, 0.0, 1.0))
		new_level = level + 1
	else
		level_label.format = TextFormat("font://Helvetica-Bold", 16.0, pool.get_colorf(1.0, 0.0, 0.0, 1.0))
	end
	
	level_label.text = "Level " .. level
	level_label.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f(0.0, -(name_label.size.y + 5), 1.0)}
	
	if can_upgrade then
		local upgrade_button = util.createButtonWithText(window, "+ Upgrade To Level " .. new_level)
		upgrade_button.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f(5.0, -80, 1.0)}
		upgrade_button.on_release = function(button)
			 UserData.instance():setTypeLevel(name, new_level)
			 self:setData(name, type_data)
		end		
	end
	
	-- TODO: Remove this
	if level > 1 then
		local downgrade_button = util.createButtonWithText(window, "- Downgrade To Level " .. level - 1)
		downgrade_button.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f(5.0, -130, 1.0)}
		downgrade_button.on_release = function(button)
			 UserData.instance():setTypeLevel(name, level - 1)
			 self:setData(name, type_data)
		end	
	end
end
