-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local pool      = chunks.pool
local util      = chunks.util

LuaUpgradeMenuWindow = chunks.class.new
{
	m_data_file = nil,
	m_types_names = nil,
	m_type_buttons = nil,
	m_upgrade_info_window = nil,
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaUpgradeMenuWindow )

function LuaUpgradeMenuWindow:created(window)
	window.hidden = true
	window.size = constants.main_menu_camera.window.world_size
	
	do -- Background
		local background = window:spawn(SpriteActor())
		assets.setTexture(background , assets.textures.castle_front_background)
		background.location = pool.get_v3f(0.0, 0.0, -1.0)
	end
	
	-- the back button
	do
		local back_button = window:spawn(Button())
		
		back_button.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f(0.0, 0.0, 1.0)}
		assets.setTexture(back_button, assets.textures.back_button)
		
		-- Dim the button.
		back_button.on_enter = function(owner)
			owner.color = colorf( 0.7, 0.7, 0.7, 1.0 )
		end	
	
		-- Undim the button.
		back_button.on_exit = function(owner)
			owner.color = colorf( 1.0, 1.0, 1.0, 1.0 )
		end

		-- Quit the game
		back_button.on_release = function(owner)
			--owner:getGame():destroy()
			--owner:getEngine().lua.m_main_menu.hidden_recursive = false
			engine:goToPreviousGame()
		end
	end
	
	self.m_upgrade_info_window = window:spawn(chunks.upgrade_info_window.spawn())	
	
	local create_on_release_function = function (button, data_file, condition)
		button.on_release = function (the_button)
			local type_names = {}
			
			for name,object in pairs(data_file) do
				if object.level_stat and #object.level_stat then
					if condition == nil or condition(object) then
						table.insert(type_names, name)
						--print(name)
					end
				end
			end
			
			table.sort(type_names, function(left, right) return left < right end)
			
			self.m_upgrade_info_window.lua:setData(nil)
			self:switchUpgradePanels(data_file, type_names)
		end
	end
		
	-- create tab buttons
	do
		local upgrade_units_button = util.createButtonWithText(window, "Units")
		upgrade_units_button.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f(50.0, 0.0, 1.0)}
		create_on_release_function(upgrade_units_button, chunks.units, function (object) return object.ally and object.ally == true end)
		upgrade_units_button.on_release()
	end
	
	do
		local upgrade_spells_button = util.createButtonWithText(window, "Spells")
		upgrade_spells_button.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f(125.0, 0.0, 1.0)}
		create_on_release_function(upgrade_spells_button, chunks.spells)
	end
end


function LuaUpgradeMenuWindow:switchUpgradePanels(data_file, type_names)
	if self.m_data_file == data_file then
		return
	end
	
	if self.m_type_buttons then
		for k,v in pairs(self.m_type_buttons) do
			if v then
				v:destroy()
			end
		end
		
		self.m_type_buttons = nil
	end
	self.m_type_buttons = {}
	
	self.m_data_file = data_file
	self.m_types_names = type_names	
	
	local num_types = #type_names
	
	if num_types == 0 then
		return
	end	
	
	local start_y = -40
	local y_spacing = 10
	local x_spacing = 10
	local button_texture = assets.textures.icon_generic
	local button_size = button_texture.size
	local button_with_spacing_x = button_size.x + x_spacing
	local button_with_spacing_y = button_size.y + y_spacing
	local num_columns = math.floor(constants.main_menu_camera.window.world_size.x / button_with_spacing_x)
	local num_rows = math.ceil(num_types / num_columns)
	
	local row = 1
	local column = 1
	for i,name in pairs(type_names) do
		local type_data = data_file[name]
		if type_data then
			local level = UserData.instance():getTypeLevel(name)
			local level_data = chunks.load.getLevelData(type_data, level)
			
			local button = util.createButtonWithText(self.owner, name, {"Debug Font", 10.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0) } )
			button.label = name
			
			button.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f((column - 1) * button_with_spacing_x, ((row - 1) * -button_with_spacing_y) + start_y, 1.0)}
			assets.setTexture(button, button_texture)

			-- Dim the button.
			button.on_enter = function(owner)
				owner.color = colorf( 0.7, 0.7, 0.7, 1.0 )
			end	
		
			-- Undim the button.
			button.on_exit = function(owner)
				owner.color = colorf( 1.0, 1.0, 1.0, 1.0 )
			end

			-- Shoow the upgrade screen
			button.on_release = function(button)
				self.m_upgrade_info_window.lua:setData(name, type_data)				
			end		
			
			table.insert(self.m_type_buttons, button)
			column = column + 1
			
			if column > num_columns then
				column = 1
				row = row + 1
			end
		end
	end	
	
	self.m_upgrade_info_window.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TL, pool.get_v3f(0.0, -160, 1.0)}
end


