-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local pool      = chunks.pool

LuaUpgradeUnitMenuGame = chunks.class.new
{
	m_asset_group = nil,
	m_dismissable = nil,
}

-- The spawner.
spawn = chunks.class.spawner( SingleTouchWindowGame, LuaUpgradeUnitMenuGame )

function LuaUpgradeUnitMenuGame:created(game)
	print("LuaUpgradeUnitMenuGame:created")
	
	local game_window = game:getWindow()
	
	-- Window camera.
	do
		local camera = game:getWindow():spawn( OrthographicCamera() )

		camera.viewport_offset = constants.main_menu_camera.window.viewport_offset
		camera.viewport_size   = constants.main_menu_camera.window.viewport_size
		camera.screen_size     = constants.main_menu_camera.window.screen_size
		camera.world_size      = constants.main_menu_camera.window.world_size
		camera.clipping_planes = constants.main_menu_camera.window.clipping_planes
		camera.clear_color     = constants.clear_color
		camera.clear_color_on  = constants.clear_color_on
		camera.clear_depth_on  = constants.clear_depth_on
		camera.scene           = game_window

		game_window.camera = camera
	end
	
	self.m_asset_group = ResourceGroup("texture")
	
	self:preloadAssets()
	
	Resources.get():addResourceGroup(self.m_asset_group)
	self.m_main_window = game_window:spawn(chunks.upgrade_unit_menu_window.spawn())
	Resources.get():removeResourceGroup(self.m_asset_group)
	
	chunks.music.checkOrPlaySong( assets.music.loop )
end

-- Input handling for the android back button.  
function LuaUpgradeUnitMenuGame:onInputEvents( game, events )
	if engine.platform.device == "android" then
		for event in events:all() do
			local e = AndroidEvent.fast_cast( event )
			if e and e.type == AndroidEvent.AndroidEvent_BackButton then
				assets.playSound(assets.sounds.button_click_wav)
				if self.m_dismissable and self.m_dismissable.lua and self.m_dismissable.lua.dismiss then
				    self.m_dismissable.lua:dismiss()
				else
				    engine:goToPreviousGame()
				end
			end
		end
	end
end


function LuaUpgradeUnitMenuGame:destroy()
	self:unloadAssets()
end


function LuaUpgradeUnitMenuGame:preloadAssets()
	--TODO: FIXME local sound_manager = Audio.get().sound
	
	-- sounds
	--sound_manager:precache(URI(assets.getSound(assets.sounds.button_click)))
end

function LuaUpgradeUnitMenuGame:unloadAssets()
	Resources.get():unload(self.m_asset_group)
end
