-- module --
module(...,package.seeall)

-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local constants = engine.platform.constants
local assets    = chunks.assets
local pool      = chunks.pool
local util      = chunks.util
local load      = chunks.load
local incent_ads = chunks.incent_ads

local white = colorf(1, 1, 1, 1)
local active_color = colorf(1.0, 0.89, 0.28, 1.0)
local inactive_color = colorf(0.75, 0.75, 0.75, 1.0)
local current_wave = 0
local scroll_view_inset = v2f(15, 4)
local scroll_view_offset = v3f(-5, 0, 0)

local wave_string = lstring("wave")
local stat_string = lstring("stat")
local gold_interp_time = 0.083

LuaUpgradeUnitMenuWindow = chunks.class.new
{
    m_inactive_tabs = nil,
    m_active_tabs = nil,
    m_labels = nil,
    m_troops = nil,
    m_gold_label = nil,
    m_weapon_cells = nil,
    m_spells = nil,
    m_castle = nil,
    m_ash_cell = nil,
    m_scrollbars = nil,
    m_scroll_windows = nil,
    m_scroll_view = nil,
    m_coin_store = nil,
    m_incent_ad_button = nil,
    m_window = nil,
    m_is_watching_ad = false
}

-- The spawner.
spawn = chunks.class.spawner( Window, LuaUpgradeUnitMenuWindow )

function LuaUpgradeUnitMenuWindow:created(window)
    m_window = window
    window.size = constants.main_menu_camera.window.world_size
    if ( engine.m_default_language_used ) then
        window.texture = "?://texture/UI/menu/upgrade/upgrade_background"
    else
        window.texture = "?://texture/UI/menu/upgrade/upgrade_background_locale"
    end
    --window.receive_input = false

    current_wave = UserData.instance():getCurrentWave()
    self.m_inactive_tabs = {}
    self.m_active_tabs = {}
    self.m_labels = {}
    self.m_weapon_cells = {}
    self.m_scrollbars = {}
    self.m_scroll_windows = {}
    self.m_troops = util.deepcopy(chunks.globals.ally_units)
    self.m_spells = {}
    self.m_castle = {
                    {name = "necronomicon"},
                    {name = "smithy"},
                    {name = "inner_wall_archer"},
                    {name = "catapults"},
                    {name = "the_pit"},
                    }

    do -- Current Wave
        local label = window:spawn(Text())
        --label.format = format
        label.style = "Georgia Bold"
        label.options = {height = 23.0, max_width = 100.0, align = TextAlign.Center}
        label.color = pool.get_colorf(0.93, 0.72, 0.26, 1.0)
        wave_string:arg(1, util.getCurrentWave())
        label.text = wave_string
        label.anchor = {Anchor.Anchor_R, Anchor.Anchor_C, pool.get_v3f(-15.0, 146.0, 1.0)}
    end

    do -- Upgrades text
        if not engine.m_default_language_used then
            local text_options =    {
                                            height = 23,
                                            max_width = 150,
                                            align = TextAlign.Right,
                                            font_color = colorf(1, 1, 1, 1),
                                            spacing = 0.0
                                        }
            local upgrades_text = util.createText(window, "UPGRADES", "Georgia Bold", text_options, {x = 0.0, y = 5.0})
            --upgrades_text.anchor = {Anchor.Anchor_L, Anchor.Anchor_L, pool.get_v3f(constants.widescreen and 200 or 250, 147, 0.5)}
            upgrades_text.anchor = {Anchor.Anchor_R, Anchor.Anchor_R, pool.get_v3f(constants.widescreen and -100 or -75, 147, 0.5)}
        end
    end

    -- the Menu button
    do
        local text = lstring("menu")
        local format = {"Georgia Bold", 18.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 45.0, TextAlign.Center, 0.0, 2.5, 0.0, false}
        local back_button =  util.createButtonWithDefaultOrLocale(window, text, format, assets.textures.blank_button_locale, assets.textures.menu_button)
        back_button.anchor = {Anchor.Anchor_TR, Anchor.Anchor_TR, pool.get_v3f(0.0, 0.0, 1.0)}

        -- Quit the game
        back_button.on_release = function(owner)
            assets.playSound(assets.sounds.button_click_wav)
            engine:goToPreviousGame()
        end
    end

    -- the Done button
    do
        local text = lstring("continue")
        local format = {"Georgia Bold", 25.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 95.0, TextAlign.Center, 0.0, 4.0, 0.0, false}
        local back_button = util.createButtonWithDefaultOrLocale(window, text, format, assets.textures.continue_button_locale, assets.textures.continue_button)
        back_button.anchor = {Anchor.Anchor_BR, Anchor.Anchor_BR, chunks.globals.continue_button_offset}

        -- Start the game
        back_button.on_release = function(owner)
            assets.playSound(assets.sounds.button_click_wav)
            -- Setting this to true will prevet the ad button from spawning when it.
            self.m_incent_ad_button = true

            if util.hasAnyUnitsUnlocked() then
                engine:spawnSelectUnitMenu()
            else
                engine:spawnMainGame()
            end
        end
    end

    -- the gold box
    do
        local gold_box = util.createWindowWithTexture(window, assets.textures.gold_box)
        gold_box.anchor = {Anchor.Anchor_BR, Anchor.Anchor_B, pool.get_v3f(-100.0, 4.0, 1.0)}

        -- the gold coin
        do
            local gold_coin = util.createWindowWithTexture(gold_box, assets.textures.upgrade_gold_coin)
            gold_coin.anchor = {Anchor.Anchor_L, Anchor.Anchor_L, pool.get_v3f(3.0, 0.0, 1.0)}

            --the gold amount label
            do
                local gold_label = gold_coin:spawn(Text())
                gold_label.style = "Helvetica-Bold"
                gold_label.options = {height = 28.0}
                gold_label.color = pool.get_colorf(1.0, 0.89, 0.28, 1.0)
                gold_label.anchor = {Anchor.Anchor_L, Anchor.Anchor_R, pool.get_v3f(-2.0, 5.0, 1.0)}
                self.m_gold_label = gold_label
                self:updateText()
            end
        end
    end

    --the Buy Gold Button
    do
        local text = lstring("buy_coins")
        local format = {"Georgia Bold", 15.0, pool.get_colorf(0.93, 0.81, 0.27, 1.0), 40.0, TextAlign.Center, -1.6, 2.5, 0.0, false}
        local buy_gold =  util.createButtonWithDefaultOrLocale(window, text, format, assets.textures.buy_coins_button_locale, assets.textures.buy_coins_button)
        buy_gold.anchor = {Anchor.Anchor_BL, Anchor.Anchor_B, pool.get_v3f(-100.0, 3.0, 1.0)}

        --IN APP PURCHASES
        buy_gold.on_release = function(owner)
            assets.playSound(assets.sounds.button_click_wav)

            util.requireNetworkConnection( function()
                if chunks.iap.purchasingAvailable() then
                    self.m_coin_store = window:spawn(chunks.coinstore.spawn())
                else
                    chunks.iap.displayPurchasingUnavailable()
                end
            end )
        end
    end

    --the scroll region
    do
        local scroll_region = util.createWindowWithTexture(window, assets.textures.scroll_region)
        if constants.widescreen then
            scroll_region.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(60.0, 8.0, 1.0)}
        else
            scroll_region.anchor = {Anchor.Anchor_R, Anchor.Anchor_R, pool.get_v3f(0.0, 8.0, 1.0)}
        end
        scroll_region.color = pool.get_colorf(0, 0, 0, 0.35)

        local top = util.createWindowWithTexture(scroll_region, assets.textures.scroll_region_top)
        top.anchor = {Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0.0, 0.0, 0.1)}
        top.force_color = true

        local right = util.createWindowWithTexture(scroll_region, assets.textures.scroll_region_right)
        right.anchor = {Anchor.Anchor_R, Anchor.Anchor_R, pool.get_v3f(0.0, 0.0, 0.1)}
        right.force_color = true

        local bottom = util.createWindowWithTexture(scroll_region, assets.textures.scroll_region_bottom)
        bottom.anchor = {Anchor.Anchor_B, Anchor.Anchor_B, pool.get_v3f(0.0, 0.0, 0.1)}
        bottom.force_color = true


        --THE SCROLL VIEW
        local scroll_view = scroll_region:spawn(ScrollView())
        scroll_view.force_color = true
        scroll_view.location = scroll_view.location + scroll_view_offset
        scroll_view.size = scroll_region.size - scroll_view_inset
        scroll_view.content_size = scroll_region.size - scroll_view_inset
        scroll_view.allows_horizontal_movement = false
        self.m_scroll_view = scroll_view

        local scrollbar = scroll_region:spawn(Scrollbar())
        scrollbar.receive_input = false
        scrollbar.force_color = true
        scrollbar.anchor = { Anchor.Anchor_R, Anchor.Anchor_R, pool.get_v3f(0, 0, 250) }
        assets.setTexture(scrollbar, assets.textures.scroll_bar)
        scrollbar.handle_inset = pool.get_v2f(3, 3.5)
        assets.setTexture(scrollbar.handle, assets.textures.scroll_control)
        scroll_view.vertical_scrollbar = scrollbar



        --ASH__ASH__ASH__ASH__ASH__ASH__ASH__ASH__ASH__ASH__ASH__ASH__ASH__
        --Ash INACTIVE Tab
        do
            local ash_inactive = util.createButtonWithTexture(scroll_region, assets.textures.tab_inactive)
            ash_inactive.force_color = true
            ash_inactive.anchor = {Anchor.Anchor_TR, Anchor.Anchor_TL, pool.get_v3f(0.0, 0.0, 1.0)}
            ash_inactive.hidden = true
            ash_inactive.label = "ash"
            ash_inactive.on_release = function(owner) assets.playSound(assets.sounds.button_click_wav); self:inactiveTabClicked(owner) end
            self.m_inactive_tabs.ash = ash_inactive

            --Ash Label
            do
                local ash_label = ash_inactive:spawn(Text())
                local format = {"Georgia Bold", 28.0, 105.0, TextAlign.Center}
                ash_label.style = format[1]
                ash_label.options = {height = format[2], max_width = format[3], align = format[4]}
                ash_label.color = active_color
                ash_label.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0.0, 7, 100.0)}
                ash_label.text = lstring("Ash")
                self.m_labels.ash = ash_label
            end
        end

        --Ash Active Tab
        do
            local ash_active = util.createWindowWithTexture(scroll_region, assets.textures.tab_active)
            ash_active.force_color = true
            ash_active.anchor = {Anchor.Anchor_TR, Anchor.Anchor_TL, pool.get_v3f(0.0, 0.0, 1.0)}
            ash_active.hidden_recursive = false
            self.m_active_tabs.ash = ash_active

            local scroll_view_size = scroll_region.size - scroll_view_inset
            local scroll_window = self:createAshScrollView(scroll_view, scroll_view_size)
            --scroll_view.location = v3f((ash_active.size.x + scroll_view_size.x)*0.5, (ash_active.size.y - scroll_view_size.y)*0.5 - 2.0, 1)
        end

        --Troops__Troops__Troops__Troops__Troops__Troops__Troops__Troops__Troops__
        --Troops INACTIVE Tab
        do
            local troops_inactive = util.createButtonWithTexture(scroll_region, assets.textures.tab_inactive)
            troops_inactive.force_color = true
            troops_inactive.anchor = {Anchor.Anchor_R, Anchor.Anchor_L, pool.get_v3f(0.0, 0.0, 1.0)}
            troops_inactive.hidden = false
            troops_inactive.label = "troops"
            troops_inactive.on_release = function(owner) assets.playSound(assets.sounds.button_click_wav); self:inactiveTabClicked(owner) end
            self.m_inactive_tabs.troops = troops_inactive

            --Troops Label
            do
                local troops_label = troops_inactive:spawn(Text())
                local format = {"Georgia Bold", 28.0, 105.0, TextAlign.Center}
                troops_label.style = format[1]
                troops_label.options = {height = format[2], max_width = format[3], align = format[4]}
                troops_label.color = inactive_color
                troops_label.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0.0, 7, 100.0)}
                troops_label.text = lstring("troops")
                self.m_labels.troops = troops_label
            end
        end

        --Troops Active Tab
        do
            local troops_active = util.createWindowWithTexture(scroll_region, assets.textures.tab_active)
            troops_active.force_color = true
            troops_active.anchor = {Anchor.Anchor_R, Anchor.Anchor_L, pool.get_v3f(0.0, 0.0, 1.0)}
            troops_active.hidden_recursive = true
            self.m_active_tabs.troops = troops_active

            local scroll_view_size = scroll_region.size - scroll_view_inset
            local scroll_window = self:createTroopsScrollView(scroll_view, scroll_view_size)
            --scroll_view.location = v3f((troops_active.size.x + scroll_view_size.x)*0.5, 0, 1)
        end

        --Castle__Castle__Castle__Castle__Castle__Castle__Castle__Castle__Castle__Castle__
        --Castle INACTIVE Tab
        do
            local castle_inactive = util.createButtonWithTexture(scroll_region, assets.textures.tab_inactive)
            castle_inactive.force_color = true
            castle_inactive.anchor = {Anchor.Anchor_BR, Anchor.Anchor_BL, pool.get_v3f(0.0, 0.0, 1.0)}
            castle_inactive.hidden = false
            castle_inactive.label = "castle"
            castle_inactive.on_release = function(owner) assets.playSound(assets.sounds.button_click_wav); self:inactiveTabClicked(owner) end
            self.m_inactive_tabs.castle = castle_inactive

            --castle Label
            do
                local castle_label = castle_inactive:spawn(Text())
                local format = {"Georgia Bold", 28.0, 105.0, TextAlign.Center}
                castle_label.style = format[1]
                castle_label.options = {height = format[2], max_width = format[3], align = format[4]}
                castle_label.color = inactive_color
                castle_label.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0.0, 7, 100.0)}
                castle_label.text = lstring("castle")
                self.m_labels.castle = castle_label
            end
        end

        --castle Active Tab
        do
            local castle_active = util.createWindowWithTexture(scroll_region, assets.textures.tab_active)
            castle_active.force_color = true
            castle_active.anchor = {Anchor.Anchor_BR, Anchor.Anchor_BL, pool.get_v3f(0.0, 0.0, 1.0)}
            castle_active.hidden_recursive = true
            self.m_active_tabs.castle = castle_active

            local scroll_view_size = scroll_region.size - scroll_view_inset
            local scroll_window = self:createCastleScrollView(scroll_view, scroll_view_size)
            --scroll_view.location = v3f((castle_active.size.x + scroll_view_size.x)*0.5, (castle_active.size.y - scroll_view_size.y)*-0.5 + 2.0, 1)

        end
    end

    util.unifyTextHeight({self.m_labels.ash, self.m_labels.troops, self.m_labels.castle})

    self:inactiveTabClicked(self.m_inactive_tabs.ash)

    self:updateAdDisplay()

    engine:setUpgradeUnitMenuWindow(self)

    -- Do the request for notifications
    local service = GameService.get()
    if service and current_wave >= 5 then
        service:registerForNotifications()
    end
end

local ad_view_callback_handle = nil
function LuaUpgradeUnitMenuWindow:updateAdDisplay()

    local game_service = GameService.get()

    -- If the timer has expired
    if ( ( not self.m_is_watching_ad ) and game_service:shouldDisplayIncentAd() ) then 

        -- Start preloading the ads.
        ad_view_callback_handle = chunks.incent_ads.preloadAd("postroll", function()

            if( self.owner:isDestroyed() ) then
                return
            end

            if( m_window:isDestroyed() ) then
                return
            end

            -- If the incent ad button doesnt exist, create it.
            if( self.m_incent_ad_button == nil ) then

                local text = lstring("free_coins")
                local format = {"Georgia Bold", 22.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 80.0, TextAlign.Center, 10, 2, 0.0, false}
                self.m_incent_ad_button =  util.createButtonWithTextAndTexture(m_window, text, format, assets.textures.watch_ad_button)
                self.m_incent_ad_button.anchor = {Anchor.Anchor_BL, Anchor.Anchor_B, pool.get_v3f(-45.0, 1.0, 1.0)}

                -- Open the ad menu.
                self.m_incent_ad_button.on_release = function(owner)

                    assets.playSound(assets.sounds.button_click_wav)

                    if( game_service:shouldDisplayIncentAd() ) then
                        m_window:spawn(chunks.incent_ad_menu.spawn())
                    end
                end

                --buy_gold.color = pool.get_colorf(0, 0, 0, 0)
                self.m_incent_ad_button:addAction(AlphaAction(0.0, InterpAction.Smooth, 0.0))
                self.m_incent_ad_button:addAction(AlphaAction(0.2, InterpAction.Smooth, 1.0))
            end
        end,
        function()
            if( self.m_incent_ad_button ) then
                self.m_incent_ad_button:addAction(AlphaAction(0.2, InterpAction.Smooth, 0.0))
                self.m_incent_ad_button:addAction(DestroyAction(0))
                self.m_incent_ad_button = nil
            end
        end)
    else
        -- If the button exists, destroy it
        if( self.m_incent_ad_button ) then
            self.m_incent_ad_button:addAction(AlphaAction(0.2, InterpAction.Smooth, 0.0))
            self.m_incent_ad_button:addAction(DestroyAction(0))
            self.m_incent_ad_button = nil
        end
    end
end

function LuaUpgradeUnitMenuWindow:updateText( _loud, _interp_from )
    
    if( _loud ) then

        local target_gold = UserData.instance():getGold()
        local interp_timer = 0

        function doGoldInterp()
            interp_timer = interp_timer + gold_interp_time

            local interp_gold = ( target_gold - _interp_from ) * interp_timer + _interp_from
            interp_gold = math.floor( interp_gold )

            if( interp_gold > target_gold ) then
                interp_gold = target_gold
            end

            stat_string:arg(1, interp_gold)
            self.m_gold_label.text = lstring(stat_string)

            if( interp_gold < target_gold ) then
                self.m_gold_label:addAction(LuaAction(gold_interp_time, doGoldInterp))
            end
        end

        self.m_gold_label:addAction(LuaAction(gold_interp_time, doGoldInterp))
        assets.playSound(assets.sounds.pickup_gold)

    else
        stat_string:arg(1, UserData.instance():getGold())
        self.m_gold_label.text = lstring(stat_string)
        util.widthConstraint(self.m_gold_label, 93)
    end
end

function LuaUpgradeUnitMenuWindow:inactiveTabClicked(button)
    for i,v in pairs(self.m_inactive_tabs) do
        v.hidden = false
    end

    for i,v in pairs(self.m_active_tabs) do
        v.hidden_recursive = true
    end

    --for i,v in pairs(self.m_scrollbars) do
        --v.hidden_recursive = true
    --end


    for i,v in pairs(self.m_labels) do
        v.color = inactive_color
    end

    for i,v in pairs(self.m_scroll_windows) do
        v.hidden_recursive = true
    end



    local name = button.label
    self.m_inactive_tabs[name].hidden = true
    self.m_active_tabs[name].hidden_recursive = false
    self.m_labels[name].color = active_color
    --self.m_scrollbars[name].hidden_recursive = false
    self.m_scroll_windows[name].hidden_recursive = false
    self.m_scroll_view.content_size = self.m_scroll_windows[name].size + pool.get_v2f(0, 2)
    self.m_scroll_view:setVerticalScrollPercent(0.0, true)
end

function LuaUpgradeUnitMenuWindow:createTroopsScrollView(parent, _size)
    --local scroll_view = parent:spawn(ScrollView())
    --scroll_view.size = _size
    --scroll_view.content_size = _size
    --scroll_view.allows_horizontal_movement = false

    self:prepareTroopsTable()

    local cell_size = assets.textures.upgrade_box.size + pool.get_v2f(0, 2)
    --scroll_view.content_size = v2f(_size.x, math.max(_size.y, cell_size.y*#self.m_troops+2))

    --[[local scrollbar = self.owner:spawn(Scrollbar())
    scrollbar.anchor = { Anchor.Anchor_R, Anchor.Anchor_R, pool.get_v3f(-0.0, 8, 250) }
    assets.setTexture(scrollbar, assets.textures.scroll_bar)
    scrollbar.handle_inset = pool.get_v2f(3, 3.5)
    assets.setTexture(scrollbar.handle, assets.textures.scroll_control)
    scroll_view.vertical_scrollbar = scrollbar
    self.m_scrollbars.troops = scrollbar
    ]]
    local scroll_window = parent.content_view:spawn(Window())
    scroll_window.size = v2f(_size.x, cell_size.y*#self.m_troops)
    scroll_window.hidden = true
    scroll_window.receive_input = false
    self.m_scroll_windows.troops = scroll_window

    --util.createWindowWithTexture(scroll_window, assets.textures.upgrade_box)
    for i=1,#self.m_troops do
        self:createTroopCell(scroll_window, i, -1.0*((i-1)*cell_size.y+2))
    end


    return scroll_window
end

function LuaUpgradeUnitMenuWindow:prepareTroopsTable()
    for i,v in ipairs(self.m_troops) do
        v.unlock_wave = load.getStat("units", v.name, "unlock_wave") or 50
        --print("get unlock level", v.name, v.unlock_wave)
    end

    table.sort(self.m_troops, function(left, right) return left.unlock_wave < right.unlock_wave end )

    --[[
    local next_unlock_wave = math.huge
    for i,v in ipairs(self.m_troops) do
        --print("remove units", v.name, v.unlock_wave)
        if v.unlock_wave > next_unlock_wave then
            self.m_troops[i] = nil
        elseif v.unlock_wave > current_wave then
            next_unlock_wave = v.unlock_wave
        end

    end
    ]]

    --table.sort(self.m_troops, function(left, right) return left.unlock_wave < right.unlock_wave end )

    --[[print("Num Cells:",#self.m_troops)
    for i,v in ipairs(self.m_troops) do
        print("resort", v.name, v.unlock_wave)
    end]]
end

function LuaUpgradeUnitMenuWindow:createTroopCell(parent, i, vertical_offset)
    local name = self.m_troops[i].name
    local unlock_wave = self.m_troops[i].unlock_wave

    local cell = parent:spawn(chunks.troop_cell.spawn())
    cell.anchor = {Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0.0, vertical_offset, 1.0)}

    cell.lua:setTroop(name, unlock_wave, current_wave)
    cell.lua.m_main_window = self.owner



end



function LuaUpgradeUnitMenuWindow:createAshScrollView(parent, _size)
    --local scroll_view = parent:spawn(ScrollView())
    --scroll_view.size = _size
    --scroll_view.content_size = _size
    --scroll_view.allows_horizontal_movement = false

    self:prepareSpellTable()

    local weapons = {}
    if UserData.instance():getLocked("iron_fist") then
        table.insert(weapons, "iron_fist")
    elseif UserData.instance():getLocked("chainsaw") then
        table.insert(weapons, "chainsaw")
    end

    local num_cells = 1 + #weapons + #self.m_spells

    local cell_size = assets.textures.upgrade_box.size + pool.get_v2f(0, 2)
    --scroll_view.content_size = v2f(_size.x, math.max(_size.y, cell_size.y*num_cells+2))

    --[[local scrollbar = self.owner:spawn(Scrollbar())
    scrollbar.anchor = { Anchor.Anchor_R, Anchor.Anchor_R, pool.get_v3f(-0.0, 8, 250) }
    assets.setTexture(scrollbar, assets.textures.scroll_bar)
    scrollbar.handle_inset = pool.get_v2f(3, 3.5)
    assets.setTexture(scrollbar.handle, assets.textures.scroll_control)
    scroll_view.vertical_scrollbar = scrollbar
    self.m_scrollbars.ash = scrollbar
    ]]


    local scroll_window = parent.content_view:spawn(Window())
    scroll_window.size = v2f(_size.x, cell_size.y*num_cells)
    scroll_window.hidden = true
    scroll_window.receive_input = false
    self.m_scroll_windows.ash = scroll_window

    --util.createWindowWithTexture(scroll_window, assets.textures.upgrade_box)
    local cell = scroll_window:spawn(chunks.ash_cell.spawn())
    cell.anchor = {Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0.0, -2, 1.0)}
    --cell.lua:setTroop("ash")
    cell.lua.m_main_window = self.owner
    self.m_ash_cell = cell

    for i,v in pairs(weapons) do
        local cell = scroll_window:spawn(chunks.weapon_cell.spawn())
        cell.anchor = {Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0.0, -1.0*(cell_size.y*i+2), 1.0)}
        cell.lua:setWeapon(v)
        cell.lua.m_main_window = self.owner
        table.insert(self.m_weapon_cells, cell)
    end

    for i,v in ipairs(self.m_spells) do
        self:createSpellCell(scroll_window, i, -1.0*((i-1+(1 + #weapons))*cell_size.y+2))
    end

    return scroll_window
end

function LuaUpgradeUnitMenuWindow:prepareSpellTable()
    self.m_spells = util.getValidSpells()

    --[[
    local next_unlock_wave = math.huge
    for i,v in ipairs(self.m_spells) do
        --print("remove units", v.name, v.unlock_wave)
        if v.unlock_wave > next_unlock_wave then
            self.m_spells[i] = nil
        elseif v.unlock_wave > current_wave then
            next_unlock_wave = v.unlock_wave
        end
    end
    ]]
end

function LuaUpgradeUnitMenuWindow:createSpellCell(parent, i, vertical_offset)
    local name = self.m_spells[i].name
    local unlock_wave = self.m_spells[i].unlock_wave


    local cell = parent:spawn(chunks.spell_cell.spawn())
    cell.anchor = {Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0.0, vertical_offset, 1.0)}

    cell.lua:setSpell(name, unlock_wave, current_wave)
    cell.lua.m_main_window = self.owner



end

function LuaUpgradeUnitMenuWindow:createCastleScrollView(parent, _size)
    --local scroll_view = parent:spawn(ScrollView())
    --scroll_view.size = _size
    --scroll_view.content_size = _size
    --scroll_view.allows_horizontal_movement = false

    self:prepareCastleTable()

    local cell_size = assets.textures.upgrade_box.size + pool.get_v2f(0, 2)
    --scroll_view.content_size = v2f(_size.x, math.max(_size.y, cell_size.y*#self.m_castle+2))

    --[[local scrollbar = self.owner:spawn(Scrollbar())
    scrollbar.anchor = { Anchor.Anchor_R, Anchor.Anchor_R, pool.get_v3f(-0.0, 8, 250) }
    assets.setTexture(scrollbar, assets.textures.scroll_bar)
    scrollbar.handle_inset = pool.get_v2f(3, 3.5)
    assets.setTexture(scrollbar.handle, assets.textures.scroll_control)
    scroll_view.vertical_scrollbar = scrollbar
    self.m_scrollbars.castle = scrollbar
    ]]

    local scroll_window = parent.content_view:spawn(Window())
    scroll_window.size = v2f(_size.x, cell_size.y*#self.m_castle)
    scroll_window.hidden = true
    scroll_window.receive_input = false
    self.m_scroll_windows.castle = scroll_window

    --util.createWindowWithTexture(scroll_window, assets.textures.upgrade_box)
    for i,v in ipairs(self.m_castle) do
        if v.name == "necronomicon" or v.name == "smithy" then
            self:createNecronomiconCell(scroll_window, i, -1.0*((i-1)*cell_size.y+2))
        else
            self:createCastleCell(scroll_window, i, -1.0*((i-1)*cell_size.y+2))
        end
    end


    return scroll_view
end

function LuaUpgradeUnitMenuWindow:prepareCastleTable()
    for i,v in ipairs(self.m_castle) do
        v.unlock_wave = load.getStat("units", v.name, "unlock_wave") or 1
    end

    table.sort(self.m_castle, function(left, right) return left.unlock_wave < right.unlock_wave end )

    --[[
    local next_unlock_wave = math.huge
    for i,v in ipairs(self.m_castle) do
        if v.unlock_wave > next_unlock_wave then
            self.m_castle[i] = nil
        elseif v.unlock_wave > current_wave then
            next_unlock_wave = v.unlock_wave
        end

    end
    ]]
end

function LuaUpgradeUnitMenuWindow:createCastleCell(parent, i, vertical_offset)
    local name = self.m_castle[i].name
    local unlock_wave = self.m_castle[i].unlock_wave

    local cell = parent:spawn(chunks.castle_cell.spawn())
    cell.anchor = {Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0.0, vertical_offset, 1.0)}

    cell.lua:setCastle(name, unlock_wave, current_wave)
    cell.lua.m_main_window = self.owner

end

function LuaUpgradeUnitMenuWindow:createNecronomiconCell(parent, i, vertical_offset)
    local name = self.m_castle[i].name
    local unlock_wave = self.m_castle[i].unlock_wave

    local cell = parent:spawn(chunks.troop_cell.spawn())
    cell.anchor = {Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0.0, vertical_offset, 1.0)}

    cell.lua:setTroop(name, unlock_wave, current_wave)
    cell.lua.m_main_window = self.owner

end

