module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local assets    = chunks.assets
local util      = chunks.util

local max_range = 400
local additional_book_offset = 50
local suck_time = 1.4
local rise_time = 0.8
local fall_time = 0.6
local fade_time = 0.5

function cast(caster, type_name, location)
	local game = caster:get():getMainGame()
	local level = UserData.instance():getTypeLevel(type_name)
	--local location = caster:get().location + pool.get_v3f(50, 0, 1)
	local new_y = Map.get():getYGivenX(GameObject.TEAM_NEUTRAL, location.x)
	local location = v3f(location.x + additional_book_offset, new_y, location.z)

	local team_to_attack = caster:get():getOppositeTeam()
	local victims = game:getObjectsInRange(team_to_attack, location.x - (max_range / 2), location.x + (max_range / 2)).values
	
	local scene = game:getScene()
	local model = "mesh/objects/spells/vortex_book.gr2"
	local texture = "texture/objects/spells/vortex_book"
	local animation_name = "idle"
	local options = { loop_count = 1 }
	local explosion = util.createGrannyEffect(scene, model, texture, animation_name, options)
	explosion.location = location
	explosion.onAnimationEvent = function (actor, animation_name)

		 if animation_name == "strike" then
			suckUnitsIn(type_name, victims, location)
		 end

		 explosion:addAction(SleepAction(suck_time))
		 local remove_book_action = SplitAction()
		 remove_book_action:addAction(AlphaAction(fade_time, InterpAction.Ramp, 0))
		 remove_book_action:addAction(ScaleAction(fade_time, InterpAction.Ramp, pool.get_v3f(0.2, 0.2, 0.1)))
		 explosion:addAction(remove_book_action)
	end
	
	
	return true
end

function suckUnitsIn(type_name, victims, location)

	local damage_remaining = chunks.load.getStat("spells", type_name, "damage", level)
	for i,target_index in ipairs(victims) do
		local target = target_index:get()
		if target then
			local is_unique = chunks.load.getStat("units", target.label, "unique", target:getLevel()) or false
			if is_unique == false then
				local health = PartHealthIndex(target_index):get()
				if health then
					local health_remaining = health:getHealthRemaining()
					if health_remaining <= damage_remaining then
						damage_remaining = damage_remaining - health_remaining
						applyActions(target, location)
					else
						applyActions(target, location)
						return true
					end
				end
			end
		end	 
	end

end

function applyActions(target, location)
	
		--local pit_loc = owner:getWorldLocation()
        --local object_loc = object:getWorldLocation()
        
        local pit_loc = location
        local object_loc = target:getWorldLocation();
        local object = target
        
        
        local range = 150 --load.getStat("units", owner.label, "range")
        local dist_x = pit_loc.x - object_loc.x
        --if math.abs(dist_x) < range then
            local render = PartRenderIndex(object:getObjectIndex()):get()
            if render then
                local actor = render.render_actor
                local loc = actor:getWorldLocation()
                object.location = loc
                object_loc = loc
                actor.location = pool.get_v3f(0, 0, 0)
                --object_loc = actor:getWorldLocation()
                --actor:addAction(MoveAction(0.5, InterpAction.Lerp, pool.get_v3f(0, 0, 0)))
            end

            --local pit_render = PartRenderIndex(self.owner:getObjectIndex()):get()
            --local sprite_actor = SpriteActor.fast_cast(pit_render.render_actor)

            local height = math.random(130,150)
			local mid_x = (pit_loc.x + object_loc.x) * 0.5
			local rotate_direction = 1
			if (object_loc.x < pit_loc.x) then
			   rotate_direction = -1 
			end
			local mid_point = v3f(mid_x, object_loc.y + height, --[[self.owner.location.z-5]] 300) -- minus 5 because the put should be 300 and the opening should be 290

            local first_offset = mid_point - object_loc
			local second_offset = (pit_loc - pool.get_v3f(0, --[[sprite_actor.size.y*0.5]] 0, 0)) - mid_point

			local full_anim = SplitAction()
			full_anim:addAction(RotateAction(suck_time, InterpAction.Smooth, v3f( 0.0, 0.0, 180 * rotate_direction)))
			full_anim:addAction(MoveOffsetAction(suck_time, InterpAction.Smooth, pool.get_v3f(pit_loc.x - object_loc.x, 0.0, 0.0)))
			local up_and_down = SequenceAction() 

			local first_split = SplitAction()
			first_split:addAction(MoveOffsetAction(rise_time, InterpAction.Fall, pool.get_v3f(0.0, first_offset.y, first_offset.z)))
			first_split:addAction(ScaleAction(rise_time, InterpAction.Smooth, pool.get_v3f(0.75, 0.75, 1.0)))
			up_and_down:addAction(first_split)
	
			object:addAction(LuaAction(0.0, function() object.team = GameObject.TEAM_NEUTRAL; --[[self:incrementCount()]] end ))
	
			local second_split = SplitAction()
			second_split:addAction(MoveOffsetAction(fall_time, InterpAction.Ramp, pool.get_v3f(0.0, second_offset.y, 0.0)))
			local stretch = SequenceAction()
			stretch:addAction(ScaleAction(fall_time - 0.1, InterpAction.Smooth, pool.get_v3f(0.4, 0.8, 1.0)))
			stretch:addAction(ScaleAction(0.1, InterpAction.Smooth, pool.get_v3f(0.05, 0.05, 0.1)))
			second_split:addAction(stretch)
			up_and_down:addAction(second_split)
			
			full_anim:addAction(up_and_down)
			object:addAction(full_anim)

            --object:addAction(DestroyAction(0))
            object:addAction(LuaAction(0.0, function()
                object.label="";
                object.color = pool.get_colorf(0,0,0,0);
                object.team = GameObject.TEAM_ENEMY;
                object:die();
                --[[self:decrementCount();]]
                end ))
			
			local orders = PartOrdersIndex(object:getObjectIndex()):get()
			if orders then
				orders:addOrder(KnockbackOrder(0, 2.0), PartOrders.QUEUE_FIRST)
			end
        
        --end
	
end