module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local util		= chunks.util
local assets	= chunks.assets
local pool		= chunks.pool
local tutorial	= chunks.tutorial

Wave01Actor = chunks.class.new
{
	game = nil,
	game_window = nil,
	game_tutorial_window = nil,
	tutorial_text_window = nil,
	hit_tutorial_window = nil,
	first_enemy = nil,
	first_kill = false,
	fade_time = 0.25,
}

local function adjustZOffset(a, o)
	a.location = a.location + pool.get_v3f(0, 0, o)
end

function bindToObject(actor)
	Wave01Actor:new(actor) -- bind the script to the object
end

function Wave01Actor:created( o )
	self.owner = o
	self.owner.label = "Wave 1 actor: tutorial"
	self.game = self.owner:getGame()
	self.game_window = self.game.lua.m_game_window
	self.game_tutorial_window = self.game.lua.m_tutorial_window

	-- Hide the spell buttons at first
	for i,button in ipairs(self.game_window.lua.m_spell_buttons) do
		button.hidden_recursive = true
	end

	-- add LuaAction to watch for Ash taking damage
	tutorial.watchForAshDamage(self.owner, 0.75, function() --function to call when ash damage below percent
		self:ashHitTutorial()
	end)

	-- spawn movement tutorial
	self:tutorialStart()
end

function Wave01Actor:unitSpawned(i, gameObjectIndex, tag)
	if self.first_enemy == nil then
		self.first_enemy = gameObjectIndex
	end
end

function Wave01Actor:tutorialStart()
	self.game.lua.m_wave_action_actor.delta_time_factor = 0 -- engine value, essentially pauses the game

	-- display glowing, flashing necronomicon gate
	local glow_gate = self.game:getScene():spawn(SpriteActor())
	glow_gate.texture = "?://"..assets.textures.necro_gate_flash.path
	glow_gate.size = assets.textures.necro_gate_flash.size
	local necro_gate = self.game.lua.m_necro:get()
	glow_gate.location = necro_gate.location + pool.get_v3f(0, 0, 1)

	tutorial.addAlphaPulse(glow_gate, 0.6, 0.0)

	-- display text and arrow to protect the gate
	self.tutorial_text_window = tutorial.createWindow(self.game_window, "top")
	adjustZOffset(self.tutorial_text_window, 100)

	local label = self.tutorial_text_window:spawn(Text())
	label.anchor = { Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, 0+5, 1) }
	local format = {"Helvetica-Bold", 20.0+2, pool.get_colorf(1.0, 1.0, 1.0, 1.0)}
	label.style = format[1]
	label.options = {height = format[2]}
	label.text = lstring("protect_the_book") --"Protect the book!"

	local arrow = tutorial.createBouncingArrow(glow_gate, "left")
	arrow.force_color = true
	arrow.location = arrow.location + pool.get_v3f(60, 0, 0)

	-- wait several seconds, then continue
	self.owner:addAction( LuaAction(3.5, function()
		local tutorial_text_window = self.tutorial_text_window
		self.tutorial_text_window = nil
		tutorial_text_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		tutorial_text_window:addAction(DestroyAction(0))
		glow_gate:clearActions()
		glow_gate:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		glow_gate:addAction(DestroyAction(0))
		self.owner:addAction(LuaAction(self.fade_time, function() self:tutorialPart1() end))
	end) )
end

function Wave01Actor:tutorialPart1()
	local call_tutorial_2 = function()
		self:tutorialPart2()
	end

	-- check ash's location; if greater than value, shortcut this tutorial
	local ash = self.game.lua.m_ash:get()
	if not ash then
		print("NO ASH FOUND!!! D8")
	end

	if ash.location.x > 350 then
		-- shortcut
		self.game.lua.m_wave_action_actor.delta_time_factor = 1
		tutorial.watchForAshDistanceToEnemies(self.owner, 250, call_tutorial_2)
	else
		self.tutorial_text_window = tutorial.createWindow(self.game_window, "top")
		self.tutorial_text_window.color = pool.get_colorf(1, 1, 1, 0)
		adjustZOffset(self.tutorial_text_window, 100)

		--add text to the window
		local label = self.tutorial_text_window:spawn(Text())
		label.anchor = { Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0, -26+5, 1) }
		local format = {"Helvetica-Bold", 20.0+2}
		label.style = format[1]
		label.options = {height = format[2]}
		label.text = lstring("touch_screen") --"Touch the screen"
		label = label:spawn(Text())
		label.anchor = { Anchor.Anchor_T, Anchor.Anchor_B, pool.get_v3f(0, 0-5, 0) }
		local format = {"Helvetica", 16.0+2, 100}
		label.style = format[1]
		label.options = {height = format[2], max_width = 300, align = TextAlign.Center}
		label.text = lstring("to_move_ash") --"to move Ash left or right"

		-- spawn direction arrows
		local arrow = self.tutorial_text_window:spawn(SpriteActor()) --util.createWindowWithTexture(self.tutorial_text_window, assets.textures.tutorial_finger, false)
		arrow.texture = "?://"..assets.textures.go_arrow.path
		arrow.size = assets.textures.go_arrow.size
		arrow.scale = pool.get_v3f(-1, 1, 1)
		arrow.color = pool.get_colorf(1, 1, 1, 0)
		arrow:addAction(SleepAction(0.5))
		arrow:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1))
		tutorial.matchAnchorAtoLocalB(arrow, "TL", self.tutorial_text_window, "BR", pool.get_v3f(0, -32, 0))

		arrow = self.tutorial_text_window:spawn(SpriteActor()) --util.createWindowWithTexture(self.tutorial_text_window, assets.textures.tutorial_finger, false)
		arrow.texture = "?://"..assets.textures.go_arrow.path
		arrow.size = assets.textures.go_arrow.size
		arrow.color = pool.get_colorf(1, 1, 1, 0)
		arrow:addAction(SleepAction(0.5))
		arrow:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1))
		tutorial.matchAnchorAtoLocalB(arrow, "TR", self.tutorial_text_window, "BL", pool.get_v3f(0, -32, 0))

		self.tutorial_text_window:addAction(AlphaAction(0.5, InterpAction.Lerp, 1))

		local clear_move_tutorial = function()
			local tutorial_text_window = self.tutorial_text_window
			self.tutorial_text_window = nil
			tutorial_text_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
			tutorial_text_window:addAction(DestroyAction(0))
			self.game.lua.m_wave_action_actor.delta_time_factor = 1

			tutorial.watchForAshDistanceToEnemies(self.owner, 250, call_tutorial_2)
		end
		tutorial.watchForAshGreaterThanLocation(self.owner, 400, clear_move_tutorial)
	end
end

function Wave01Actor:tutorialPart2()
	self.tutorial_text_window = tutorial.createWindow(self.game_window, "top")
	adjustZOffset(self.tutorial_text_window, 100)

	--add text to the window
	local label = self.tutorial_text_window:spawn(Text())
	label.anchor = { Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0, 0+5, 1) }
	local format = {"Helvetica-Bold", 20.0+2}
	label.style = format[1]
	label.options = {height = format[2]}
	label.text = lstring("ash_attacks_auto") --"Ash attacks automatically"

	self.tutorial_text_window.color = pool.get_colorf(1, 1, 1, 0)
	self.tutorial_text_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1))

	-- function wrappers
	local call_tutorial_3 = function() self:tutorialPart3() end
	local clear_window = function()
		local tutorial_text_window = self.tutorial_text_window
		self.tutorial_text_window = nil
		tutorial_text_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		tutorial_text_window:addAction(DestroyAction(0))

		-- begin the next phase of the tutorial
		tutorial.watchForUnitDamage(self.owner, self.first_enemy, 0.65, call_tutorial_3)
	end

	-- clear tutorial window after a while
	self.owner:addAction( LuaAction(3.0, clear_window) )
end

-- Spell tutorial
function Wave01Actor:tutorialPart3()
	self.tutorial_text_window = tutorial.createWindow(self.game_window, "top")
	adjustZOffset(self.tutorial_text_window, 100)
	local window_loc = self.tutorial_text_window.location

	--add text to the window
	local label = self.tutorial_text_window:spawn(Text())
	label.anchor = { Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0, -26+5, 1) }
	local format = {"Helvetica-Bold", 14.0+2}
	label.style = format[1]
	label.options = {height = format[2]}
	label.text = lstring("tap_skill_button") --"Tap the Skill button"
	label = label:spawn(Text())
	label.anchor = { Anchor.Anchor_T, Anchor.Anchor_B, pool.get_v3f(0, 0-5, 0) }
	local format = {"Helvetica", 14.0+2}
	label.style = format[1]
	label.options = {height = format[2]}
	label.text = lstring("for_powerful_attack") --"for a powerful attack"

	--draw a glow around the spell
	local spell_button = self.game_window.lua.m_spell_buttons[1]
	local circle_button = util.createButtonWithTexture(self.tutorial_text_window, assets.textures.tutorial_circle)
	tutorial.matchAnchorAtoParentB(circle_button, "C", spell_button, "C", pool.get_v3f(0, 0, 1))
	local c_size = math.min(spell_button.size.x, spell_button.size.y)
	circle_button.size = pool.get_v2f(c_size, c_size)

	spell_button.hidden_recursive = false
	spell_button.color = pool.get_colorf(1, 1, 1, 0)
	spell_button:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1))

	--animate/pulse circle
	local circle_loop = LoopAction()
	circle_loop:addAction(ScaleAction(0.4, InterpAction.Fall, pool.get_v3f(1.25, 1.25, 1)))
	circle_loop:addAction(ScaleAction(0.4, InterpAction.Ramp, pool.get_v3f(1, 1, 1)))
	circle_button:addAction(circle_loop)

	local pointer = tutorial.createBouncingArrow(self.tutorial_text_window, "right")
	tutorial.matchAnchorAtoB(pointer, "R", spell_button, "L", pool.get_v3f(16, 0, 1)) --offset
	pointer.location = pointer.location - pool.get_v3f(window_loc.x, window_loc.y, 0)

	self.tutorial_text_window.color = pool.get_colorf(1, 1, 1, 0)
	self.tutorial_text_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1))

	circle_button.on_release = function()
		local tutorial_text_window = self.tutorial_text_window
		self.tutorial_text_window = nil
		tutorial_text_window:addAction(AlphaAction(0.5, InterpAction.Smooth, 0.0))
		tutorial_text_window:addAction(DestroyAction(0.1))
		spell_button:on_release()
	end
end

function Wave01Actor:ashHitTutorial()
	if self.tutorial_text_window ~= nil then
		print("TUTORIAL TEXT WINDOW EXISTS! YAY! :)")
		self.tutorial_text_window:addAction(AlphaAction(0.5, InterpAction.Smooth, 0.0))
		self.tutorial_text_window:addAction(LuaAction(0.0, function() self.game.paused = true end))
	else
		print("TUTORIAL TEXT WINDOW IS NIL")
		self.game.paused = true
	end

	local blackout = self.game_tutorial_window:spawn(Window())
	local bo_loc = blackout.location
	blackout.location = pool.get_v3f(bo_loc.x, bo_loc.y, bo_loc.z + 5)
	blackout.size = self.game_tutorial_window.size
	blackout.color = pool.get_colorf(0, 0, 0, 0)
	blackout:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0.5))

	self.hit_tutorial_window = tutorial.createWindow(self.game_tutorial_window, nil, blackout.location + v3f(0, 0, 1))
	adjustZOffset(self.hit_tutorial_window, 100)

	--add text to the window
	local label = self.hit_tutorial_window:spawn(Text())
	label.anchor = { Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0, -24+5, 1) }
	local format = {"Helvetica-Bold", 20.0+2}
	label.style = format[1]
	label.options = {height = format[2]}
	label.text = lstring("ash_is_hurt") --"Ash is hurt!"
	label = label:spawn(Text())
	label.anchor = { Anchor.Anchor_T, Anchor.Anchor_B, pool.get_v3f(0, 0-5, 0) }
	local format ={"Helvetica", 16.0+2, 300.0}
	label.style = format[1]
	label.options = {height = format[2], max_width = format[3], align = TextAlign.Center}
	label.text = lstring("run_left") -- "Run left to avoid attacks"

	-- spawn arrow
	local arrow = util.createButtonWithTexture(self.hit_tutorial_window, assets.textures.go_arrow, false)
	arrow.color = pool.get_colorf(1, 1, 1, 0)
	arrow:addAction(AlphaAction(0.5, InterpAction.Smooth, 1))
	tutorial.matchAnchorAtoLocalB(arrow, "R", self.hit_tutorial_window, "L", pool.get_v3f(0, 0, 1))

	local loop = LoopAction()
	local sequence = SequenceAction()
	sequence:addAction(AlphaAction(0.7, InterpAction.Smooth, 0.3))
	sequence:addAction(AlphaAction(0.7, InterpAction.Smooth, 1.0))
	loop:addAction(sequence)
	arrow:addAction(loop)

	self.hit_tutorial_window.color = pool.get_colorf(1, 1, 1, 0)
	self.hit_tutorial_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1))

	local dismiss_window = function()
		blackout:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		blackout:addAction(DestroyAction(0))

		local hit_tutorial_window = self.hit_tutorial_window
		self.hit_tutorial_window = nil

		hit_tutorial_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		hit_tutorial_window:addAction(DestroyAction(0))

		self.game.paused = false

		if self.tutorial_text_window ~= nil then
			self.tutorial_text_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1.0))
		end
	end

	arrow.on_enter = dismiss_window
end

function Wave01Actor:gameOver(team)
	--if team == GameObject.TEAM_ALLY then
	--	print("THE GOOD GUYS HAVE WON!")
	--elseif team == GameObject.TEAM_ENEMY then
	--	print("THE BAD GUYS HAVE WON! YOU LOST!")
	--end

	-- Clear all tutorial elements if the level ends

	if self.hit_tutorial_window then
		print("--destroying hit tutorial window")
		self.hit_tutorial_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		self.hit_tutorial_window:addAction(DestroyAction(0))
	end

	if self.tutorial_text_window then
		print("--destroying tutorial text window")
		self.tutorial_text_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		self.tutorial_text_window:addAction(DestroyAction(0))
	end

	if self.hit_tutorial_window then
		self.hit_tutorial_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		self.hit_tutorial_window:addAction(DestroyAction(0))
	end
end
