module(...,package.seeall)
-- includes --
local engine	= engine.instance
local chunks	= engine.chunks
local util		= chunks.util
local assets	= chunks.assets
local pool		= chunks.pool
local tutorial	= chunks.tutorial

Wave02Actor = chunks.class.new
{
	owner = nil,
	game = nil,
	game_window = nil,
	action_actor = nil,
	tutorial_window_action_actor = nil,
	text_box = nil,
	tutorial_window = nil,
	damage_tutorial_window = nil,
	first_kill = false,
	fade_time = 0.25,
}

local function adjustZOffset(a, o)
	a.location = a.location + pool.get_v3f(0, 0, o)
end

function bindToObject(actor)
	Wave02Actor:new(actor) -- bind the script to the object
end

function Wave02Actor:created( o )
	self.owner = o
	self.owner.label = "Wave actor"
	self.game = self.owner:getGame()
	self.game_window = self.game.lua.m_game_window
	self.action_actor = self.game_window:spawn(ActorBase())
	self.action_actor.hidden = true
	self.tutorial_window = self.game.lua.m_tutorial_window


	tutorial.watchForAshDamage(self.owner, 0.5, function() self:ashHitTutorial() end)

	self:tutorialPart1()
end

function Wave02Actor:unitSpawned(i, gameObjectIndex, tag)

end

function Wave02Actor:tutorialPart1()
	self.text_box = tutorial.createWindow( self.game_window, "bottom-right" )
	adjustZOffset(self.text_box, 100)

	--add text to the window
	local label = self.text_box:spawn(Text())
	label.anchor = { Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0, -24+5, 1) }
	local format = {"Helvetica-Bold", 20.0+2}
	label.style = format[1]
	label.options = {height = format[2]}
	label.text = lstring("smithy_makes_iron") --"Smithy makes iron"
	label = label:spawn(Text())
	label.anchor = { Anchor.Anchor_T, Anchor.Anchor_B, pool.get_v3f(0, 0-5, 0) }
	local format = {"Helvetica", 16.0+2}
	label.style = format[1]
	label.options = {height = format[2]}
	label.text = lstring("to_summon_allies") --"to summon allies"

	-- spawn bouncing arrow
	local smithy = self.game_window.lua.m_smithy_button
	local pointer_window = smithy:spawn(Window())
	pointer_window.anchor = {Anchor.Anchor_L, Anchor.Anchor_R, pool.get_v3f(-16, 0, 101)}
	pointer_window.size = assets.textures.tutorial_pointer.size
	pointer_window.hidden = true
	local pointer = tutorial.createBouncingArrow(pointer_window, "left")

	self.text_box.color = pool.get_colorf(1, 1, 1, 0)
	self.text_box:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1))

	local t_win = self.text_box
	local prep_tutorial_2 = function()
		local text_box = self.text_box
		self.text_box = nil
		pointer_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		pointer_window:addAction(DestroyAction(0))
		text_box:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		text_box:addAction(DestroyAction(0))
		self.owner:addAction(LuaAction(self.fade_time + 0.5, function() self:tutorialPart2() end))
	end

	tutorial.watchForIronCountOver(self.owner, self.game, 4.5, prep_tutorial_2)
end

function Wave02Actor:tutorialPart2()
	self.text_box = tutorial.createWindow( self.game_window, "bottom-right" )
	self.text_box.color = pool.get_colorf(1, 1, 1, 0)
	adjustZOffset(self.text_box, 100)

	--add text to the window
	local label = self.text_box:spawn(Text())
	label.anchor = { Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0, -24+5, 1) }
	local format = {"Helvetica-Bold", 20+2}
	label.style = format[1]
	label.options = {height = format[2]}
	label.text = lstring("tap_peasant") --"Tap the peasant"
	label = label:spawn(Text())
	label.anchor = { Anchor.Anchor_T, Anchor.Anchor_B, pool.get_v3f(0, 0-5, 0) }
	local format = {"Helvetica", 16.0+2}
	label.style = format[1]
	label.options = {height = format[2]}
	label.text = lstring("to_summon_him") --"to summon him"

	local peasant = self.game_window.lua.m_spawn_buttons[1]
	local pointer_window = peasant:spawn(Window())
	pointer_window.anchor = {Anchor.Anchor_B, Anchor.Anchor_T, pool.get_v3f(0, 0, 101)}
	pointer_window.size = assets.textures.tutorial_pointer.size
	pointer_window.hidden = true
	local pointer = tutorial.createBouncingArrow(pointer_window, "down")

	--draw a circle around the peasant icon
	local circle_button = util.createButtonWithTexture(self.text_box, assets.textures.tutorial_circle)
	circle_button.size = assets.textures.icon_peasant.size
	tutorial.matchAnchorAtoParentB(circle_button, "BL", self.game_window, "BL", pool.get_v3f(0, 0, 1))

	--animate circle
	local circle_loop = LoopAction()
	circle_loop:addAction(ScaleAction(0.4, InterpAction.Fall, pool.get_v3f(1.25, 1.25, 1)))
	circle_loop:addAction(ScaleAction(0.4, InterpAction.Ramp, pool.get_v3f(1, 1, 1)))
	circle_button:addAction(circle_loop)

	self.text_box.color = pool.get_colorf(1, 1, 1, 0)
	self.text_box:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1))

	circle_button.on_release = function()
		local text_box = self.text_box
		self.text_box = nil
		text_box:addAction(AlphaAction(self.fade_time, InterpAction.Smooth, 0.0))
		text_box:addAction(DestroyAction(0))
		pointer_window:addAction(AlphaAction(self.fade_time, InterpAction.Smooth, 0.0))
		pointer_window:addAction(DestroyAction(0))
		peasant:on_release()
	end
end

function Wave02Actor:ashHitTutorial()
	if self.text_box ~= nil then
		self.text_box:addAction(AlphaAction(0.5, InterpAction.Smooth, 0.0))
	end

	-- print("ASH HIT TUTORIAL")
	self.damage_tutorial_window = tutorial.createWindow(self.game_window, "top")
	adjustZOffset(self.damage_tutorial_window, 100)

	-- add text
	local label = self.damage_tutorial_window:spawn(Text())
	label.anchor = { Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0, -24+5, 1) }
	local format = {"Helvetica-Bold", 20.0+2}
	label.style = format[1]
	label.options = {height = format[2]}
	label.text = lstring("ash_is_hurt") --"Ash is hurt!"
	label = label:spawn(Text())
	label.anchor = { Anchor.Anchor_T, Anchor.Anchor_B, pool.get_v3f(0, 0-5, 0) }
	local format = {"Helvetica", 16.0+2}
	label.style = format[1]
	label.options = {height = format[2]}
	label.text = lstring("you_lose_if_fall") --"You lose if he falls"

	-- add arrows to Ash's health bar
	local magnitude = 0.5
	local speed = 0.2
	local pause = 0.3
	local delay = 0.1
	local arrow1 = tutorial.createBouncingArrow(self.action_actor, "up", magnitude, speed, pause, delay)
	local arrow2 = tutorial.createBouncingArrow(self.action_actor, "up", magnitude, speed, pause, delay*2)
	local arrow3 = tutorial.createBouncingArrow(self.action_actor, "up", magnitude, speed, pause, delay*3)

	arrow1.scale = arrow1.scale * pool.get_v3f(0.3, 0.3, 1)
	arrow2.scale = arrow2.scale * pool.get_v3f(0.3, 0.3, 1)
	arrow3.scale = arrow3.scale * pool.get_v3f(0.3, 0.3, 1)

--	adjustZOffset(arrow1, 200)
--	adjustZOffset(arrow2, 200)
--	adjustZOffset(arrow3, 200)

	tutorial.matchAnchorAtoB(arrow1, "TL", self.game_window, "TL", pool.get_v3f(48, -36, 101))
	tutorial.matchAnchorAtoB(arrow2, "L", arrow1, "R", pool.get_v3f(-7, 0, 0))
	tutorial.matchAnchorAtoB(arrow3, "L", arrow2, "R", pool.get_v3f(-7, 0, 0))

	self.damage_tutorial_window.color = pool.get_colorf(1, 1, 1, 0)
	self.damage_tutorial_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1))

	self.damage_tutorial_window:addAction( LuaAction(5, function()
		local damage_tutorial_window = self.damage_tutorial_window
		self.damage_tutorial_window = nil
		damage_tutorial_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		damage_tutorial_window:addAction(DestroyAction(0))

		arrow1:clearActions()
		arrow1:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		arrow1:addAction(DestroyAction(0))
		arrow2:clearActions()
		arrow2:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		arrow2:addAction(DestroyAction(0))
		arrow3:clearActions()
		arrow3:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		arrow3:addAction(DestroyAction(0))

		if self.text_box ~= nil then
			self.text_box:addAction(AlphaAction(self.fade_time, InterpAction.Smooth, 1))
		end
	end) )
end

function Wave02Actor:unitDied(game_object_index)
end

function Wave02Actor:gameOver(team)
	if self.action_actor then
		print("--clearing tutorial actions")
		self.action_actor:clearActions()
		self.action_actor:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
	end

	if self.tutorial_window_action_actor then
		print("--clearing tutorial window actions")
		self.tutorial_window_action_actor:clearActions()
		self.tutorial_window_action_actor:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
	end

	if self.hit_tutorial_window then
		print("--destroying hit tutorial window")
		self.hit_tutorial_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		self.hit_tutorial_window:addAction(DestroyAction(0))
	end

	if self.damage_tutorial_window then
		print("--destroying hit tutorial window")
		self.damage_tutorial_window:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		self.damage_tutorial_window:addAction(DestroyAction(0))
	end

	if self.text_box then
		print("--destroying tutorial text window")
		self.text_box:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
		self.text_box:addAction(DestroyAction(0))
	end

end
