module(...,package.seeall)
-- includes --
local engine	= engine.instance
local chunks	= engine.chunks
local util		= chunks.util
local assets	= chunks.assets
local pool		= chunks.pool
local tutorial	= chunks.tutorial

Wave03Actor = chunks.class.new
{
	owner = nil,
	game = nil,
	game_window = nil,
	action_actor = nil,
	tutorial_window_action_actor = nil,
	tutorial_window = nil,
	fade_time = 0.25,
}

function bindToObject(actor)
	Wave03Actor:new(actor) -- bind the script to the object
end

function Wave03Actor:created( o )
	self.owner = o
	self.owner.label = "Wave actor"
	self.game = self.owner:getGame()
	self.game_window = self.game.lua.m_game_window
	self.action_actor = self.game_window:spawn(ActorBase())
	self.action_actor.hidden = true
	self.tutorial_window = self.game.lua.m_tutorial_window

	self:tutorialPart1()
end

function Wave03Actor:tutorialPart1()
	--Iron
	local iron_bar
	for i = 415, 445, 15 do
		local ib = util.spawnCoin(self.game, "iron_bar", pool.get_v3f(i, 0, 0), 1)
		if i == 430 then
			iron_bar = ib:get()
		end
	end

	self.owner:addAction(LuaAction(1.25, function()
		local text_box = tutorial.createWindow(self.game_window, "top")
		text_box.location = text_box.location + pool.get_v3f(0, 0, 100)
		text_box.color = pool.get_colorf(1, 1, 1, 0)
		text_box:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1))

		local label = text_box:spawn(Text())
		label.anchor = { Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0, -24+5, 1) }
		local format = {"Helvetica-Bold", 20+2}
		label.style = format[1]
		label.options = {height = format[2]}
		label.text = lstring("grab_iron") --"Grab iron bars"
		label = label:spawn(Text())
		label.anchor = { Anchor.Anchor_T, Anchor.Anchor_B, pool.get_v3f(0, 0-5, 0) }
		local format = {"Helvetica", 16.0+2}
		label.style = format[1]
		label.options = {height = format[2]}
		label.text = lstring("for_smithy") --"for your smithy"

		local circle_button = iron_bar:spawn(SpriteActor())
		circle_button.texture = "?://" .. assets.textures.tutorial_circle.path
		--circle_button.anchor = pool.get_v3f(0, 0, 1)
		circle_button.size = assets.textures.tutorial_circle.size
		circle_button.color = pool.get_colorf(1, 1, 1, 0)
		circle_button:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1))

		tutorial.watchForAshGreaterThanLocation(self.owner, 400, function()
			text_box:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
			text_box:addAction(DestroyAction(0))
--			circle_button:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
--			circle_button:addAction(DestroyAction(0))
		end)
	end))

	--Gold
	local gold_coin
	for i = 965, 995, 15 do
		local gc = util.spawnCoin(self.game, "gold_coin", pool.get_v3f(i, 0, 0), 5)
		if i == 980 then
			gold_coin = gc:get()
		end
	end

	tutorial.watchForAshGreaterThanLocation(self.owner, 750, function()
		local text_box = tutorial.createWindow(self.game_window, "top")
		text_box.location = text_box.location + pool.get_v3f(0, 0, 100)
		text_box.color = pool.get_colorf(1, 1, 1, 0)
		text_box:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1))

		local label = text_box:spawn(Text())
		label.anchor = { Anchor.Anchor_T, Anchor.Anchor_T, pool.get_v3f(0, -24+5, 1) }
		local format = {"Helvetica-Bold", 20+2}
		label.style = format[1]
		label.options = {height = format[2]}
		label.text = lstring("grab_coins") --"Grab gold coins"
		label = label:spawn(Text())
		label.anchor = { Anchor.Anchor_T, Anchor.Anchor_B, pool.get_v3f(0, 0-5, 0) }
		local format = {"Helvetica", 16.0+2}
		label.style = format[1]
		label.options = {height = format[2]}
		label.text = lstring("for_upgrades") --"for upgrades"

		local circle_button = gold_coin:spawn(SpriteActor())
		circle_button.texture = "?://" .. assets.textures.tutorial_circle.path
		--circle_button.anchor = pool.get_v3f(0, 0, 1)
		circle_button.size = assets.textures.tutorial_circle.size
		circle_button.color = pool.get_colorf(1, 1, 1, 0)
		circle_button:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 1))

		tutorial.watchForAshGreaterThanLocation(self.owner, 950, function()
			text_box:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
			text_box:addAction(DestroyAction(0))
--			circle_button:addAction(AlphaAction(self.fade_time, InterpAction.Lerp, 0))
--			circle_button:addAction(DestroyAction(0))
		end)
	end)
end

function Wave03Actor:unitSpawned(i, game_object_index, tag)
end

function Wave03Actor:unitDied(game_object_index)
end

function Wave03Actor:gameOver(team)
end
