module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local util      = chunks.util
local assets    = chunks.assets
local load      = chunks.load
local achievements = chunks.achievements
local globals	= chunks.globals

local playRootSound = assets.playSound
local stat_string = lstring("stat")

LuaWeaponCell = chunks.class.new
{
	m_name = nil,
	m_icon = nil,
	m_level_number_label = nil,
	m_attack_icon = nil,
	m_attack_label = nil,
	m_health_label = nil,
	m_upgrade_button = nil,
	m_cost_label = nil,
	m_gold_icon = nil,
}

-- The spawner.
spawn = chunks.class.spawner( Button, LuaWeaponCell )

function LuaWeaponCell:created( owner )

	assets.setTexture(owner, assets.textures.upgrade_box)

	owner.on_enter = function(owner) owner.color = pool.get_colorf(0.5, 0.5, 0.5, 1.0) end
	owner.on_exit  = function(owner) owner.color = pool.get_colorf(1.0, 1.0, 1.0, 1.0) end


	--local cell = util.createWindowWithTexture( parent, )

end


function LuaWeaponCell:setWeapon(name)
	local owner = self.owner
	self.m_name = name

	local function purchase()
		local user_data_instance = UserData.instance()
		local upgrade_cost = load.getStat("weapons", name, "cost")
		local gold = user_data_instance:getGold()
		if gold >= upgrade_cost then
			user_data_instance:setLocked(self.m_name, false)
			user_data_instance:setGold( gold - upgrade_cost)
			self:updateText()
			self.m_main_window.lua:updateText()
			self.m_main_window.lua.m_ash_cell.lua:updateText()
			playRootSound(assets.sounds.upgrade)
			achievements.typeUpgraded("weapons", self.m_name)
			achievements.goldChanged()
		else
			playRootSound(assets.sounds.error)
		end
	end

	local function unpurchase()
		local user_data_instance = UserData.instance()
		local upgrade_cost = load.getStat("weapons", name, "cost") * globals.downgrade_gold_return
		local gold = user_data_instance:getGold()
		if gold >= upgrade_cost then
			user_data_instance:setLocked(self.m_name, true)
			user_data_instance:setGold( gold + math.floor(upgrade_cost))
			self:updateText()
			self.m_main_window.lua:updateText()
			self.m_main_window.lua.m_ash_cell.lua:updateText()
			playRootSound(assets.sounds.upgrade)
			achievements.typeUpgraded("weapons", self.m_name)
			achievements.goldChanged()
		else
			playRootSound(assets.sounds.error)
		end
	end

	owner.on_release = function(owner)
		assets.playSound(assets.sounds.button_click_wav)
		local info_window = self.m_main_window:spawn( chunks.infowindow.spawn() )
		info_window.lua:setNameAndChunkName(self.m_name, "weapons", purchase, unpurchase, true)
		end

	--create Icon
	do
		local unlocked = not UserData.instance():getLocked(name)

		local icon_asset = assets.findTextureByPathName( chunks.load.getStat("weapons", name, "icon_texture") ) or assets.textures.icon_generic
		if not unlocked then
			-- Weapons are purchasable so don't show their locked icon --icon_asset = assets.textures.icon_locked
		end

		local icon = util.createWindowWithTexture(owner, icon_asset)
		icon.anchor = {Anchor.Anchor_L, Anchor.Anchor_L, pool.get_v3f(7.0, 0.0, 1.0)}
		self.m_icon = icon

		-- create Title
		do
			local title_label = icon:spawn(Text())
			local format = {"Georgia Bold", 20.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0), 150.0}
			title_label.style = format[1]
			title_label.options = {height = format[2], max_width = format[4]}
			title_label.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TR, pool.get_v3f(45.0, -5.0+5, 1.0)}
			title_label.text = lstring(name .. "_title") -- load.getStat("weapons", name, "title")
		end

		-- create Level Label
		do
			local level_label = icon:spawn(Text())
			local format = {"Helvetica-Bold", 20.0, 120.0}
			--format.width_constraint = 150
			level_label.style = format[1]
			level_label.options = {height = format[2], max_width = format[3]}
			--level_label.format = format
			level_label.anchor = {Anchor.Anchor_L, Anchor.Anchor_BR, pool.get_v3f(45.0, 16.0+5, 1.0)}
			level_label.text = lstring(name .. "_cell_description") --tostring( load.getStat("weapons", name, "cell_description") )
		end

		--create Attack Icon
		do
			local icon_asset = assets.textures.attack_icon
			if name == "chainsaw" then
				icon_asset = assets.textures.chainsaw_icon
			elseif name == "iron_fist" then
				icon_asset = assets.textures.iron_fist_icon
			end

			local attack = util.createWindowWithTexture(icon, icon_asset)
			attack.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TR, pool.get_v3f(-2, -7.5, 1.0)}
			attack.hidden_recursive = true
			self.m_attack_icon = attack
		end

		-- create Attack Label
		do
			local attack_label = icon:spawn(Text())
			local format = {"Helvetica-Bold", 13.0, 150.0}
			attack_label.style = format[1]
			attack_label.options = {height = format[2], max_width = format[3]}
			attack_label.anchor = {Anchor.Anchor_TL, Anchor.Anchor_TR, pool.get_v3f(12, -7.5+3, 1.0)}
			--attack_label.text = "LEVEL " .. UserData.instance():getTypeLevel(name)
			self.m_attack_label = attack_label
		end

		if unlocked then

			--[[ create Level Label
			do
				local level_label = icon:spawn(Label())
				level_label.format = TextFormat("font://Georgia Bold", 18.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0))
				level_label.anchor = {Anchor.Anchor_BL, Anchor.Anchor_BR, pool.get_v3f(45.0, 5.0, 1.0)}
				level_label.text = "LEVEL"


				-- create Level Number Label
				do
					local level_number_label = level_label:spawn(Label())
					level_number_label.format = TextFormat("font://Helvetica-Bold", 30.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0))
					level_number_label.anchor = {Anchor.Anchor_BL, Anchor.Anchor_BR, pool.get_v3f(7.0, -2.0, 1.0)}
					--level_number_label.text = tostring( UserData.instance():getTypeLevel(name) )
					self.m_level_number_label = level_number_label
				end
			end
			]]

			--[[ create Iron Cost Label
			do
				local iron_label = icon:spawn(Label())
				iron_label.format = TextFormat("font://Helvetica-Bold", 14.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0))
				iron_label.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(13,-19,1)}
				iron_label.text = tostring( load.getStat("weapons", name, "iron_cost") )
			end
			]]

			--[[create Health Icon
			do
				local health = util.createWindowWithTexture(icon, assets.textures.health_icon)
				health.anchor = {Anchor.Anchor_BL, Anchor.Anchor_BR, pool.get_v3f(1.0, 7.5, 1.0)}
			end

			-- create Health Label
			do
				local health_label = icon:spawn(Label())
				health_label.format = TextFormat("font://Helvetica-Bold", 12.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0))
				health_label.anchor = {Anchor.Anchor_BL, Anchor.Anchor_BR, pool.get_v3f(15, 7.5, 1.0)}
				--health_label.text = "LEVEL " .. UserData.instance():getTypeLevel(name)
				self.m_health_label = health_label
			end
			]]

		else -- if unlocked

			--create Upgrade Button

			local upgrade_button = util.createButtonWithTexture(owner, assets.textures.upgrade_button_blank)
			upgrade_button.anchor = {Anchor.Anchor_R, Anchor.Anchor_R, pool.get_v3f(-7.0, 0, 1.0)}
			self.m_upgrade_button = upgrade_button

			upgrade_button.on_release = function(owner)
				assets.playSound(assets.sounds.button_click_wav)
				local info_window = self.m_main_window:spawn( chunks.infowindow.spawn() )
				info_window.lua:setNameAndChunkName(self.m_name, "weapons", purchase, unpurchase, true)
			end

			--create Next label
			do
				local next_label = upgrade_button:spawn(Text())
				local format = {"Helvetica-Bold", 18.0, 50.0, TextAlign.Center}
				next_label.style = format[1]
				next_label.options = {height = format[2], max_width = format[3], align = format[4]}
				next_label.anchor = {Anchor.Anchor_C, Anchor.Anchor_C, pool.get_v3f(0.0, 7.5+5, 1.0)}
				next_label.text = lstring("BUY")
			end

			--Create cost window
			do
				local cost_window = upgrade_button:spawn(Window())
				cost_window.anchor = {Anchor.Anchor_B, Anchor.Anchor_B, pool.get_v3f(0-1, 10, 1)}
				cost_window.hidden = true

				--create Gold Icon
				local gold_icon = util.createWindowWithTexture(cost_window, assets.textures.upgrade_gold_coin)
				gold_icon.scale = pool.get_v3f(0.5, 0.5, 1)
				gold_icon.anchor = {Anchor.Anchor_L, Anchor.Anchor_L, pool.get_v3f(0, 0, 1)}
				self.m_gold_icon = gold_icon

				--create Cost Label
				local cost_label = cost_window:spawn(Text())
				local format = {"Helvetica-Bold", 18.0}
				cost_label.style = format[1]
				cost_label.options = {height = format[2]}
				cost_label.anchor = {Anchor.Anchor_R, Anchor.Anchor_R, pool.get_v3f(0, 0+4, 1)}
				self.m_cost_label = cost_label
			end







			--[[
			local unlocked_after = owner:spawn(Label())
			unlocked_after.format = TextFormat("font://Georgia Bold", 16.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0))
			unlocked_after.anchor = {Anchor.Anchor_BL, Anchor.Anchor_BL, pool.get_v3f(109, 8, 1.0)}
			unlocked_after.text = "Unlocked At Wave " .. (unlock_wave)

			local wave_number = owner:spawn(Label())
			wave_number.format = TextFormat("font://Georgia Bold", 16.0, pool.get_colorf(1.0, 1.0, 1.0, 1.0))
			wave_number.anchor = {Anchor.Anchor_BR, Anchor.Anchor_BR, pool.get_v3f(27, -10, 1.0)}
			wave_number.text = "Wave " .. unlock_wave
			]]
		end

	end

	self:updateText()

end

function LuaWeaponCell:updateText()
	local name = self.m_name

	if self.m_upgrade_button and self.m_cost_label then
		stat_string:arg(1, tostring( load.getStat("weapons", name, "cost") ))
		self.m_cost_label.text = stat_string
		local cost_window = self.m_cost_label:getOwner()
		local c = 38
		if self.m_cost_label.size.x > c then
			self.m_cost_label.scale = pool.get_v3f(c / self.m_cost_label.size.x, c / self.m_cost_label.size.x, 1)
		end
		cost_window.size = pool.get_v2f(self.m_gold_icon.size.x * self.m_gold_icon.scale.x + self.m_cost_label.size.x * self.m_cost_label.scale.x, math.max(self.m_gold_icon.size.y * self.m_gold_icon.scale.y, self.m_cost_label.size.y * self.m_cost_label.scale.y))
	end

	if UserData.instance():getLocked(name) then
		return
	end



	--local level = UserData.instance():getTypeLevel(name)

	--self.m_level_number_label.text = tostring( level )

	local ash_level = UserData.instance():getTypeLevel("ash")
	local damage = load.getStat("weapons", name, "damage", ash_level)

	self.m_attack_icon.hidden_recursive = false
	stat_string:arg(1, tostring( damage ))
	self.m_attack_label.text = stat_string

	--self.m_health_label.text = tostring( load.getStat("weapons", name, "health") )


	if self.m_upgrade_button then
		self.m_upgrade_button.hidden_recursive = true
		self.m_upgrade_button.receive_input = false
	end

	local icon_asset = assets.findTextureByPathName( chunks.load.getStat("weapons", name, "icon_texture") ) or assets.textures.icon_generic
	assets.setTexture(self.m_icon, icon_asset)





end
