module(...,package.seeall)
-- includes --
local engine    = engine.instance
local chunks    = engine.chunks
local pool      = chunks.pool
local util      = chunks.util
local load      = chunks.load

LuaWiseman = chunks.class.inherit(chunks.baseunit.LuaBaseUnit)
{
	m_last_attack_object = nil,
	m_time_since_last_heal = 0,
	m_range = 0,
	m_reload_delay = 0,
	m_weapon = nil,
	m_previous_patients = nil,
}

bindToObject = chunks.class.gameObjectBinder(LuaWiseman)

function LuaWiseman:onBindToObject(object)	
	LuaWiseman.super.onBindToObject(self, object)	
	local actual_object = self.owner
	
	self.m_previous_patients = {}
	
	local inventory = PartInventoryIndex(object):get()
	local weapon = PartWeaponIndex( inventory:getEquippedItemAtSlot(PartInventory.WEAPON_EQUIP_SLOT):get():getObjectIndex() ):get()
	self.m_range = weapon.range
	self.m_reload_delay = weapon.reload_delay
	--print(self.m_range, self.m_reload_delay)
	self.m_weapon = weapon
end

-- Callbacks from C++
-- Callbacks from C++
function LuaWiseman:processPartLogic(part, dt)
	self.m_has_ever_been_unblocked = true
	self:process(part, dt)
end

function LuaWiseman:blockedPartLogic(part, dt)
	if self.m_has_ever_been_unblocked then
		self:process(part, dt)
	else
        part:get():requestProcess(0.2)
    end
end


function LuaWiseman:process(part, dt)	
	part:get():requestProcess(0.2)
	
	self.m_time_since_last_heal = self.m_time_since_last_heal + dt
	
	local object_index = part:get():getObjectIndex()
	local object = object_index:get()
	local orders = PartOrdersIndex(object_index):get()
	
	
	local reload_delay = self.m_reload_delay
	local range = self.m_range
	
	if self.m_time_since_last_heal >= reload_delay and orders:isMoving() then
		local patients = object:getMainGame():getObjectsInRange(object.team, object.location.x-range, object.location.x+range).values
		for i,enemy_index in ipairs(patients) do
			local enemy = enemy_index:get()
			local enemy_health = PartHealthIndex(enemy_index):get()
			local enemy_movement = PartMovementIndex(enemy_index):get()
			if enemy and enemy_movement and enemy_health and not self.m_previous_patients[enemy.id] then
				self.m_previous_patients[enemy.id] = true
				self.m_time_since_last_heal = 0
				
				self.m_weapon:fire(enemy_index, true)
				--print("MAGIC HEAL SHOT AT "..enemy.label)
				break;
			end --if enemy, enemy_health, previous_patients
			
		end --for loop
	end

	if orders:isAttacking() and not orders:isMoving() then
		self.m_previous_patients = {}
		--part:get():requestProcess(0.2)
	end
	
	
	if orders:hasOrders() then
		--part:get():requestProcess(0.2)
		return
	end
	
	
	local otherside = Map.get():getFirstPoint(object:getOppositeTeam()).x
	local attack_object_index = object:getMainGame():getClosestObjectToAttack(object.team, otherside)
	if attack_object_index:get() then
		orders:addOrder(AttackOrder(attack_object_index), PartOrders.QUEUE_END)
		--part:get():requestProcess(0.2)
	else
		--part:get():requestProcess(0.2)
	end

end

function LuaWiseman:destroy(owner)
	local cooldown_button = owner:getGame().lua.m_buttons_unique[owner.label]
	if cooldown_button then
		owner:getGame().lua.m_buttons_unique[owner.label] = nil
		cooldown_button:makeReady()
	end
end

function LuaWiseman:onKnockedBack(distance)
    local owner = self.owner
    local game = owner:getMainGame()
    local pit = game.lua.m_pit
    if pit and pit:get() and owner.team == GameObject.TEAM_ENEMY then
        pit:get().lua:throwIntoPit(owner)
    end
end

function LuaWiseman:onDeath(owner)
	LuaWiseman.super.onDeath(self, owner)

	if owner.team == GameObject.TEAM_ENEMY then
		util.tryToSpawnCoin(owner)
		
		local bonus = load.getStat( "units", owner.label, "ash_kill_bonus", owner:getLevel() ) or 0
		local bonus = math.floor( bonus * 0.1 + 0.5 )
		self.owner:getMainGame().lua:addGold(bonus)
		--print("AWARDED ", bonus)
	end
end

function LuaWiseman:getCustomTargeting()
	local owner = self.owner

	if owner.team == GameObject.TEAM_ALLY then
		
        local percentage = 0.50
        percentage = (1-chunks.globals.ash_health_bar_show_percentage)*(1-percentage) + percentage
        --print("percentage=", percentage)

        local ash_index = owner:getMainGame().lua.m_ash
		local ash = ash_index:get()
		if ash then
			if math.abs(ash.location.x - owner.location.x) <= self.m_range then
				local health = PartHealthIndex(ash_index):get()
				if health then
					local total_health = health.health
					local health_remaining = health:getHealthRemaining()
					if health_remaining / total_health < percentage then
                        --print("WISEMAN CHOSING TO HEAL ASH")
                        return ash_index
					end	--if less than 50%
				end --if has health part
			end --if in range
		end --if ash
		
	end --if TEAM_ALLY
	
	return GameObjectIndex()
end